class System:
    """The base class describing the system used in RDF calculation"""    
    def __init__(self,filename,filetype):
        self.NMType=0
        self.NMolMType=[]
        self.MType=[]
        self._input_file_name=filename
        self._input_file_type=filetype
        self.AType=[]
        self.NAType=0
        self.box=[]
        self.RDFs=[]
        self.ADFs=[]
        self.RDFOutputFile=""
        self.AtomByID=[]
        self._AtomsSortedByType_=[]
        self._try_file()                
        if self._input_file_type=='old':
            self.setup_from_file_input_MagiC1()
        elif self._input_file_type=='new':
            self.setup_from_file_input_MagiC2()
        else:
            print 'The format of input file is not stated - Detect automatically'            
     #   self._SortAtomsByType_()        
    def _CleanAndUp_(self,s):
        s=s.replace('\'','').replace('\"','').strip().strip(',')
        if s.startswith('&'): s=s.upper()
        if '=' in s:
            ss=s.split('=')
            ss[0]=ss[0].upper()
            s='='.join(ss)
        return s                
    def _try_file(self):
        from sys import exit
        try:            
            ifile=open(self._input_file_name,'r')            
            lines=ifile.readlines()
            ifile.close()            
            if self._input_file_type==None:                            
                print 'The format of input file is not stated - Detect automatically'
                lines=[self._CleanAndUp_(l) for l in lines]
                lines=[l for l in lines if not l.startswith(('!','#'))]
                lines=[l for l in lines if l != '']
                if lines[0]=='&TRAJ':
                    print 'Old file format detected'
                    self._input_file_type='old'
                elif '&PARAMETERS' in lines: 
                    print 'New file format detected'
                    self._input_file_type='new'
                else: 
                    print 'Can not detect file format. Please specify it manually in command line: rdf.py -i input.file -t new '
                    exit(1)            
        except:
            print 'Unable to open file:',self._input_file_name; exit(1)
        print 'file ',self._input_file_name,' succesfully opened for reading'

    def _enumerate_atoms(self):        
        i=0
        for moltype in self.MType:
           for atom in moltype.Atoms:
                atom.ID=i
                self.AtomByID.append(atom)
                i=i+1

    def _ReadProp_(self,knd,lines,old=None, must=True, default=None):    
        import re
        import sys
        query=filter(lambda x: re.search('^{0}|{1} *='.format(knd,old),x),lines)
        if len(query)>0:
            return query[0].split('=')[1].strip()
        else:
            if must:
                sys.exit('Error: No '+knd+' value provided')
            else:
                return default            

    def _CheckTags_(self,knd, lines):
        import sys
        if lines.count('&'+knd)!=0:
            if lines.count('&'+knd)>1:
                sys.exit('too many &'+knd+' tags in the file')
            if lines.count('&END'+knd)!=1:        
                sys.exit('too many or too few &END'+knd+' tags in the file')        
        else:
                sys.exit('No &'+knd+' section found in the file')              
        return lines.index('&'+knd),lines.index('&END'+knd)                

    def setup_from_file_input_MagiC2(self):
        from sys import exit
        import Trajectory
        import MolType
        import DFset
        import Atom
        import DF
        import MTException as MTE
        import glob
                
        # Read input file, remove comments and empties, capitalize keywords and tags
        ifile=open(self._input_file_name,'r')
        all_lines=ifile.readlines()
        ifile.close() 
        all_lines = map(self._CleanAndUp_, all_lines)        
        all_lines=[l for l in all_lines if not l.startswith(('!','#'))]
        all_lines=[l for l in all_lines if l != '']
        
#        Reading parameters
        #searching for lines which belong to system and trajectory descibing part 
        iTrajStart,iTrajEnd=self._CheckTags_('PARAMETERS',all_lines)
        lines=all_lines[iTrajStart:iTrajEnd]

        print 'Reading Parametrs section'

        # Analyzing Molecular Types
        MType=[]
        try:
            strNameMType=self._ReadProp_('NAMEMTYPE',lines,old='NAMOL')
            NameMType=strNameMType.replace('\'','').replace(' ','').split(',')
            for iNameMType in NameMType:
                if iNameMType.endswith(('.mcm','.mmol')):
                    MType.append(MolType.MolType(iNameMType))
                else:
                    print 'No types of the molecular file descritions were provided: detecting them automatically'
#                    ls=[l for l in glob.glob(iNameMType+'.*') if l.endswith(('.mcm','.mmol'))]
                    ls=[l for l in glob.glob(iNameMType+'.mcm')+glob.glob(iNameMType+'.mmol')]
#                    print len(ls)
#                    ls=[l.split('') for l in ls]
                    if len(ls)==1:
                        MType.append(MolType.MolType(ls[0]))
                    elif len(ls)==2:
                        print 'Two molecular description files are detected for the type {0}'.format(iNameMType)
                        print 'MMOL file will be used.'                        
                        MType.append(MolType.MolType([l for l in ls if l.endswith('.mmol')][0]))
                    else:
                        print 'No or too few molecular description files are detected for the type {0}'.format(iNameMType)
                        print ls                    
                        exit() 
            NameMType=[name.replace('.mcm','').replace('.mmol','') for name in NameMType]            
        except: 
            raise MTE.RDFReadError("Unable to read MolecularType descriptions")
        self.MType=MType
        
        #Dump all bonds which could be read from mcm-file.
        for iMType in MType:
            iMType.Bonds=[]

        # Global enumaration of atoms in Molecular types.            
        self._enumerate_atoms()        

        print 'NameMType={}'.format(NameMType)        
        self.NMType=len(self.MType)
        print 'Number of Molecular types detected={}'.format(self.NMType)
        # Number of molecules of each type: NMolMType NMType
        try:
            NMolMType=self._ReadProp_('NMOLMTYPE',lines,old='NSPEC')
            NMolMType=NMolMType.replace(' ','').split(',')
            NMolMType=[int(n) for n in NMolMType]            
            self.NMolMType=NMolMType
        except:
            raise MTE.RDFReadError("Unable to read NMolMType values from the file: got {0} instead of an integer array".format(NMolMType))            

        if (len(self.NMolMType)!=self.NMType):
            raise MTE.RDFReadError("Number of molecular types ({0}) differs from the size of NMolMType ({1})".format(self.NMType,len(self.NMolMType)))            

#----------------------------------------------------------------------------------

# Trajectory file and format
        try:
            TrajFileName=self._ReadProp_('TRAJFILE',lines, old='FNAME').replace('\'','').replace('\"','')           
        except:
            raise MTE.RDFReadError("Unable to read trajectory file name")
        print 'TrajFile={}'.format(TrajFileName)        
        try:
            TrajFileStart=int(self._ReadProp_('BEGINFILE',lines,old='NFBEG',must=False, default=1))
            TrajFileStop=int(self._ReadProp_('ENDFILE',lines,old='NFEND',must=False, default=1))
            TrajFileStep=int(self._ReadProp_('STEP',lines,old='NFBEG',must=False, default=1))            
                    # Analyze extension of the file:
            if TrajFileName.upper().endswith(('.XMOL','.TRR','.XTC')):
                TrajFileType=TrajFileName.split('.')[-1]
                TrajFileName='.'.join(TrajFileName.split('.')[0:-1])
            else:
                TrajFileType=self._ReadProp_('NFORM',lines,old='NFORM').strip("'")            
        except:      
           raise MTE.RDFReadError("Error while reading trajectory parameters NFBEG, NFEND, ISTEP, NFORM from the file")            
        print 'Trajectory File Format ={}'.format(TrajFileType)        
        # get total number of atoms in the system:        
        n=sum([self.MType[ii].NAtom*self.NMolMType[ii] for ii in xrange(0,self.NMType)])
        
#Here we initilize trajectory object
        if TrajFileType.upper()=='XMOL':
            self.Trajectory=Trajectory.XMOL_Trajectory(TrajFileName,TrajFileType,TrajFileStart,TrajFileStop,TrajFileStep,n)
        elif  TrajFileType.upper() in ('XTC','TRR'):            
            self.Trajectory=Trajectory.Gromacs_Trajectory(TrajFileName+'.'+TrajFileType,TrajFileType,TrajFileStart,TrajFileStop,TrajFileStep,n)
        else:
            raise MTE.RDFReadError("Undefined type of the trajectory file: {0} instead of a XMOL, XTC or TRR".format(TrajFileType.upper()))            

# Reading periodic box size from input file. it will be used if nothing stated in trajectory            
        try:
            box=self._ReadProp_('BOX',lines,must=False)      
            if box!=None:
                if len(box.split())==3:
                    self.box=[float(i) for i in box.split()]
                    print 'BOX={}'.format(self.box)
            else:
                print 'No periodic box size provided: It will be averaged from the trajectory.'
        except:
            raise MTE.RDFReadError("Error while reading periodic box size BOX from the file")            

# Now reading RDF related specifications form input
# Reading Output filename

        self.RDFOutputFile=self._ReadProp_('OUTPUTFILE',lines, old='FOUTRDF')
        if not(self.RDFOutputFile.endswith('.rdf')):
            self.RDFOutputFile+'.rdf'
        print "RDF will be written to file {}".format(self.RDFOutputFile)
#Reading max distance of intermolecular RDFs
        try:
            rmax=float(self._ReadProp_('RMAXNB',lines, old='RDFCUT'))
        except:
            raise MTE.RDFReadError("Unable to read RDFCUT value: got {0} instead of a real number".format(self._ReadProp_('RMAXNB',lines, old='RDFCUT')))
        print "RMaxNB={}".format(rmax)
#Reading r-resolution of intermolecular RDFs
        try:
            resol=float(self._ReadProp_('RESOLNB',lines, old='DELTAR'))            
        except:
            raise MTE.RDFReadError("Unable to read ResolNB value: got {0} instead of a real number".format(self._ReadProp_('RESOLNB',lines, old='DELTAR')))
        print "ResolNB={}".format(resol)
#Reading r-resolution of intramolecular RDFs
        try:
            resoli=float(self._ReadProp_('RESOLB',lines, old='DELTARI'))
        except:
            raise MTE.RDFReadError("Unable to read ResolB value: got {0} instead of a real number".format(self._ReadProp_('RESOLB',lines, old='DELTARI')))
        print "ResolB={}".format(resoli)
#Reading r-resolution of intramolecular RDFs
        try:
            resolphi=float(self._ReadProp_('RESOLA',lines, old='DELTAPHI',default=1.0, must=False))
        except:        
            raise MTE.RDFReadError("Unable to read ResolA value: got {0} instead of a real number".format(self._ReadProp_('RESOLA',lines, old='DELTAPHI',default=1.0, must=False)))
        print "ResolA={}".format(resolphi)
#Reading max distance of intramolecular RDFs
        try:
            rmaxi=float(self._ReadProp_('RMAXB',lines, old='RMAX'))
        except:
            raise MTE.RDFReadError("Unable to read RMaxB value: got {0} instead of a real number".format(self._ReadProp_('RMAXB',lines, old='RMAX')))
        print "RMaxB={}".format(rmaxi)
#Reading min distance of intramolecular RDFs
#        try:
#            rmini=float(self._ReadProp_('RMINB',lines, old='RMIN'))
#        except:
#            raise MTE.RDFReadError("Unable to read RMinB value: got {0} instead of a real number".format(self._ReadProp_('RMINB',lines, old='RMIN')))
#        print "RMinB={}".format(rmini)
        
            
#----------------------------------------------------------------------------------------------------------------
# Reading CGAtomType Names
        print '\n READING CG-atom types definitions'
        iATypeStart,iATypeEnd=self._CheckTags_('CGTYPES',all_lines)
        lines=all_lines[iATypeStart+1:iATypeEnd]
        
        for (i,l) in enumerate(lines):        
            if len(l.split(':'))!=2:
                raise MTE.RDFReadError("Error while reading list of CG-atom types: line {0}".format(l))
            (atomtypename,atomnames)=(l.split(':')[0].strip(),l.split(':')[1].split())            
            a=Atom.AtomType(atomtypename)
            for aname in atomnames:
                alst=[A for iMol in self.MType for A in iMol.Atoms if A.Name==aname]
                if alst==[]:
                    raise MTE.RDFReadError("Error while reading list of CG-atoms : atom {0} is not present in molecular topology files (mcm-files)".format(aname))
                else:
                    a.Atoms=a.Atoms+alst            
            a.Set_Atom_Names(atomnames)
            self.AType.append(a) 
            a.ID=len(self.AType)
        
        for iAType in self.AType:
            for iAtom in iAType.Atoms:
                iAtom.Type=iAType
# Define local number of each atom
        for MT in self.MType:
            for (i,a) in enumerate(MT.Atoms):
                a.Number=i+1                
        self.NAType=len(self.AType)
        for AT in self.AType:
            print '{0}:{1}'.format(AT.Name, [a.Name for a in AT.Atoms])
        print '\n'
#-------------------------------------------------------------------------                    
# Reading list of intermolecular RDFs
        def CleanMType(MType, line):
            if isinstance(MType, MolType.MolType): 
                if MType in self.MType:
                    return MType
                else:
                    raise MTE.RDFReadError("Can not find such MolType {2} in the system: Line {0} \n System types:{1}".format(line,[i.Name for i in self.MType]), MType.Name)            
            else:
                MType=[MT for MT in self.MType if MT.Name==MType]
                if len(MType)==0:
                    raise MTE.RDFReadError("Can not find such MolType in the system: Line {0} \n System types:{1}".format(line,[i.Name for i in self.MType]))            
                # Check and convert provided atom types to a Atom.AtomType instances
                if len(MType)>1: # Check if 2 ATypes stated
                    raise MTE.RDFReadError("Error: Two or more MolTypes with such name exist in the system! Something is wrong! \n line: {}".format(line))
                return MType[0]

        def CleanATypes(ATypes, line):
            import Atom
            # Check and convert provided atom types to a Atom.AtomType instances
            if len(ATypes)!=2: # Check if 2 ATypes stated
                raise MTE.RDFReadError("Error in Pair of AtomTypes stated in line: {}".format(line))
            if all([isinstance(at,Atom.AtomType) for at in ATypes]): # Correct type - do nothing
                pass
            elif all([isinstance(at,str) for at in ATypes]): # Strings -> Convert to AType
                ATypes=[iAT for iat in ATypes for iAT in self.AType if iat==iAT.Name]
                if len(ATypes)!=2:
                    raise MTE.RDFReadError("Can not find such AtomTypes in the system. line: {0} \n System types:{1}".format(line,[i.Name for i in self.AType]))
            else: 
                raise MTE.RDFReadError("Error in Pair of AtomTypes stated in line: {}".format(line))
            return ATypes
            
        def CleanAPairs(APairs, line, MT=None):
            import Atom
            
            if MT==None:
                MTs=self.MType
            else:
                MTs=[MT]
                
            # Check that we got a list of pairs:
            if all([len(iAPairs)!=2 for iAPairs in APairs]):
                raise MTE.RDFReadError("Error in the list of Atom pairs stated in line: {}".format(line))                
            if all([isinstance(iap,Atom.Atom) for ap in APairs for iap in ap]): # Correct type - do nothing
                pass
            elif all([isinstance(iap,str) for ap in APairs for iap in ap]): # Strings -> convert to Atom.Atom
                Atoms=[a for iMT in MTs for a in iMT.Atoms]
#                print Atoms
                APairs=[[a for i in ip for a in Atoms if a.Name==i ] for ip in APairs]
#                print APairs
                if any([len(ap)!=2 for ap in APairs]):
                    raise MTE.RDFReadError("Can not find some of the stated Atoms in the system: {0} \n System atoms:{1}".
                    format(line,[i.Name for i in Atoms]))                    

            return APairs
        def CleanATriplets(APairs, line, MT=None):
            import Atom
            
            if MT==None:
                MTs=self.MType
            else:
                MTs=[MT]

            # Check that we got a list of pairs:
            if all([len(iAPairs)!=3 for iAPairs in APairs]):
                raise MTE.RDFReadError("Error in the list of Atom triplets stated in line: {}".format(line))                
            if all([isinstance(iap,Atom.Atom) for ap in APairs for iap in ap]): # Correct type - do nothing
                pass
            elif all([isinstance(iap,str) for ap in APairs for iap in ap]): # Strings -> convert to Atom.Atom
                Atoms=[a for iMT in MTs for a in iMT.Atoms]
                APairs=[[a for i in ip for a in Atoms if a.Name==i ] for ip in APairs]
                if any([len(ap)!=3 for ap in APairs]):
                    raise MTE.RDFReadError("Can not find some of the stated Atoms in the system: {0} \n System atoms:{1}".
                    format(line,[i.Name for i in Atoms]))                    
            return APairs

        def Add_NB_APairs(ATypes,APairs,line):
            ATypes=CleanATypes(ATypes, line)
            APairs=CleanAPairs(APairs, line)
            # Here I should add check if the atoms belong to stated Types
            
            #Check if RDF with Types ATypes already exist, if not we create it
            rdflst=[rdf for rdf in self.RDFs if (rdf.Type=='NB') and (set(rdf.AtomTypes)==set([at.Name for at in ATypes]))]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one NB RDF with types {0} found: ".format([AT.Name for AT in ATypes]))
            elif len(rdflst)==0: # if we found no RDF: Create a new one
                trdf=DF.RDF_NB(Name=ATypes[0].Name+"-"+ATypes[1].Name,
                            Min=0.0,
                            Max=rmax,
                            Npoints=int(rmax/resol),
                            Kind='RDF',Type='NB',
                            _AtomGroups_=[],
                            AtomTypes=[AT.Name for AT in ATypes])
                self.RDFs.append(trdf)
            elif len(rdflst)==1:
                trdf=rdflst[0]            
            # At this point we only have a single RDF in the list, which already exist or just has been created.
            #Add pairs which are not present there (check for the order of pairs!)            
            for ap in APairs: # Here we add one pair at a time to avoid adding double records.
                if set(ap) not in [set(AP) for AP in trdf._AtomGroups_]:
                    trdf._AtomGroups_.append(ap)
#                    trdf.NGroups+=1
            print 'Added NB RDF: {0}-{1}: Atoms: {2}'.format(ATypes[0].Name, ATypes[1].Name, [ '-'.join([i.Name for i in AP]) for AP in APairs])

        def Del_NB_APairs(ATypes,APairs,line):
            ATypes=CleanATypes(ATypes, line)
            APairs=CleanAPairs(APairs, line)            
            #Check if RDF with Types ATypes already exist, if not we create it
            rdflst=[rdf for rdf in self.RDFs if (rdf.Type=='NB') and (set(rdf.AtomTypes)==set([at.Name for at in ATypes]))]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one NB RDF with types {0} found: ".format([AT.Name for AT in ATypes]))
            elif len(rdflst)==0: # if we found no RDF: Create a new one
                print 'Warning: No NB-RDF with types {0} found: Nothing to delete, no action taken.'.format([AT.Name for AT in ATypes])
            elif len(rdflst)==1:
                trdf=rdflst[0]                            
                # Find pairs in the RDF
                for ap in APairs: # 
                    aplist=[AP for AP in trdf._AtomGroups_ if set(ap)==set(AP)]
                    if len(aplist)>0:
                        for ap in aplist:
                            trdf._AtomGroups_.remove(ap)
#                            trdf.NGroups+=-1
                    else:
                        print 'Warning: No atom pair {0} found in the rdf: Nothing to delete, no action taken.'.format([ap[0].Name,ap[1].Name])
                if len(trdf._AtomGroups_)==0:
                    self.RDFs.remove(trdf)
                print 'Removed pairs from NB RDF: {0}-{1}: Atoms: {2}'.format(ATypes[0].Name, ATypes[1].Name, ['-'.join([i.Name for i in AP]) for AP in APairs])

                    
        def Add_NB_ATypes(ATypes, line):
            ATypes=CleanATypes(ATypes, line)           
            # Generate list of pairs
            lpairs=[[ia, ja] for ia in ATypes[0].Atoms for ja in ATypes[1].Atoms]
            # Now the identical record shall be removed from list            
            APairs=[]
            for p in lpairs:
                if set(p) not in [set(AP) for AP in APairs]:
                    APairs.append(p)            
            # Call ADD_NB_APair:
            print 'Add All atom pairs to NB RDF: {0}-{1}'.format(ATypes[0].Name, ATypes[1].Name, ['-'.join([i.Name for i in AP]) for AP in APairs])
            Add_NB_APairs(ATypes, APairs, line)
 
        def Del_NB_ATypes(ATypes, line):
            ATypes=CleanATypes(ATypes, line)
            rdflst=[rdf for rdf in self.RDFs if (rdf.Type=='NB') and (set(rdf.AtomTypes)==set([at.Name for at in ATypes]))]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one NB RDF with types {0} found: ".format([AT.Name for AT in ATypes]))
            elif len(rdflst)==0: # if we found no RDF: Create a new one
                print 'Warning: No NB-RDF with types {0} found: Nothing to delete, no action taken.'.format([AT.Name for AT in ATypes])
            elif len(rdflst)==1:
                trdf=rdflst[0]                            
                self.RDFs.remove(trdf)            
                print 'Removed NB RDF: {0}-{1}'.format(ATypes[0].Name, ATypes[1].Name)
                
        def Add_NB_All():
            # Generate list of pairs
            lpairs=[[ia, ja] for ia in self.AType for ja in self.AType]
            # Now the identical record shall be removed from list            
            ATPairs=[]
            for p in lpairs:
                if set(p) not in [set(AP) for AP in ATPairs]:
                    ATPairs.append(p)            
            # Call Add_NB_AType for each pair
            print 'Add All possible NB RDFs:'
            for p in ATPairs:
                Add_NB_ATypes(p,None)

# DEtecting lines
        print '\nReading NB-RDF section'
        iNBStart,iNBEnd=self._CheckTags_('RDFSNB',all_lines)
        lines=all_lines[iNBStart+1:iNBEnd]
        
        for l in lines:
            action=l.split(':')[0].strip()
            if not (action.upper() in ['ADD','DEL']):
                raise MTE.RDFReadError("Unknown command in NB-RDF definition. Line:{0}".format(l))

            if len(l.split(':'))>1:                
                if len(l.split(':'))==2: # 3 Separators - All or AtomTypes
                    if l.split(':')[1].strip().upper()=='ALL':
                        if action.upper()=='ADD': 
                            Add_NB_All()
                    else: # AtomTypes
                        ATypes=l.split(':')[1].replace(' ','').split('--')
                        if action.upper()=='ADD':
                            Add_NB_ATypes(ATypes, l)
                        elif action.upper()=='DEL':                
                            Del_NB_ATypes(ATypes, l)
                elif len(l.split(':'))==3: # 3 separators - Atom Pairs
                    ATypes=l.split(':')[1].replace(' ','').split('--')
                    APairs=[i.split() for i in l.split(':')[2].strip().split(',')]
                    if action.upper()=='ADD':
                        Add_NB_APairs(ATypes, APairs, l)                                                                        
                    elif action.upper()=='DEL':                
                        Del_NB_APairs(ATypes, APairs, l)                                                                                                
                else: # Too many separators
                    raise MTE.RDFReadError("Can't read AtomPairs in NB-RDF definition. Too many ':' separators. \nLine:{0}".format(l))
            else: # Only one separator - error
                raise MTE.RDFReadError("Can't read AtomTypes in NB-RDF definition. Line:{0}".format(l))
                                            
#Reading intramolecular Bond RDFs
        def Add_B_APairs(MType, BNumber, APairs, line):
            MType=CleanMType(MType, line)
            APairs=CleanAPairs(APairs, line, MT=MType)
            # Here I should add check if the atoms belong to stated Type
            print 'Add atom pairs to B-RDF: {0}:{1}:{2}'.format(MType.Name, BNumber, ['-'.join([i.Name for i in AP]) for AP in APairs])            
            #Check if RDF with Type and BondNumber already exist, if not we create it
            rdflst=[rdf for rdf in self.RDFs if (rdf.Type=='B' and rdf.BondNumber==BNumber and rdf.MolTypeName==MType.Name)]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one B RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            elif len(rdflst)==0: # if we found no RDF: Create a new bond and new RDF                
                bond=Atom.Bond(ID=BNumber, 
                            AtomGroups=APairs,
                            Type='B',
                            MolType=MType.Name)
                MType.Bonds.append(bond)
                trdf=DF.RDF_B(Name=APairs[0][0].Type.Name+"-"+APairs[0][1].Type.Name,
                           Min=0.0,
                           Max=rmaxi,
                           Npoints=int(rmaxi/resoli),
                           Kind='RDF',Type='B',
                           _AtomGroups_=[],
                           AtomGroups=[],
                           MolTypeName=MType.Name,
                           BondNumber=BNumber)                
                self.RDFs.append(trdf)
            elif len(rdflst)==1: # one RDF found -> update bond and RDF records                
                print 'The RDF and Bond already exist: just add extra atom pairs'
                trdf=rdflst[0]
                bond=[b for b in MType.Bonds if b.ID==BNumber][0]                
            # At this point we only have a single RDF in the list, which already exist or just has been created.
            #Add pairs which are not present there (check for the order of pairs!)            
            for ap in APairs: # Here we add one pair at a time to avoid adding double records.
                if set(ap) not in [set(AP) for AP in trdf._AtomGroups_]:
                    trdf._AtomGroups_.append(ap)
                    trdf.AtomGroups.append([i.Number for i in ap])                    


        def Add_A_ATriplets(MType, BNumber, APairs, line):
            MType=CleanMType(MType, line)
            APairs=CleanATriplets(APairs, line, MT=MType)
            print 'Add atom triplets to A-RDF: {0}:{1}:{2}'.format(MType.Name, BNumber, [['-'.join([ i.Name for i in AP])] for AP in APairs])            

            # Here I should add check if the atoms belong to stated Type
            
            #Check if RDF with Type and BondNumber already exist, if not we create it
            rdflst=[rdf for rdf in self.ADFs if (rdf.Type=='A' and rdf.BondNumber==BNumber and rdf.MolTypeName==MType.Name)]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one A RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            elif len(rdflst)==0: # if we found no RDF: Create a new bond and new RDF                
                bond=Atom.Bond(ID=BNumber, 
                            AtomGroups=APairs,
                            Type='A',
                            MolType=MType.Name)
                MType.Bonds.append(bond)
                trdf=DF.RDF_A(Name='-'.join([i.Type.Name for i in APairs[0]]),
                           Min=0.0,
                           Max=180.0,
                           Npoints=int(180.0/(resolphi)),
                           Kind='RDF',Type='A',                           
                           _AtomGroups_=[],
                           AtomGroups=[],
                           MolTypeName=MType.Name,
                           BondNumber=BNumber)                                            
                self.ADFs.append(trdf)
            elif len(rdflst)==1: # one RDF found -> update bond and RDF records                
                trdf=rdflst[0]
                bond=[b for b in MType.Bonds if b.ID==BNumber][0]                
            # At this point we only have a single RDF in the list, which already exist or just has been created.
            #Add pairs which are not present there (check for the order of pairs!)            
            for ap in APairs: # Here we add one pair at a time to avoid adding double records.
                if ap not in trdf._AtomGroups_:
                    trdf._AtomGroups_.append(ap)
                    trdf.AtomGroups.append([i.Number for i in ap])
                    
        def Add_A_MTypeAll(MType,line):
            MType=CleanMType(MType, line)
            print 'Add all possible A-RDFs in Molecule: {0}'.format(MType.Name)  
            CrossList=[]
            for A in MType.Atoms:               
                BondList=[(A, AG, B.ID ) for B in MType.Bonds if B.Type=='B' for AG in B.AtomGroups if (A in AG)]
                if len(BondList)>=2: # Make cross-list
                    CrossList=CrossList+[[A,[k for k in i[1]+j[1] if k!=A],(i[2],j[2])] for i in BondList for j in BondList if i!=j and i[2]<=j[2]]
            
            BondIDs=set([B.ID for B in MType.Bonds if B.Type=='B'])
            uBondIDPairs=[(i,j) for i in BondIDs for j in BondIDs if i<=j]
            
            iBond=len(MType.Bonds)
            for IDPair in uBondIDPairs:
                lTriplets=[[l[1][0], l[0], l[1][1]] for l in CrossList if l[2]==IDPair]
                lTriplets=[l for il,l in enumerate(lTriplets) if all([k[0]!=l[2] for ik,k in enumerate(lTriplets) if ik<il])]
                if len(lTriplets)>0:
                    iBond+=1
                    Add_A_ATriplets(MType,iBond, lTriplets,line)                    
                    print 'Added Angle-Bond:{0} {1} {2}\n'.format(MType.Name,iBond, ['-'.join([a.Name for a in t]) for t in lTriplets])

        def Add_A_All(line):            
            print 'Add all possible A-RDFs for all Molecular Types:{0}'.format([MT.Name for MT in self.MType])
            for MT in self.MType:
                Add_A_MTypeAll(MT, line)

        def Del_A_ATriplets(MType, BNumber, APairs,line):
            MType=CleanMType(MType, line)
            APairs=CleanATriplets(APairs, line)
            print 'Remove atom triplets {2} from Bond and A-RDF: {0}:{1}'.format(MType.Name, BNumber, [['-'.join([ i.Name for i in AP])] for AP in APairs])            
            
            #Check if RDF with Type and BondNumber already exist, if not we create it
            rdflst=[rdf for rdf in self.ADFs if (rdf.Type=='A' and rdf.BondNumber==BNumber and rdf.MolTypeName==MType.Name)]
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one A RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            elif len(rdflst)==0: 
                raise MTE.RDFReadError("Error: No A-RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            else:
                bonds=[B for B in MType.Bonds if B.Type=='A' and B.ID==BNumber]
                if len(bonds)!=1:
                    raise MTE.RDFReadError("Error: No bonds or more than one bond with BondNumber {1} found in MolType {0}: ".format(MType.Name,BNumber))

                bond=bonds[0]
                if len([AG for AG in bond.AtomGroups if (([AG[0],AG[1],AG[2]] in APairs) or ([AG[2],AG[1],AG[0]] in APairs))])==0:
                    raise MTE.RDFReadError("Error: No AtomTriplets{2} found in the bond {1}:{0}: ".format(MType.Name,BNumber,[[a.Name for a in AP] for AP in APairs]))
                bond.AtomGroups=[AG for AG in bond.AtomGroups if not (([AG[0],AG[1],AG[2]] in APairs) or ([AG[2],AG[1],AG[0]] in APairs))]            
                trdf=rdflst[0]
            # At this point we only have a single RDF in the list, which already exist or just has been created.
            #Add pairs which are not present there (check for the order of pairs!)            
                bond.AtomGroups=[AG for AG in bond.AtomGroups if not (([AG[0],AG[1],AG[2]] in APairs) or ([AG[2],AG[1],AG[0]] in APairs))]
                trdf._AtomGroups_=[AG for AG in trdf._AtomGroups_ if not (([AG[0],AG[1],AG[2]] in APairs) or ([AG[2],AG[1],AG[0]] in APairs))]
                trdf.AtomGroups=[[i.Number for i in AG] for AG in trdf._AtomGroups_]
                
                if len(bond.AtomGroups)==0:
                    print 'No more atom triplets left in A-Bond {0}{1} - Remove it'.format(MType.Name,BNumber)
                    MType.Bonds.remove(bond)
                if len(trdf.AtomGroups)==0:
                    print 'No more atom triplets left in A-RDF {0}{1} - Remove it'.format(MType.Name,BNumber)
                    self.ADFs.remove(trdf)
           
        def Del_A_BondNumber(MType, BNumber, line):
            MType=CleanMType(MType, line)
            print 'Remove bond and A-RDF from Moltype:{0} Bond:{1}'.format(MType.Name, BNumber)            
            rdflst=[rdf for rdf in self.ADFs if (rdf.Type=='A' and rdf.BondNumber==BNumber and rdf.MolTypeName==MType.Name)]
            
            if len(rdflst)>1: # if we found more than one RDF:
                raise MTE.RDFReadError("Error: More than one A RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            elif len(rdflst)==0: # if we found no RDF: Create a new bond and new RDF                
                raise MTE.RDFReadError("Error: No A-RDF with MolType {0} and BondNumber {1} found: ".format(MType.Name,BNumber))
            else:
                bonds=[B for B in MType.Bonds if B.Type=='A' and B.ID==BNumber]
                if len(bonds)!=1:
                    raise MTE.RDFReadError("Error: No bonds or more than one bond with BondNumber {1} found in MolType {0}: ".format(MType.Name,BNumber))
                MType.Bonds.remove(bonds[0])
                self.ADFs.remove(rdflst[0])
        def Del_A_MTypeAll(MType, line):
            MType=CleanMType(MType, line)
            print 'Remove all A-bonds and A-RDFs from MolTYpe:{0}'.format(MType.Name)
            for i in [B.ID for B in MType.Bonds if B.Type=='A']:
                Del_A_BondNumber(MType, i, line)
            
#----------------------------------------------------------------------------
        print '\nReading pairwise Bond-RDF section'
        iBStart,iBEnd=self._CheckTags_('RDFSB',all_lines)
        lines=all_lines[iBStart+1:iBEnd]
        
        for l in lines:
            action=l.split(':')[0].strip()
            if not (action.upper() in ['ADD']):
                raise MTE.RDFReadError("Unknown command in B-RDF definition. Only Add:MolType:N_Bond: A1 A2, A2 A3 are supported \nLine:{0}".format(l))

            if len(l.split(':'))==4:                
                BMType=l.split(':')[1].strip() # Name of moleular type the bond belongs to
                try:
                    BNumber=int(l.split(':')[2].strip())
                except:
                    raise MTE.RDFReadError("Can not read bond number in B-RDF definition. Line:{0}".format(l))    
                APairs=[i.split() for i in l.split(':')[3].strip().split(',')]
                Add_B_APairs(BMType, BNumber, APairs, l)                                                                        
            else: # Only one separator - error
                raise MTE.RDFReadError("Can't read B-RDF definition. Line:{0}".format(l))

        print '\nReading angle bending Bond-RDF section'
        iBStart,iBEnd=self._CheckTags_('RDFSA',all_lines)
        lines=all_lines[iBStart+1:iBEnd]
        
        for l in lines:
            splt=l.split(':')
            if len(splt)==1:
                raise MTE.RDFReadError("Unknown command in A-RDF definition. Line:{0}".format(l))                    
            else: # len >1
                action=splt[0].strip().upper()
                if not (action in ['ADD','DEL']):
                    raise MTE.RDFReadError("Unknown command in A-RDF definition. Line:{0}".format(l))
                MType=splt[1].strip()
                
                if (action)=='ADD' and MType.upper()=='ALL':
                    Add_A_All(l)
                else:
                    BNumber=splt[2].strip()            
                    if BNumber.upper()=='ALL':
                        if action=='ADD':
                            Add_A_MTypeAll(MType,l)
                        if action=='DEL':
                            Del_A_MTypeAll(MType,l)
                    else:
                        try:
                            BNumber=int(BNumber)
                        except:
                            raise MTE.RDFReadError("Can not read bond number in A-RDF definition. Line:{0}".format(l))    
                        if len(splt)==3 and action=='DEL':
                            Del_A_BondNumber(MType,BNumber,l)
                        else:    
                            if len(splt)==4:
                                ATriplets=[i.split() for i in splt[3].strip().split(',')]
                                if action=='ADD':
                                    Add_A_ATriplets(MType, BNumber, ATriplets,l)
                                if action=='DEL':
                                    Del_A_ATriplets(MType, BNumber, ATriplets,l)                    
                        
            if len(splt)>4: # Too many separators
                    raise MTE.RDFReadError("Can't read AtomPairs in A-RDF definition. Too many ':' separators. \nLine:{0}".format(l))
                    
        self.DFset=DFset.DFset(Kind='RDF', Name=None, 
                    Types=[i.Name for i in self.AType], 
                    Min=0,
                    Max=rmax,
                    Npoints=int(rmax/resol), 
                    DFs=self.RDFs+self.ADFs) 
        
        
        print '\n Reindex all the B- and A- bonds in the system, to keep consistency in case if some automatically generated bonds were deleted.'
        for MT in self.MType:           
#            print MT.Name
            DFsetBA = [iDF for iDF in self.DFset if iDF.Type!='NB']
#            print DFsetBA
            Bs=[iDF for iDF in DFsetBA if iDF.MolTypeName==MT.Name]
#            print Bs
            for B in Bs:
#                print B.Name, B.BondNumber
                B.BondNumber=Bs.index(B)+1                    
#                print B.BondNumber
            
                
        print '\nSUMMARY: Following RDFs will be generated:'
        print 'NB-RDFs:'
        for rec in [' : '.join([RDF.Name, ', '.join([' '.join([a.Name for a in AG ]) for AG in RDF._AtomGroups_])]) for RDF in self.RDFs if RDF.Type=='NB']:
            print rec
        
        print '\nB-RDFs:'
        for rec in [' : '.join([RDF.MolTypeName, str(RDF.BondNumber),', '.join([' '.join([a.Name for a in AG]) for AG in RDF._AtomGroups_])]) for RDF in self.RDFs if RDF.Type=='B']:
            print rec

        print '\nA-RDFs:'
        for rec in [' : '.join([RDF.MolTypeName, str(RDF.BondNumber),', '.join([' '.join([a.Name for a in AG]) for AG in RDF._AtomGroups_])]) for RDF in self.ADFs if RDF.Type=='A']:
            print rec

    def setup_from_file_input_MagiC1(self):
        from sys import exit
        import Trajectory
        import MolType
        import DFset
        import Atom
        import DF
        import MTException as MTE
        
        # read from old-school input file format
        ifile=open(self._input_file_name,'r')
        lines=ifile.readlines()
        ifile.close() 
        # remove all commented lines # !        
        for i in lines[:]:
            if i.lstrip().startswith(('#','!')):
                lines.remove(i)       
        ll=[]
        for i in range(0,len(lines)):
            ll.append(lines[i].rstrip('\r\n,')) #cut EOL and commas
        
            #searching for lines which belong to system and trajectory descibing part 
        iTrajStart=len(ll)+1
        for i in range(0,len(ll)):
            if '&TRAJ' in ll[i]: 
                iTrajStart=i; break
        if iTrajStart>len(ll):
            print "No trajectory-related section found in file."; exit(1)
        iTrajEnd=-1
        for i in range(iTrajStart,len(ll)):
            if '&END' in ll[i]: 
                iTrajEnd=i; break        
        if iTrajEnd<iTrajStart:
            print "End of trajectory-related section not found in file."; exit(1)
        values={}
        for i in range(iTrajStart+1,iTrajEnd):
            (a,b)=ll[i].split('=')
            a=a.lstrip()
            b=b.lstrip()
            values[a]=b
# Reading parameters from dictionary "values", one by one
        #NMType
        try:
            self.NMType=int(values['NTYPES'])
        except:
            raise MTE.RDFReadError('Unable to read NTYPES value from the file: got {0} instead of an integer'.format(values['NTYPES']))
        #NMolMType
        try:
            self.NMolMType=[int(object) for object in values['NSPEC'].split(',')]
        except:
            raise MTE.RDFReadError("Unable to read NSPEC values from the file: got {0} instead of an integer array".format(values['NSPEC']))            
        #PATHDB - path to molecular database
        try:
            pathdb=values['PATHDB'].strip("'")
        except:
#            print "Unable to read PATHDB value from file"; exit(1)
            raise MTE.RDFReadError("Unable to read PATHDB value from the file: got {0} instead of a string".format(values['PATHDB']))            
        #NAMOL - names of the molecular types
        try:
            MolTypeNames=[obj.strip("'") for obj in values['NAMOL'].split(',')]            
        except:
            raise MTE.RDFReadError("Unable to read NAMOL values from the file")  
        if len(MolTypeNames)!=self.NMType: 
            raise MTE.RDFReadError("Number of molecular types ({0}) differs from the size of molecular types list ({1})".format(self.NMType,len(MolTypeNames)))            
# Initializing array of molecular types        
        for i in range(0,self.NMType):
            a=MolType.MolType(MolTypeNames[i],'mmol',pathdb)
            self.MType.append(a)
# Global Enumerating atoms in Molecular types.            
        self._enumerate_atoms()        
#----------------------------------------------------------------------------------
# reading trajectory-related data                        
        try:
            TrajFileName=values['FNAME'].strip("'")            
        except:
            raise MTE.RDFReadError("Unable to read trajectory file name: got {0} instead of a string".format(values['FNAME']))            
        try:
            TrajFileStart=int(values['NFBEG'])
            TrajFileStop=int(values['NFEND'])
            TrajFileStep=int(values['ISTEP'])            
            TrajFileType=values['NFORM'].strip("'")
        except:
           raise MTE.RDFReadError("Unable to read trajectory parameters NFBEG, NFEND, ISTEP, NFORM from the file")            
# Reading periodic box size from input file. it will be used if nthing stated in trajectory            

        if ('BOXL' in values) and ('BOYL' in values) and ('BOZL' in values):
            self.box=[float(values['BOXL']),float(values['BOYL']),float(values['BOZL'])]
# get total number of atoms in the system:        
        n=0
        for ii in range(0, len(self.MType)):
            n=(self.MType[ii].NAtom)*self.NMolMType[ii]+n
        
#Here we initilize trajectory object
        if TrajFileType.upper()=='XMOL':
            self.Trajectory=Trajectory.XMOL_Trajectory(TrajFileName,TrajFileType,TrajFileStart,TrajFileStop,TrajFileStep,n)
        elif  TrajFileType.upper() in ('XTC','TRR'):            
            self.Trajectory=Trajectory.Gromacs_Trajectory(TrajFileName,TrajFileType,TrajFileStart,TrajFileStop,TrajFileStep,n)
        else:
            raise MTE.RDFReadError("Undefined type of the trajectory file: {0} instead of a XMOL, XTC or TRR".format(TrajFileType.upper()))            
# Now reading RDF related specifications form input
            #searching for lines which belong to rdf descibing part 
        iRDFStart=len(ll)+1
        for i in range(0,len(ll)):
            if '&RDFIN' in ll[i]: 
                iRDFStart=i; break
        if iRDFStart>len(ll):
            raise MTE.RDFReadError("No RDF-related section found.")
        iRDFEnd=-1
        for i in range(iRDFStart,len(ll)):
            if '&END' in ll[i]: 
                iRDFEnd=i; break        
        if iRDFEnd<iRDFStart:
            raise MTE.RDFReadError("End of the RDF-related section is not found")
        values={}
        for i in range(iRDFStart+1,iRDFEnd):
            (a,b)=ll[i].split('=')
            a=a.lstrip()
            b=b.lstrip()
            values[a]=b
# Reading Output filename
        self.RDFOutputFile=values['FOUTRDF'].strip('"').strip("'")
        if not(self.RDFOutputFile.endswith('.rdf')):
            self.RDFOutputFile=self.RDFOutputFile+'.rdf'
#Reading max distance of intermolecular RDFs
        try:
            rmax=float(values['RDFCUT'])
        except:          
            raise MTE.RDFReadError("Unable to read RDFCUT value: got {0} instead of a real number".format(values['RDFCUT']))
#Reading number of intermolecular RDFs
        try:
            nrdf=int(values['NRDF'])
        except:
            raise MTE.RDFReadError("Unable to read NRDF value: got {0} instead of a integer number".format(values['NRDF']))
#Reading number of intramolecular RDFs
        try:
            nrdfi=int(values['NRDFI'])
        except:
            print "Unable to read NRDFI from input file: No intramolecular bond distributions to calculate"; nrdfi=0            
#Reading number of intramolecular ADFs
        try:
            nadf=int(values['NADF'])
        except:
            print "Unable to read NADF from input file: No intramolecular angle distributions to calculate"; nadf=0      
#Reading r-resolution of intermolecular RDFs
        try:
            resol=float(values['DELTAR'])            
        except:
            raise MTE.RDFReadError("Unable to read DELTAR value: got {0} instead of a real number".format(values['DELTAR']))
#Reading r-resolution of intramolecular RDFs
        try:
            resoli=float(values['DELTARI'])            
        except:
            if nrdfi>0: 
                raise MTE.RDFReadError("Unable to read DELTARI value: got {0} instead of a real number".format(values['DELTARI']))
#Reading r-resolution of intramolecular RDFs
        try:
            resolphi=float(values['DELTAPHI'])                        
        except:
            if nadf>0: 
                    raise MTE.RDFReadError("Unable to read DELTAPHI value: got {0} instead of a real number".format(values['DELTAPHI']))
                
#Reading max distance of intramolecular RDFs
        try:
            rmaxi=float(values['RMAX'])
        except:
            if nrdfi>0:
                    raise MTE.RDFReadError("Unable to read RMAX value: got {0} instead of a real number".format(values['RMAX']))
            
#Reading number of CG atomic types in the system
        try:
            natomtypes=int(values['NATOMTYPES'])
            self.NAType=natomtypes
        except:
            raise MTE.RDFReadError("Unable to read NATOMTYPES value: got {0} instead of a integer number".format(values['NATOMTYPES']))
# Reading CGAtomType Names
        for i in (ll[iRDFEnd+1:iRDFEnd+1+natomtypes]):
            if len(i.split(':'))!=2:
                raise MTE.RDFReadError("Error while reading list of CG-atom types: line {0}".format(i))
            (atomtypename,atomnames)=(i.split(':')[0].strip(),i.split(':')[1].split())            
            # Create atom type object            
            a=Atom.AtomType(atomtypename)
            for name in atomnames:                
                atom_match=[iAtom for iMType in self.MType for iAtom in iMType.Atoms if iAtom.Name==name]
                if atom_match==[]:
                    raise MTE.RDFReadError("Error while reading list of CG-atoms : atom {0} is not present in molecular topology files (mcm-files)".format(name))
                if len(atom_match)>1:
                    raise MTE.RDFReadError("Error Warning while reading list of CG-atoms : atom {0} is present in molecular topology files more than once ".format(name))
                a.Atoms.append(atom_match[0])                
            
            a.Set_Atom_Names(atomnames)
            self.AType.append(a) 
            a.ID=len(self.AType)
            ll.remove(i)  
        for iAType in self.AType:
            for iAtom in iAType.Atoms:
                iAtom.Type=iAType
# Define local number of each atom
        for MT in self.MType:
            for a in xrange(0,len(MT.Atoms)):
                MT.Atoms[a].Number=a+1                
                    
# Reading list of intermolecular RDFs
        npairs=[int(0*obj) for obj in range(0,nrdf)]
        ipairs=[]
        apair=[]
# generating temporary dictionary for following RDF records checkig
        atypes=dict([(obj.Name,obj.AtomNames) for obj in self.AType])
#Reading intermolecular RDFs
        for k in range(0,nrdf):
# Reading an RDF-record from input
            apair.append(''.join(ll[iRDFEnd+1].split()).split('--'))
            if ll[iRDFEnd+2].lstrip().startswith('&'): # if many pairs            
                try:
                    npairs[k]=int(ll[iRDFEnd+2].lstrip().split('&')[1])
                    ipairs.append([[int(obj.split()[0])-1,int(obj.split()[1])-1] for obj in ll[iRDFEnd+3:iRDFEnd+3+npairs[k]]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic pairs in intermolecular distribution {0}, line {1}".format(k,obj))
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3+npairs[k])
            else:
                try:
                    npairs[k]=1
                    ipairs.append([[int(ll[iRDFEnd+2].split()[0])-1,int(ll[iRDFEnd+2].split()[1])-1]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic pairs in intermolecular distribution {0}, line {1}".format(k,ll[iRDFEnd+2]))                    
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3)
# Checking consistency of readed intermolecular RDF
            for j in range(0,npairs[k]):
                if not (self.AtomByID[ipairs[k][j][0]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationError('Error in the intermolecular distribution',self,k,j,ipairs,apair,atypes)                    
                if not (self.AtomByID[ipairs[k][j][1]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationError('Error in the intermolecular distribution',self,k,j,ipairs,apair,atypes)                    
                # Creating RDF object
            rdfatompairs=[[self.AtomByID[ipairs[k][j][0]],self.AtomByID[ipairs[k][j][1]]] for j in range(0, npairs[k])]
            trdf=DF.RDF_NB(Name=apair[k][0]+"-"+apair[k][1],
                            Min=0.0,
                            Max=rmax,
                            Npoints=int(rmax/resol),
                            Kind='RDF',Type='NB',
                            _AtomGroups_=rdfatompairs,
                            AtomTypes=[rdfatompairs[0][i].Type.Name for i in (0,1)])
            self.RDFs.append(trdf)
#Reading intramolecular RDFs
# Reading list of intramolecular RDFs
        npairs=[int(0*obj) for obj in range(0,nrdfi)]
        ipairs=[]
        apair=[]
        imtype=[]
# generating temporary dictionary for following RDF records checkig
        atypes=dict([(obj.Name,obj.AtomNames) for obj in self.AType])
        
#Reading intramolecular RDFs
        for k in range(0,nrdfi): # over all intramolecular pairwise distribuions
# Reading one RDF-record from input
            apair.append(ll[iRDFEnd+1].split()[0].split('--'))
            imtype.append(int(ll[iRDFEnd+1].split()[1]))
            if ll[iRDFEnd+2].lstrip().startswith('&'): # if many pairs            
                try:
                    npairs[k]=int(ll[iRDFEnd+2].lstrip().split('&')[1])
                    ipairs.append([[int(obj.split()[0])-1,int(obj.split()[1])-1] for obj in ll[iRDFEnd+3:iRDFEnd+3+npairs[k]]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic pairs in intramolecular bond distribution {0}, line {1}".format(k,obj))
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3+npairs[k])
            else:
                try:
                    npairs[k]=1
                    ipairs.append([[int(ll[iRDFEnd+2].split()[0])-1,int(ll[iRDFEnd+2].split()[1])-1]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic pairs in intramolecular distribution {0}, line {1}".format(k,ll[iRDFEnd+2]))
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3)
# Checking consistency of the read intramolecular RDF
            for j in xrange(0,npairs[k]): # over all atom pairs in the given distribution
                if not (self.AtomByID[ipairs[k][j][0]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationErrorIRDF('Error in the intramolecular distribution',
                    self,k,j,ipairs,apair,atypes,imtype)                    
                if not (self.AtomByID[ipairs[k][j][1]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationErrorIRDF('Error in the intramolecular distribution',
                    self,k,j,ipairs,apair,atypes,imtype)
                if not (set((self.AtomByID[ipairs[k][j][0]],self.AtomByID[ipairs[k][j][1]])).issubset(self.MType[imtype[k]-1].Atoms)):
                    raise MTE.RDFValidationErrorIRDF('Error in the intramolecular distribution',
                      self,k,j,ipairs,apair,atypes,imtype)                                    
#Set bonds in the CG_Molecular_Type:
            pairs=[[self.AtomByID[ipairs[k][j][m]] for m in (0,1)] for j in xrange(0,npairs[k])]
            bond=Atom.Bond(ID=len(self.MType[imtype[k]-1].Bonds)+1, 
                            AtomGroups=pairs,
                            Type='B',
                            MolType=self.MType[imtype[k]-1].Name)
            self.MType[imtype[k]-1].Bonds.append(bond)
# Creating RDF object        
            rdfatompairs=[[self.AtomByID[ipairs[k][j][0]],self.AtomByID[ipairs[k][j][1]]] for j in range(0, npairs[k])]
            AtomGroups=[[a.Number for a in pair] for pair in rdfatompairs]
            trdf=DF.RDF_B(Name=apair[k][0]+"-"+apair[k][1],
                           Min=0.0,
                           Max=rmaxi,
                           Npoints=int(rmaxi/resoli),
                           Kind='RDF',Type='B',
                           _AtomGroups_=rdfatompairs,
                           AtomGroups=AtomGroups,
                           MolTypeName=self.MType[imtype[k]-1].Name,
                           BondNumber=self.MType[imtype[k]-1].Bonds.index(bond)+1)
                
            self.RDFs.append(trdf)

#Reading intramolecular ADFs
# Reading list of intramolecular ADFs
        npairs=[int(0*obj) for obj in range(0,nadf)] #They are actually triples
        ipairs=[]
        apair=[]
        imtype=[]
# generating temporary dictionary for following ADF records checkig
        atypes=dict([(obj.Name,obj.AtomNames) for obj in self.AType])        
#Reading intramolecular ADFs
        for k in range(0,nadf):
# Reading one ADF-record from input
            apair.append(ll[iRDFEnd+1].split()[0].split('--'))
            imtype.append(int(ll[iRDFEnd+1].split()[1]))
            if ll[iRDFEnd+2].lstrip().startswith('&'): # if many triples            
                try:
                    npairs[k]=int(ll[iRDFEnd+2].lstrip().split('&')[1])
                    ipairs.append([[int(obj.split()[0])-1,int(obj.split()[1])-1,int(obj.split()[2])-1] for obj in ll[iRDFEnd+3:iRDFEnd+3+npairs[k]]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic triplets in intramolecular angle distribution {0}, line {1}".format(k,obj))
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3+npairs[k])
            else:
                try:
                    npairs[k]=1
                    ipairs.append([[int(ll[iRDFEnd+2].split()[0])-1,int(ll[iRDFEnd+2].split()[1])-1,int(ll[iRDFEnd+2].split()[2])-1]])
                except:
                    raise MTE.RDFReadError("Unable to read atomic triplets in intramolecular angle distribution {0}, line {1}".format(k,ll[iRDFEnd+2]))
                ll.__delslice__(iRDFEnd+1,iRDFEnd+3)
# Checking consistency of readed ADF
            for j in xrange(0,npairs[k]):
                if not (self.AtomByID[ipairs[k][j][0]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationErrorADF('Error in the intramolecular angle distribution',
                      self,k,j,ipairs,apair,atypes,imtype) 
                if not (self.AtomByID[ipairs[k][j][1]].Name in (atypes[apair[k][0]]+atypes[apair[k][1]])):        
                    raise MTE.RDFValidationErrorADF('Error in the intramolecular angle distribution',
                      self,k,j,ipairs,apair,atypes,imtype) 
                if (not(set((self.AtomByID[ipairs[k][j][0]],self.AtomByID[ipairs[k][j][1]],self.AtomByID[ipairs[k][j][2]])).issubset(self.MType[imtype[k]-1].Atoms))):
                    raise MTE.RDFValidationErrorADF('Error in the intramolecular angle distribution',
                      self,k,j,ipairs,apair,atypes,imtype) 

#Set bonds in the CG_Molecular_Type:
            pairs=[[self.AtomByID[ipairs[k][j][m]] for m in (0,1,2)] for j in xrange(0,npairs[k])]
            pairs123=[[self.AtomByID[ipairs[k][j][m]] for m in (0,2,1)] for j in xrange(0,npairs[k])]
            bond=Atom.Bond(ID=len(self.MType[imtype[k]-1].Bonds)+1, 
                            AtomGroups=pairs123,
                            Type='A',
                            MolType=self.MType[imtype[k]-1].Name)

            self.MType[imtype[k]-1].Bonds.append(bond)
# Creating ADF object

            rdfatompairs=[[self.AtomByID[ipairs[k][j][0]],self.AtomByID[ipairs[k][j][2]],self.AtomByID[ipairs[k][j][1]]] for j in range(0, npairs[k])]
            AtomGroups=[[a.Number for a in pair] for pair in rdfatompairs]
            
            trdf=DF.RDF_A(Name=apair[k][0]+"-"+apair[k][1]+'.ADF',
                            Min=0.0,
                            Max=180.0,
                            Npoints=int(180.0/(resolphi)),
                            Kind='RDF',Type='A',
                            _AtomGroups_=rdfatompairs,
                            AtomGroups=AtomGroups,
                            MolTypeName=self.MType[imtype[k]-1].Name,
                            BondNumber=self.MType[imtype[k]-1].Bonds.index(bond)+1)                            
                            
            self.ADFs.append(trdf)
        self.DFset=DFset.DFset(Kind='RDF', Name=None, 
                    Types=[i.Name for i in self.AType], 
                    Min=0,
                    Max=rmax,
                    Npoints=int(rmax/resol), 
                    DFs=self.RDFs+self.ADFs) 
    def _RDF_pairs_matrix(self):
        #Creating list of pairs from configuration to be analized for certain RDF        
        #Loop over all RDFs in the system        
        for rdf in self.RDFs:
            if rdf.Type=='NB': # intermolecular RDF
                list=[]
                for pair in rdf._AtomGroups_: # Over all pairs of atoms of the RDF
                    #search for molecular type of current pair and check that there is only one according Mol_Type
                    if (int(sum([obj.Atoms.count(pair[0]) for obj in self.MType]))!=1):
                        print "first atom of pair is presented in ",int(sum([obj.Atoms.count(pair[0]) for obj in self.MType])),"moltypes"
                        exit(4)
                    if (int(sum([obj.Atoms.count(pair[1]) for obj in self.MType]))!=1):
                        print "first atom of pair is presented in ",int(sum([obj.Atoms.count(pair[1]) for obj in self.MType])),"moltypes"
                        exit(4)
                    for moltype1 in self.MType:
                        if pair[0] in moltype1.Atoms:
                            break
                    for moltype2 in self.MType:
                        if pair[1] in moltype2.Atoms:
                            break 
                    indmol=[self.MType.index(moltype1),self.MType.index(moltype2)] # pairs of moltype indexes
                    if pair[0]==pair[1]: # Pair of same atoms (they must belong to different molecules)
                        # Number of atoms listed before
                        prefix=[int(sum([self.NMolMType[obj]*self.MType[obj].NAtom for obj in range(0,indmol[k])])) for k in [0,1]]                        
                        for imol in range(0,self.NMolMType[indmol[0]]): # Loop over all molecules of moltype1
                            a=prefix[0]+imol*self.MType[indmol[0]].NAtom+self.MType[indmol[0]].Atoms.index(pair[0]) #first atom of pair positions
                            for jmol in range(imol+1,self.NMolMType[indmol[1]]): # Loop over moleculs of moltype 2
                                b=prefix[1]+jmol*self.MType[indmol[1]].NAtom+self.MType[indmol[1]].Atoms.index(pair[1]) #first atom of pair positions
                                list.append([a,b])
                        
                    if pair[0]!=pair[1]: # Pair of different atoms (they could belong to same molecule as well)
                        # Number of atoms listed before
                        prefix=[int(sum([self.NMolMType[obj]*self.MType[obj].NAtom for obj in range(0,indmol[k])])) for k in [0,1]]                        
                        for imol in range(0,self.NMolMType[indmol[0]]): # Loop over all molecules of moltype1
                            a=prefix[0]+imol*self.MType[indmol[0]].NAtom+self.MType[indmol[0]].Atoms.index(pair[0]) #first atom of pair positions
                            for jmol in range(0,self.NMolMType[indmol[1]]): # Loop over moleculs of moltype 2
                                b=prefix[1]+jmol*self.MType[indmol[1]].NAtom+self.MType[indmol[1]].Atoms.index(pair[1]) #second atom of pair positions
                                list.append([a,b])
                rdf._pairslist=list
        for rdf in self.RDFs:
            if rdf.Type =='B': # intramolecular RDF
                list=[]
                for pair in rdf._AtomGroups_: # Over all pairs of atoms of the RDF
                    if (int(sum([obj.Atoms.count(pair[0]) for obj in self.MType]))!=1):
                        print "first atom of pair is presented in ",int(sum([obj.Atoms.count(pair[0]) for obj in self.MType])),"moltypes"
                        exit(4)
                    if (int(sum([obj.Atoms.count(pair[1]) for obj in self.MType]))!=1):
                        print "first atom of pair is presented in ",int(sum([obj.Atoms.count(pair[1]) for obj in self.MType])),"moltypes"
                        exit(4)
                    for moltype1 in self.MType:
                        if pair[0] in moltype1.Atoms:
                            break
                    for moltype2 in self.MType:
                        if pair[1] in moltype2.Atoms:
                            break 
                    if moltype1!=moltype2:
                        print "intramolecular RDF:",self.RDFS.index(rdf),' has atoms which belong to different molecular types'
                        exit(4)
                    indmol=self.MType.index(moltype1) # pairs of moltype indexes
                    prefix=int(sum([self.NMolMType[obj]*self.MType[obj].NAtom for obj in range(0,indmol)]))
                    for imol in range(0,self.NMolMType[indmol]): # Loop over all molecules of moltype1
                            a=prefix+imol*self.MType[indmol].NAtom+self.MType[indmol].Atoms.index(pair[0]) #first atom of pair positions
                            b=prefix+imol*self.MType[indmol].NAtom+self.MType[indmol].Atoms.index(pair[1]) #second atom of pair positions
                            list.append([a,b])
                            #remove this pair from intermolecular RDFs if exist
                            for irdf in self.RDFs:
                                if irdf.Type=='NB':
                                    while [a,b] in irdf._pairslist:
                                        irdf._pairslist.remove([a,b])
                                    while [b,a] in irdf._pairslist:
                                        irdf._pairslist.remove([b,a])
                rdf._pairslist=list
    def _ADF_pairs_matrix(self):
        #Creating list of pairs from configuration to be analized for certain ADF        
        #Loop over all ADFs in the system        
        for adf in self.ADFs:
            if adf.Type=='A': # intramolecular ADF
                list=[]
                for pair in adf._AtomGroups_: # Over all triplets of atoms of the ADF
                    if (int(sum([obj.Atoms.count(pair[0]) for obj in self.MType]))!=1):
                        print "First atom of triplet is presented in ",int(sum([obj.Atoms.count(pair[0]) for obj in self.MType])),"moltypes"
                        exit(4)
                    if (int(sum([obj.Atoms.count(pair[1]) for obj in self.MType]))!=1):
                        print "Second atom of triplet is presented in ",int(sum([obj.Atoms.count(pair[1]) for obj in self.MType])),"moltypes"
                        exit(4)
                    if (int(sum([obj.Atoms.count(pair[2]) for obj in self.MType]))!=1):
                        print "Third atom of triplet is presented in ",int(sum([obj.Atoms.count(pair[2]) for obj in self.MType])),"moltypes"
                        exit(4)
                    for moltype1 in self.MType:
                        if pair[0] in moltype1.Atoms:
                            break
                    for moltype2 in self.MType:
                        if pair[1] in moltype2.Atoms:
                            break 
                    for moltype3 in self.MType:
                        if pair[2] in moltype3.Atoms:
                            break 
                    if (moltype1!=moltype2) or (moltype3!=moltype2) or (moltype1!=moltype3):
                        print "intramolecular ADF:",self.ADFs.index(adf),' has atoms which belong to different molecular types'
                        exit(4)
                    indmol=self.MType.index(moltype1) # index of molecular type ADF belongs to
                    prefix=int(sum([self.NMolMType[obj]*self.MType[obj].NAtom for obj in range(0,indmol)]))
                    for imol in range(0,self.NMolMType[indmol]): # Loop over all molecules of moltype1
                            a=prefix+imol*self.MType[indmol].NAtom+self.MType[indmol].Atoms.index(pair[0]) #first atom of triplet positions
                            b=prefix+imol*self.MType[indmol].NAtom+self.MType[indmol].Atoms.index(pair[1]) #second - central atom of triplet positions
                            c=prefix+imol*self.MType[indmol].NAtom+self.MType[indmol].Atoms.index(pair[2]) #trird atom of triplet positions

                            list.append([a,b,c])
                            #remove this pair from intermolecular RDFs if exist
                            for irdf in self.RDFs:
                                if irdf.Type=='NB':
                                    while [a,c] in irdf._pairslist:
                                        irdf._pairslist.remove([a,c])
                                    while [c,a] in irdf._pairslist:
                                        irdf._pairslist.remove([c,a])
                adf._pairslist=list
                       
    def RDFs_Accumulate_new(self):
        import numpy
                
    # Loop over trajectory configurations, counting pairs
        self.Trajectory.connect()
        eof=self.Trajectory.readconf()
        for rdf in self.RDFs:
            rdf._ghist=numpy.zeros(len(rdf._ghist))
        box_ac=[0.0,0.0,0.0,0]
        if self.Trajectory.box!=[]: 
            box=self.Trajectory.box            
        else:
            box=self.box
        conf=1
        while eof==0: 
            box_ac[0]+=box[0]
            box_ac[1]+=box[1]
            box_ac[2]+=box[2]
            box_ac[3]+=1        
            for rdf in self.RDFs:		
                c=self.Trajectory.conf
                dr=c[[p[1] for p in rdf._pairslist],:]-c[[p[0] for p in rdf._pairslist],:]
                dr=numpy.abs(dr)
                dx=dr[:,0]
                dy=dr[:,1]
                dz=dr[:,2]
                dx=numpy.where(dx>0.5*box[0],box[0]-dx, dx)
                dy=numpy.where(dy>0.5*box[1],box[1]-dy, dy)
                dz=numpy.where(dz>0.5*box[2],box[2]-dz, dz)
                dr2=dx*dx+dy*dy+dz*dz
                drr=numpy.sqrt(dr2)
                drr=numpy.where(drr>=rdf.Max,0,drr)
                drr=numpy.where(drr<rdf.Min,0,drr)
                i=(drr/rdf.Resol).astype(int)
                rdf._ghist+=numpy.bincount(i, minlength=len(rdf._ghist))
            print 'Configuration:',conf,'analyzed \r',   
            eof=self.Trajectory.readconf()
            if self.Trajectory.box!=[]: 
                box=self.Trajectory.box            
            conf+=1
        box_ac[0]=box_ac[0]/box_ac[3]
        box_ac[1]=box_ac[1]/box_ac[3]
        box_ac[2]=box_ac[2]/box_ac[3]
        print ''
        if self.box!=box_ac[0:3]:
            print 'Box stated in input ',self.box,' do not agree with trajectory averaged value ',box_ac[0:4]
            print 'Program continue with avergared box size'
            self.box=box_ac[0:3]

    def ADFs_Accumulate_new(self):
        import numpy
                
        self._ADF_pairs_matrix() #Create matrix of triplets for ADFs
        for adf in self.ADFs:
            adf._ghist=numpy.zeros(len(adf._ghist))

    # Loop over trajectory configurations, counting pairs
        self.Trajectory.connect()
        eof=self.Trajectory.readconf()
        conf=1        
        while eof==0: 
            for adf in self.ADFs:
                c=self.Trajectory.conf
                
                r21=c[[p[1] for p in adf._pairslist],:]-c[[p[0] for p in adf._pairslist],:]
                r23=c[[p[1] for p in adf._pairslist],:]-c[[p[2] for p in adf._pairslist],:]
                cosphi=numpy.sum(numpy.multiply(r21,r23),axis=1)/numpy.sqrt(numpy.sum(numpy.multiply(r21,r21),axis=1)*numpy.sum(numpy.multiply(r23,r23),axis=1))
                phi=numpy.arccos(cosphi)*179.99999/numpy.pi
                i=((phi)/adf.Resol).astype(int)
                adf._ghist+=numpy.bincount(i, minlength=len(adf._ghist))
            print 'Configuration:',conf,'analyzed \r',   
            eof=self.Trajectory.readconf()
            conf+=1            

    def Normalize(self,DFs):                
        if (len(self.box)==3): 
            vol=self.box[0]*self.box[1]*self.box[2]
        else:
            vol=1.0
        for df in DFs:
            df.Normalize(vol)
    def Smooth(self,DFs):
        for df in DFs:
            df.Smooth()
    def Trim(self,DFs,tolerance):
        for df in DFs:
            df.Trim(tolerance)
    def Plot(self,DFs,t=5.0):
        import time
        for df in DFs:
            df.Plot()
            time.sleep(t)        
    def RDFs_SaveForMagic_old(self):
        if len(self.RDFs)>0:
            try:
                ofile=open(self.RDFOutputFile,'w')
            except:
                print "Error:Unable to create RDF output file:",self.RDFOutputFile;            
            ofile.write(str(self.NAType)+' '+str(self.RDFs[0].Npoints)+' '+str(self.RDFs[0].rmin)+' '+str(self.RDFs[0].rmax)+'\n')
            for type in self.AType:
                ofile.write(type.Name+'\n')        
            for rdf in self.RDFs:
                if rdf.Type=='NB':
    # Find numbers of CG_Atom_Type of rdf
                    a=rdf.AtomGroups[0]
                    i=[-1,-1]
                    for iAT in self.AType:
                        for k in (0,1): 
                            if a[k] in iAT.Atoms: i[k]=self.AType.index(iAT)
                    if (i[0]==-1) or (i[1]==-1):
                        print  "RDF Error: Refered CGAtomTypes not found for atom pair",a[0].Name,a[1].Name; exit(6)
    # Write RDF table to file
                    for j in xrange(rdf.FirstNonZero,rdf.LastNonZero+1):
                        line=str(rdf.g[j,0])+' '+str(rdf.g[j,1])+' '+str(i[0]+1)+' '+str(i[1]+1)+'\n'
                        ofile.write(line)
    #intramolecular part        
            ofile.write('intra \n')
            for rdf in self.RDFs:
                if rdf.Type=='B':
    #Writing header of each rdf
                    line=str(len(rdf.AtomGroups))+' '+str(rdf.LastNonZero-rdf.FirstNonZero+1)\
                    +' '+str(rdf.g[rdf.FirstNonZero,0]-rdf.resol*0.5)+' '+str(rdf.g[rdf.LastNonZero,0]+rdf.resol*0.5)+'\n'
                    ofile.write(line)
                    for ipair in rdf.AtomGroups:
                        line=str(self.AtomByID.index(ipair[0])+1)+' '+str(self.AtomByID.index(ipair[1])+1)+'\n'                    
                        ofile.write(line)
    # Write RDF table to file
                    for i in xrange(rdf.FirstNonZero,rdf.LastNonZero+1):
                        line=str(rdf.g[i,0])+' '+str(rdf.g[i,1])+'     '+\
                        rdf.AtomGroups[0][0].Name+'-'+rdf.AtomGroups[0][1].Name+'\n'        
                        ofile.write(line)        
            ofile.close()
    def DFs_SaveForMagic(self):
        import sys
        if self.RDFs[0].Type!='NB':
            sys.exit("Error: No intermolecular RDF calculated:")                      
        try:
            self.DFset.Write(self.RDFOutputFile)
        except:
            print "Error:Unable to write RDF output file:",self.RDFOutputFile;            
    def Calculate(self,DFs,tolerance=0.001,plot=True,smooth=True,trim=True):
        if DFs==self.ADFs:
            self.ADFs_Accumulate_new()
        else:
            self.RDFs_Accumulate_new()    
            
        self.Normalize(DFs)
        if smooth:
            self.Smooth(DFs)
        if trim:
            self.Trim(DFs,tolerance)
        if plot:
            self.Plot(DFs)
    def RDFs_Calculate(self,tolerance=0.001,plot=True,trim=True,smooth=True):
        self.Calculate(self.RDFs,tolerance=tolerance, plot=plot, trim=trim, smooth=smooth)
    def ADFs_Calculate(self,tolerance=1.0e-6,plot=True,trim=True,smooth=True):
        self.Calculate(self.ADFs,tolerance=tolerance, plot=plot, trim=trim, smooth=smooth)            
    def _SortAtomsByType_(self):        
# Define atom-type for every atom        
        for iatype in self.AType:
            for iatom in iatype.Atoms:
                iatom.Type=iatype
        for imtype in self.MType: # Over all molecule types
            for iatype in self.AType: # Over all atom types            
                for iatom in imtype.Atoms: # Over all atoms of the molecule
                    if iatom in iatype.Atoms: # If the atom is of type iatype
                        self._AtomsSortedByType_.append(iatom)
    
    def write_MCMs(self):
        # First we need to assign atom.TypeID
        # list all atomtypes
        seq=[A.Type for MT in self.MType for A in MT.Atoms]
        seen = set()
        # unique list, preserving order
        s = [ x for x in seq if not (x in seen or seen.add(x))]
        # dictionary of Type:TypeID
        d=dict([(v,i) for i,v in enumerate(s)])
        # Set TypeID values
        for iMType in self.MType:
            for A in iMType.Atoms:
                A.TypeID=d[A.Type]+1
#                A.TypeID=self.AType.index(A.Type)+1
                
# Loop over all CG_molecular types to be written in mcm files 
        for iMType in self.MType:
            iMType.WriteMCM() 
        
            
        
