!
!      Subroutines implementing simulation algorithms
!
!      SCVEL   - temperature controll in MD
!      MCStep  - MC (Metropolis) single atom shift
!      MCTrans  -  MC (Metropolis) single molecule shift
!      MCROT    - MC (Metropolis) single molecule rotation
!

!==================== MCStep =======================================

subroutine MCStep(Q,SX,SY,SZ,DR,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,&
                RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,ALPHA,RMN,RMX,&
                PotB_riBond,RX,RY,RZ,RKV,SSIN,SCOS,&
                DE,ENER,EEL,EINT,EINT_ang,NSTOT,ITYPE,STypeAtom,NNUM,NA,SiteAtom,Bond_iSjS,NAI,NKV,IUD,&
                LMoveMType,ID,NII,NII_ang,BTypeBond,&
                AdrAtomMol,NSiteMType,NOP,MTypeMol,site_ang_bonds,site_ang_bond_edge,num_site_ang_bonds)
    use modforces
    use utilsmod
    implicit none
!-------Global Variables--------------------
    real(8) DR,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,&
                    RMAX,RDFINC,RECUT,COULF,ALPHA,RDINC,DE,ENER,EEL,EINT,EINT_ang
    real(8),allocatable:: RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),PotNB_riSjS(:,:,:),&
                    RMN(:),RMX(:),PotB_riBond(:,:),Q(:),SX(:),SY(:),SZ(:)
    integer(4), allocatable::AdrAtomMol(:),NSiteMType(:),MTypeMol(:),ITYPE(:),&
                    STypeAtom(:),NNUM(:),SiteAtom(:),Bond_iSjS(:,:),NAI(:),&
                    site_ang_bonds(:,:,:),site_ang_bond_edge(:,:),num_site_ang_bonds(:),BTypeBond(:)
    integer(4) NSTOT,NA,NKV,ID,NII,NII_ang,NOP
    integer(8) IUD    
    logical LMoveMType(:)
!-------Local Variables---------------------
    real(8) CHI,X1,Y1,Z1,DDI,DDI_ang,DDE,DEE,DEW,DX,DY,DZ,RRN,RRO,FORCE,&
            DX_IK,DY_IK,DZ_IK,DX_JK,DY_JK,DZ_JK,DX_IJ,DY_IJ,DZ_IJ,phi_new,phi_old
    integer(4) I,IST,IMOL,ISA,J,JST,JMOL,JSA,IPP,K,ibond
    logical LEXIT

    !  Choice of trial particle
    do
        I=NSTOT*rand()+1
	if(LMoveMType(ITYPE(I))) exit
    enddo
    IST=STypeAtom(I)
    ISA=SiteAtom(I)
    IMOL=NNUM(I)
    CHI=Q(I)
    X1=SX(I)+DR*(rand()-0.5d0)
    Y1=SY(I)+DR*(rand()-0.5d0)
    Z1=SZ(I)+DR*(rand()-0.5d0)
    !  Init Energy difference counters
    DDI=0.
    DDI_ang=0.0
    DDE=0.
    DEE=0.
    DEW=0.
    LEXIT=.false. ! exit from main loop identifier for OMP-code
    ! Calculate energy differences: First in the same molecule, than with all other molecules
    do J=AdrAtomMol(IMOL)+1,AdrAtomMol(IMOL)+NSiteMType(MTypeMol(IMOL)) ! Loop over all atoms in the molecule of the atom I, except atom I
        if (J.ne.I) then ! Not a shifted atom
            JMOL=NNUM(J)
!  	    write(*,'(a,2i5,9f8.4)')' Pairs: ',I,J,RRN,RRO,SX(I),SY(I),SZ(I),SX(J),SY(J),SZ(J),RCUT
            JSA=SiteAtom(J)
            IPP=Bond_iSjS(ISA,JSA)
            if(IPP.le.0) then ! No bond between atoms
                DX=SX(J)-X1
                DY=SY(J)-Y1
                DZ=SZ(J)-Z1
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRN=sqrt(DX**2+DY**2+DZ**2)
                DX=SX(J)-SX(I)
                DY=SY(J)-SY(I)
                DZ=SZ(J)-SZ(I)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRO=sqrt(DX**2+DY**2+DZ**2)
                if(RRN.le.RCUT) then
                    DDE=DDE+ENERG(RRN,STypeAtom(I),STypeAtom(J),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)                  
                endif
                if(RRO.le.RCUT) then
                    DDE=DDE-ENERG(RRO,STypeAtom(I),STypeAtom(J),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)                    
                endif
!                if (((CHI*Q(J))**2).gt.0.0000000001) then
                if ((CHI*Q(J)).ne.0.d0) then
                    if(RRN.le.RECUT) DEE=DEE+CHI*Q(J)*COULF*derfc(ALPHA*RRN)/RRN
                    if(RRO.le.RECUT) DEE=DEE-CHI*Q(J)*COULF*derfc(ALPHA*RRO)/RRO
                endif
            elseif (BTypeBond(IPP).eq.1) then ! If Pair bond (not an angle-bending)
                DX=SX(J)-X1
                DY=SY(J)-Y1
                DZ=SZ(J)-Z1
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRN=sqrt(DX**2+DY**2+DZ**2)
                if(RRN.gt.RMN(IPP).and.RRN.lt.RMX(IPP))  then ! If new interatomic distance is in bond range
                    DX=SX(J)-SX(I)
                    DY=SY(J)-SY(I)
                    DZ=SZ(J)-SZ(I)
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RRO=sqrt(DX**2+DY**2+DZ**2)
                    if(RRO.le.RMN(IPP).or.RRO.ge.RMX(IPP)) &
                            write((ID+6),'(a,4i5,7f8.4)')' Bad distance: ',I,J,ISA,JSA,RRO,SX(I),SY(I),SZ(I),SX(J),SY(J),SZ(J)
                    DDI=DDI+ENERI(RRN,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)   &
                                    -ENERI(RRO,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
!                    if(((CHI*Q(J))**2).gt.0.0000000001)
                    if ((CHI*Q(J)).ne.0.d0) DEE=DEE+CHI*Q(J)*COULF*((derfc(ALPHA*RRN)-1.)/RRN-(derfc(ALPHA*RRO)-1.)/RRO)
                else
!                    write(6+ID,*) 'NewMCStep: Trial bond distance out of range - skipped'
                    LEXIT=.true.
                    return ! New distance is out of allowed range, skip
                endif
            elseif (BTypeBond(IPP).eq.2) then ! Covalent angle bond
                !First locate the corner atom
                K=site_ang_bond_edge(ISA,JSA)+AdrAtomMol(IMOL)
                ! Calculate interatomic distances:
                DX_IK=X1-SX(K)
                DY_IK=Y1-SY(K)
                DZ_IK=Z1-SZ(K)
                call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                DX_JK=SX(J)-SX(K)
                DY_JK=SY(J)-SY(K)
                DZ_JK=SZ(J)-SZ(K)
                call PBC(DX_JK,DY_JK,DZ_JK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                ! Calculate trial angle
               phi_new=dacos((DX_IK*DX_JK+DY_IK*DY_JK+DZ_IK*DZ_JK)/sqrt((DX_IK**2+DY_IK**2+DZ_IK**2)*(DX_JK**2+DY_JK**2+DZ_JK**2)))&
                        *RAD2DEG
                if(phi_new.gt.RMN(IPP).and.phi_new.lt.RMX(IPP))  then ! If new angle is in allowed bond angle range
                    DX_IK=SX(I)-SX(K)
                    DY_IK=SY(I)-SY(K)
                    DZ_IK=SZ(I)-SZ(K)
                    call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    ! Calculate old angle
                    phi_old=dacos((DX_IK*DX_JK+DY_IK*DY_JK+DZ_IK*DZ_JK)/&
                        sqrt((DX_IK**2+DY_IK**2+DZ_IK**2)*(DX_JK**2+DY_JK**2+DZ_JK**2)))*RAD2DEG
                    if(phi_old.le.RMN(IPP).or.phi_old.ge.RMX(IPP)) &
                            write((ID+6),'(a,5i5,10f8.4)')' Bad angle: ',I,J,K,ISA,JSA,phi_old,&
                                    SX(I),SY(I),SZ(I),SX(J),SY(J),SZ(J),SX(K),SY(K),SZ(K)
                    DDI_ang=DDI_ang+ENERI_ang(phi_new,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)   &
                                    -ENERI_ang(phi_old,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
                    DX=SX(J)-X1
                    DY=SY(J)-Y1
                    DZ=SZ(J)-Z1
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RRN=sqrt(DX**2+DY**2+DZ**2)
                    DX=SX(J)-SX(I)
                    DY=SY(J)-SY(I)
                    DZ=SZ(J)-SZ(I)
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RRO=sqrt(DX**2+DY**2+DZ**2)
!                    if(((CHI*Q(J))**2).gt.0.0000000001)
                    if ((CHI*Q(J)).ne.0.d0) DEE=DEE+CHI*Q(J)*COULF*((derfc(ALPHA*RRN)-1.)/RRN-(derfc(ALPHA*RRO)-1.)/RRO) ! Exclude electrostatic part
                else !Trial bond angle out of range - skipped
                    LEXIT=.true.
                    return
                endif
            endif
        endif
    enddo
!   Consider energy change due to shift of edge atom in covalent angle bond. So here it is assumed that atom I stands in edge of angle bond
    do ibond=1,num_site_ang_bonds(ISA) ! Loop over all covalent angle bonds
        J=site_ang_bonds(1,ibond,ISA)+AdrAtomMol(IMOL) ! 1st side atom
        K=site_ang_bonds(2,ibond,ISA)+AdrAtomMol(IMOL) ! 2nd side atom
        IPP=Bond_iSjS(SiteAtom(J),SiteAtom(K))
        DX_IJ=X1-SX(J)
        DY_IJ=Y1-SY(J)
        DZ_IJ=Z1-SZ(J)
        call PBC(DX_IJ,DY_IJ,DZ_IJ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
        DX_IK=X1-SX(K)
        DY_IK=Y1-SY(K)
        DZ_IK=Z1-SZ(K)
        call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
        ! Calculate trial angle
        phi_new=dacos((DX_IJ*DX_IK+DY_IJ*DY_IK+DZ_IJ*DZ_IK)/sqrt((DX_IJ**2+DY_IJ**2+DZ_IJ**2)*(DX_IK**2+DY_IK**2+DZ_IK**2)))*RAD2DEG
        if(phi_new.gt.RMN(IPP).and.phi_new.lt.RMX(IPP))  then ! If new angle is in allowed bond angle range
            DX_IJ=SX(I)-SX(J)
            DY_IJ=SY(I)-SY(J)
            DZ_IJ=SZ(I)-SZ(J)
            call PBC(DX_IJ,DY_IJ,DZ_IJ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
            DX_IK=SX(I)-SX(K)
            DY_IK=SY(I)-SY(K)
            DZ_IK=SZ(I)-SZ(K)
            call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
            ! Calculate old angle
            phi_old=dacos((DX_IJ*DX_IK+DY_IJ*DY_IK+DZ_IJ*DZ_IK)/sqrt((DX_IJ**2+DY_IJ**2+DZ_IJ**2)*(DX_IK**2+DY_IK**2+DZ_IK**2)))&
                        *RAD2DEG
            if(phi_old.le.RMN(IPP).or.phi_old.ge.RMX(IPP)) &
                    write((ID+6),'(a,3i5,10f8.4)')' Bad angle: ',J,K,I,phi_old,&
                            SX(J),SY(J),SZ(J),SX(K),SY(K),SZ(K),SX(I),SY(I),SZ(I)
            DDI_ang=DDI_ang+ENERI_ang(phi_new,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)   &
                            -ENERI_ang(phi_old,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
        else !Trial bond angle out of range - skipped
            LEXIT=.true.
            return
        endif
    enddo
!    Now let's loop over all other molecules. Thus only intermolecular interactions are considered.
    do JMOL=1,NOP ! Over all molecules
        if (JMOL.ne.IMOL) then ! Not the same molecule
            do J=AdrAtomMol(JMOL)+1,AdrAtomMol(JMOL)+NSiteMType(MTypeMol(JMOL)) !Over all atoms in a molecule
                JSA=SiteAtom(J)
                DX=SX(J)-X1
                DY=SY(J)-Y1
                DZ=SZ(J)-Z1
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRN=sqrt(DX**2+DY**2+DZ**2)
                DX=SX(J)-SX(I)
                DY=SY(J)-SY(I)
                DZ=SZ(J)-SZ(I)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRO=sqrt(DX**2+DY**2+DZ**2)
                if(RRN.le.RCUT) then
                    DDE=DDE+ENERG(RRN,STypeAtom(I),STypeAtom(J),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                endif
                if(RRO.le.RCUT) then
                    DDE=DDE-ENERG(RRO,STypeAtom(I),STypeAtom(J),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                endif
!                if (((CHI*Q(J))**2).gt.0.0000000001) then
                if ((CHI*Q(J)).ne.0.d0) then
                    if(RRN.le.RECUT) DEE=DEE+CHI*Q(J)*COULF*derfc(ALPHA*RRN)/RRN
                    if(RRO.le.RECUT) DEE=DEE-CHI*Q(J)*COULF*derfc(ALPHA*RRO)/RRO
                endif
            enddo
        endif
    enddo    
!  Corrections due to electrostatics
    if (.not. LEXIT) then
        if(CHI.ne.0.d0)call DIFEW(X1,Y1,Z1,I,CHI,DEW,RX,RY,RZ,RKV,SSIN,SCOS,NKV,SX,SY,SZ)
    	DE=DDI+DDE+DEE+DEW+DDI_ang
!	write(*,'(a,i5,4f12.4)')' atom ',I,DE,DEE,DDI,DDI_ang
!  MC transition
      	if(DE.gt.0.0)then
            if(DE.gt.18.d0)return
            if(dexp(-DE).lt.rand())return
      	end if
!  New configuration
      	SX(I)=X1
      	SY(I)=Y1
      	SZ(I)=Z1
      	ENER=ENER+DE
        EEL=EEL+DEW+DEE
        EINT=EINT+DDI
        EINT_ang=EINT_ang+DDI_ang
      	IUD=IUD+1
       	if(CHI.ne.0.d0)call RECSIN (NKV,SSIN,SCOS)
    endif
    return
end !MCStep
!==================== MCTrans =======================================

subroutine MCTrans(Q,SX,SY,SZ,MCTransStep,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,ALPHA,&
            RX,RY,RZ,RKV,SSIN,SCOS,DE,ENER,EEL,NSTOT,STypeAtom,NNUM,NA,NKV,IUD,LMoveMType,&
            AdrAtomMol,MTypeMol,NSiteMType,NOP,IPRINT,ID)
    use modforces
    use utilsmod
!-------Global Variables--------------------
    real(8) HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,RMAX,RDFINC,RECUT,COULF,ALPHA,DE,ENER,EEL,MCTransStep
    real(8),allocatable:: RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),PotNB_riSjS(:,:,:),&
                    Q(:),SX(:),SY(:),SZ(:)
    integer(4) NSTOT,NA,NKV,NOP,IPRINT,ID
    integer(8) IUD
    integer(4),allocatable::STypeAtom(:),NNUM(:),NSiteMType(:),MTypeMol(:),AdrAtomMol(:)
    logical LMoveMType(:)
!-------Local Variables---------------------
    real(8) DDE,DEE,DEW,DX,DY,DZ,RRN,RRO,FORCE,deltax,deltay,deltaz
    real(8), allocatable:: deltaR(:,:)
    integer(4) I,J,JST,INATOM,JJ,nchatom,K
    integer(4),allocatable:: ichatomlist(:,:)

    !  Choice of trial Molecule
    do
        I=NOP*rand()+1
        INATOM=NSiteMType(MTypeMol(I)) ! Number of atoms in the selected molecule
	if(LMoveMType(MTypeMol(I)).and.(INATOM.gt.1)) exit
    enddo    
    allocate(deltaR(3,INATOM),ichatomlist(INATOM,2)) ! Array of shifted coordinates of the selected molecule.
    deltaR(:,:)=0.d0
    nchatom=0
  ! Trial shift
    deltax=(rand()-0.5d0)*MCTransStep
    deltay=(rand()-0.5d0)*MCTransStep
    deltaz=(rand()-0.5d0)*MCTransStep
    if (IPRINT.gt.7) write((ID+6),*) 'MCTrans trans: Ich,Q,    Xo,     Yo,     Zo,     Xn,    Yn,    Zn'
    do J=1,INATOM  ! Forming array of shifted coordinates
        JJ=AdrAtomMol(I)+J ! Global atom number of J-th atom in the molecule
        deltaR(1,J)=SX(JJ)+deltax
        deltaR(2,J)=SY(JJ)+deltay
        deltaR(3,J)=SZ(JJ)+deltaz
        if(Q(JJ).ne.0.d0) then
            nchatom=nchatom+1
            ichatomlist(nchatom,1)=J ! Atom number in the molecule
            ichatomlist(nchatom,2)=JJ ! Global atom number
            if (IPRINT.gt.7)write((ID+6),'(a,i5,7f9.4)') 'MCTrans Trans:',nchatom,Q(JJ),SX(JJ),SY(JJ),SZ(JJ),&
                        deltaR(1,J),deltaR(2,J),deltaR(3,J)
        endif
    enddo
!  Energy difference
    DDE=0.
    DEE=0.
    DEW=0.
    do J=1,INATOM ! Loop over all atoms in selected molecule
      	JJ=AdrAtomMol(I)+J
      	JST=STypeAtom(JJ) ! atomic type of J-th atom in the selected molecule
      	do K=1,NSTOT ! Loop over all atoms in the system
            if(NNUM(K).ne.I) then ! If K-atom does not belong to the selected molecule
                DX=SX(K)-deltaR(1,J)
		DY=SY(K)-deltaR(2,J)
		DZ=SZ(K)-deltaR(3,J)
		call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
		RRN=sqrt(DX**2+DY**2+DZ**2)
		DX=SX(K)-SX(JJ)
		DY=SY(K)-SY(JJ)
		DZ=SZ(K)-SZ(JJ)
		call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
		RRO=sqrt(DX**2+DY**2+DZ**2)
		if(RRN.le.RCUT)DDE=DDE+ENERG(RRN,JST,STypeAtom(K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
		if(RRO.le.RCUT)DDE=DDE-ENERG(RRO,JST,STypeAtom(K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
		if(RRN.le.RECUT) DEE=DEE+Q(JJ)*Q(K)*COULF*derfc(ALPHA*RRN)/RRN
		if(RRO.le.RECUT) DEE=DEE-Q(JJ)*Q(K)*COULF*derfc(ALPHA*RRO)/RRO
            endif
	enddo
    enddo
!  Corrections due to electrostatics
    call DIFEWM(ichatomlist,nchatom,RX,RY,RZ,RKV,SSIN,SCOS,SX,SY,SZ,NKV,deltaR,Q,DEW)
    DE=DDE+DEE+DEW
    if (IPRINT.gt.7) write((ID+6),'(a,i5,4f10.7)')' MCTrans trans: I, DE, DDE, DEE, DEW ',I,DE,DDE,DEE,DEW
    if(DE.gt.0)then
        if(DE.gt.18.d0) then
            deallocate(deltaR,ichatomlist)
            return
        endif
        if(dexp(-DE).lt.rand())then 
            deallocate(deltaR,ichatomlist)
            return
        endif
    end if
!  New configuration saving
    do J=1,INATOM
        JJ=AdrAtomMol(I)+J ! Global atom number of J-th atom in the molecule
        SX(JJ)=deltaR(1,J)
        SY(JJ)=deltaR(2,J)
        SZ(JJ)=deltaR(3,J)
    enddo
    ENER=ENER+DE
    EEL=EEL+DEE+DEW
    IUD=IUD+1
    if (IPRINT.gt.7) write((ID+6),*) "MCTrans trans: ENER=",ENER," DE=",DE
    call RECSINM(NKV,SSIN,SCOS)
    deallocate(deltaR,ichatomlist)
    return
end

!==================== MCRot =======================================
    subroutine MCRot(MCRotStep,WX,WY,WZ,Q,SX,SY,SZ,X,Y,Z,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,&
                    RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,ALPHA,&
                    RX,RY,RZ,RKV,SSIN,SCOS,&
                    DE,ENER,EEL,EINT,&
                    NSTOT,STypeAtom,NNUM,NA,NKV,IUD,LMoveMType,AdrAtomMol,MTypeMol,NSiteMType,NOP,IPRINT,&
                    IND,Bond_iSjS,SiteAtom,AdrSiteMType,MassSite,SumMassMType,ID)
    use modforces
    use utilsmod
!-------Global Variables--------------------
    real(8) HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,RMAX,RDFINC,RECUT,COULF,ALPHA,DE,ENER,EEL,EINT,MCRotStep
    real(8),allocatable:: RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),PotNB_riSjS(:,:,:),&
                    FORC(:,:,:),Q(:),SX(:),SY(:),SZ(:),WX(:),WY(:),WZ(:),X(:),Y(:),Z(:),MassSite(:),SumMassMType(:)
    integer(4) NSTOT,NA,NKV,NOP,IPRINT,ID
    integer(8) IUD
    integer(4),allocatable::STypeAtom(:),NNUM(:),NSiteMType(:),MTypeMol(:),AdrAtomMol(:),&
            IND(:,:,:),Bond_iSjS(:,:),SiteAtom(:),AdrSiteMType(:)
    logical LMoveMType(:),LMD
!-------Local Variables---------------------
    real(8) DDE,DEE,DEW,DX,DY,DZ,RRN,RRO,FORCE,phi,theta,psi,X1,Y1,Z1,ran,Dint,DintCOM,ddintcom
    real(8), allocatable:: deltaR(:,:)
    integer(4) I,J,JST,INATOM,JJ,nchatom,K
    integer(4),allocatable:: ichatomlist(:,:)

!  Choice of trial Molecule
    do
        I=NOP*rand()+1
        if (MTypeMol(I).eq.0) then
            write((ID+6),*) 'MCRot: Warning unknown molecular type of molecule number:',I
            stop
        else
            INATOM=NSiteMType(MTypeMol(I)) ! Number of atoms in the selected molecule
            if(LMoveMType(MTypeMol(I)).and.(INATOM.gt.1)) exit
        endif
    enddo
!Check center of mass
  call GETCOM(I,MTypeMol,SX,SY,SZ,NSiteMType,MassSite,SumMassMType,&
        WX,WY,WZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,AdrAtomMol,AdrSiteMType,X,Y,Z)

! Trial rotation generation
    call RanRotVec(x1,y1,z1)
    phi=rand()*2*3.141592653d0*MCRotStep
! Trial coordinates

    allocate(deltaR(3,INATOM),ichatomlist(INATOM,2)) ! Array of shifted coordinates of the selected molecule.
    deltaR(:,:)=0.d0
    ichatomlist(:,:)=0
    nchatom=0

  ! Trial rotaton
    if (IPRINT.gt.7) write((ID+6),*) 'MCRot: Ich,Q,    Xo,     Yo,     Zo,     Xn,    Yn,    Zn'

    do J=1,INATOM  ! Forming array of shifted coordinates
        JJ=AdrAtomMol(I)+J ! Global atom number of J-th atom in the molecule
        deltaR(1,J)=WX(JJ)
        deltaR(2,J)=WY(JJ)
        deltaR(3,J)=WZ(JJ)
        call rotate2(deltaR(1,J),deltaR(2,J),deltaR(3,J),phi,x1,y1,z1) ! Rotating atoms around center of MassSite of their molecule
        deltaR(1,J)=X(I)+deltaR(1,J) !Convert to system-wide coordinates (add COM-coordinates)
        deltaR(2,J)=Y(I)+deltaR(2,J)
        deltaR(3,J)=Z(I)+deltaR(3,J)
        if(Q(JJ).ne.0.d0) then
            nchatom=nchatom+1
            ichatomlist(nchatom,1)=J ! Atom number in the molecule
            ichatomlist(nchatom,2)=JJ! Global atom number
            if (IPRINT.gt.7) write((ID+6),'(a,i5,7f9.4)')'MCRot:',nchatom,Q(JJ),SX(JJ),SY(JJ),SZ(JJ),&
                                    deltaR(1,J),deltaR(2,J),deltaR(3,J)
        endif
    enddo
!  Energy difference
    DDE=0.
    DEE=0.
    DEW=0.
    Dint=0.
    do J=1,INATOM ! Loop over all atoms in the selected molecule
        JJ=AdrAtomMol(I)+J
        JST=STypeAtom(JJ) ! atomic type of J-th atom in the selected molecule
        do K=1,NSTOT ! Loop over all atoms in the system
            if(NNUM(K).ne.I) then ! If K-atom does not belong to the selected molecule
                DX=SX(K)-deltaR(1,J)
                DY=SY(K)-deltaR(2,J)
                DZ=SZ(K)-deltaR(3,J)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRN=sqrt(DX**2+DY**2+DZ**2)
                DX=SX(K)-SX(JJ)
                DY=SY(K)-SY(JJ)
                DZ=SZ(K)-SZ(JJ)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRO=sqrt(DX**2+DY**2+DZ**2)
                if(RRN.le.RCUT)DDE=DDE+ENERG(RRN,JST,STypeAtom(K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                if(RRO.le.RCUT)DDE=DDE-ENERG(RRO,JST,STypeAtom(K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                if(RRN.le.RECUT) DEE=DEE+Q(JJ)*Q(K)*COULF*derfc(ALPHA*RRN)/RRN
                if(RRO.le.RECUT) DEE=DEE-Q(JJ)*Q(K)*COULF*derfc(ALPHA*RRO)/RRO
            endif
        enddo
    enddo
! Check intramolecular short range interactions
    do J=1,INATOM-1 ! Loop over all atoms in selected molecule
        do K=J+1,INATOM ! Loop over all other atoms in the selected molecule
            if (Bond_iSjS(SiteAtom(AdrAtomMol(I)+J),SiteAtom(AdrAtomMol(I)+K)).le.0) then
                DX=deltaR(1,K)-deltaR(1,J)
                DY=deltaR(2,K)-deltaR(2,J)
                DZ=deltaR(3,K)-deltaR(3,J)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRN=sqrt(DX**2+DY**2+DZ**2)
                DX=SX(AdrAtomMol(I)+K)-SX(AdrAtomMol(I)+J)
                DY=SY(AdrAtomMol(I)+K)-SY(AdrAtomMol(I)+J)
                DZ=SZ(AdrAtomMol(I)+K)-SZ(AdrAtomMol(I)+J)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RRO=sqrt(DX**2+DY**2+DZ**2)
                if ((IND(1,STypeAtom(AdrAtomMol(I)+J),STypeAtom(AdrAtomMol(I)+K)).ne.-2).and.(RRN.le.RCUT)) &
                    DDE=DDE+ENERG(RRN,STypeAtom(AdrAtomMol(I)+J),STypeAtom(AdrAtomMol(I)+K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                if ((IND(1,STypeAtom(AdrAtomMol(I)+J),STypeAtom(AdrAtomMol(I)+K)).ne.-2).and.(RRO.le.RCUT)) &
                    DDE=DDE-ENERG(RRO,STypeAtom(AdrAtomMol(I)+J),STypeAtom(AdrAtomMol(I)+K),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
            endif
        enddo
    enddo
!  Corrections due to electrostatics
    call DIFEWM(ichatomlist,nchatom,RX,RY,RZ,RKV,SSIN,SCOS,SX,SY,SZ,NKV,deltaR,Q,DEW)
    DE=DDE+DEE+DEW+Dint
!  MC transition
    ran=rand()
    if (DE.gt.500) DE=500.0
    if ((DE.lt.0).or.(dexp(-DE).gt.ran)) then ! If accepted
!        if (IPRINT.ge.8) write(*,*) "MCRot: accepted"
        do J=1,INATOM
            JJ=AdrAtomMol(I)+J
            SX(JJ)=deltaR(1,J)
            SY(JJ)=deltaR(2,J)
            SZ(JJ)=deltaR(3,J)
            WX(JJ)=SX(JJ)-X(I)
            WY(JJ)=SY(JJ)-Y(I)
            WZ(JJ)=SZ(JJ)-Z(I)
        enddo
        IUD=IUD+1
        ENER=ENER+DE
        EINT=EINT+Dint
        EEL=EEL+DEE+DEW
        if (IPRINT.ge.7) write((ID+6),'(a,i5,5f10.7)')'MCRot:I,DE,DDE,DEE,DEW,Dint ',I,DE,DDE,DEE,DEW,dint
        if (IPRINT.gt.7) write((ID+6),*) "MCRot: ENER=",ENER," DE=",DE
        call RECSINM(NKV,SSIN,SCOS)
    endif
    deallocate(deltaR,ichatomlist)
    return
end
