!DEC$ FREEFORM
!Utility module which reshapes allocatable arrays of different dimensions and type.
module dynarray
    use precision
interface reshape  ! Generic operator for allocating and initializing or reshaping a 1D array. 
   module procedure reshapeint
   module procedure reshapeint8
   module procedure reshapereal
!   module procedure reshapechar4   
   module procedure reshapechar_atomname
end interface
interface reshape2  ! Generic operator for allocating and initializing or reshaping a 2D array. 
   module procedure reshapereal2
   module procedure reshapeint2
   module procedure reshapeint8_2

end interface

contains
    subroutine reshapeint(array,newsize)
        integer(4),allocatable:: array(:),tarray(:)
        integer(4) newsize,oldsize
        if (newsize.lt.0) stop 'ReshaipeInt: wrong new size!'
        if (.not. allocated(array)) then 
            allocate(array(newsize))
            array(:)=0
        elseif (size(array).lt.newsize) then
            allocate(tarray(newsize))
!				write(*,*) 'Debug reshapeint tarray allocated:', allocated(tarray)
!				write(*,*) 'Debug reshapeint tarray size:', size(tarray)
            oldsize=size(array)
!				write(*,*) 'Debug reshapeint old size:', size(array)
            tarray(:)=0
            tarray(1:oldsize)=array(:)
            deallocate(array)
!				write(*,*) 'Debug reshapeint array deallocate'
            allocate(array(newsize))
!				write(*,*) 'Debug reshapeint array allocated again', allocated(array)
!				write(*,*) 'Debug reshapeint array size:', size(array)
            array(:)=0
            array(1:oldsize)=tarray(1:oldsize)
            deallocate(tarray)                
        endif
    !		write(*,*) 'Debug reshapeint finish'	
	end subroutine reshapeint
        
        subroutine reshapeint8(array,newsize)
        integer(8),allocatable:: array(:),tarray(:)
        integer(4) newsize,oldsize
        if (newsize.lt.0) stop 'ReshaipeInt: wrong new size!'
        if (.not. allocated(array)) then 
            allocate(array(newsize))
            array(:)=0
        elseif (size(array).lt.newsize) then
            allocate(tarray(newsize))
            oldsize=size(array)
            tarray(:)=0
            tarray(1:oldsize)=array(:)
            deallocate(array)
            allocate(array(newsize))
            array(:)=0
            array(1:oldsize)=tarray(1:oldsize)
            deallocate(tarray)                
        endif
	end subroutine reshapeint8

	subroutine reshapereal(array,newsize)
            real(8),allocatable:: array(:),tarray(:)
            integer(4) newsize,oldsize
            if (newsize.lt.0) stop 'ReshaipeReal: New array size was not set!!!'
            if (.not. allocated(array)) then 
                allocate(array(newsize))
                array(:)=0.d0
            elseif (size(array).lt.newsize) then
                allocate(tarray(newsize))
                oldsize=size(array)
                tarray(:)=0
                tarray(1:oldsize)=array(:)
                deallocate(array)
                allocate(array(newsize))
                array(:)=0
                array(1:oldsize)=tarray(1:oldsize)
                deallocate(tarray)
            endif
	end subroutine reshapereal
        
	subroutine reshapereal2(array,newsize1,newsize2)
		real(8),allocatable:: array(:,:),tarray(:,:)
		integer(4) newsize1,newsize2
		if ((newsize1 .lt. 0) .or. (newsize2 .lt. 0)) stop 'ReshaipeReal2: New array size was not set!!!'
		if (.not. allocated(array)) then 
                    allocate(array(newsize1,newsize2))				
                    array(:,:)=0
		elseif ((size(array,1).lt.newsize1) .or. (size(array,2).lt.newsize2)) then				
                    allocate(tarray(max(newsize1,size(array,1)),max(newsize2,size(array,2))))
                    tarray(:,:)=0
                    tarray(1:size(array,1),1:size(array,2))=array(:,:)
                    deallocate(array)
                    allocate(array(size(tarray,1),size(tarray,2)))
                    array(:,:)=tarray(:,:)
                    deallocate(tarray)
		endif
	end subroutine reshapereal2
        
	subroutine reshapechar4(array,newsize)
		character(4),allocatable:: array(:),tarray(:)
		integer(4) newsize,oldsize
		if (newsize.lt.0) stop 'ReshaipeChar4: New array size was not set!!!'
                if (.not. allocated(array)) then 
                    allocate(array(newsize))
                    array(:)=''
		elseif (size(array).lt.newsize) then
                    allocate(tarray(newsize))
                    oldsize=size(array)
                    tarray(:)=''
                    tarray(1:oldsize)=array(:)
                    deallocate(array)
                    allocate(array(newsize))
                    array(:)=tarray(:)
                    deallocate(tarray)
		endif
	end subroutine reshapechar4
        subroutine reshapechar_atomname(array,newsize)
		character(atom_name_len),allocatable:: array(:),tarray(:)
		integer(4) newsize,oldsize
		if (newsize.lt.0) stop 'ReshaipeChar4: New array size was not set!!!'
                if (.not. allocated(array)) then 
                    allocate(array(newsize))
                    array(:)=''
		elseif (size(array).lt.newsize) then
                    allocate(tarray(newsize))
                    oldsize=size(array)
                    tarray(:)=''
                    tarray(1:oldsize)=array(:)
                    deallocate(array)
                    allocate(array(newsize))
                    array(:)=tarray(:)
                    deallocate(tarray)
		endif
	end subroutine reshapechar_atomname
        
	subroutine reshapeint2(array,newsize1,newsize2)
		integer(4),allocatable:: array(:,:),tarray(:,:)
		integer(4) newsize1,newsize2
		if ((newsize1 .lt. 0) .or. (newsize2 .lt. 0)) stop 'ReshaipeInt2: New array size was not set!!!'
		if (.not. allocated(array)) then 
                    allocate(array(newsize1,newsize2))				
                    array(:,:)=0.0
		elseif ((size(array,1).lt.newsize1) .or. (size(array,2).lt.newsize2)) then                    
                    allocate(tarray(max(newsize1,size(array,1)),max(newsize2,size(array,2))))				
                    tarray(:,:)=0
                    tarray(1:size(array,1),1:size(array,2))=array(:,:)
                    deallocate(array)
                    allocate(array(size(tarray,1),size(tarray,2)))
                    array(:,:)=tarray(:,:)
                    deallocate(tarray)			
		endif
	end subroutine reshapeint2	
        
        subroutine reshapeint8_2(array,newsize1,newsize2)
		integer(8),allocatable:: array(:,:),tarray(:,:)
		integer(4) newsize1,newsize2
		if ((newsize1 .lt. 0) .or. (newsize2 .lt. 0)) stop 'ReshaipeInt2: New array size was not set!!!'
		if (.not. allocated(array)) then 
                    allocate(array(newsize1,newsize2))				
                    array(:,:)=0.0
		elseif ((size(array,1).lt.newsize1) .or. (size(array,2).lt.newsize2)) then                    
                    allocate(tarray(max(newsize1,size(array,1)),max(newsize2,size(array,2))))				
                    tarray(:,:)=0
                    tarray(1:size(array,1),1:size(array,2))=array(:,:)
                    deallocate(array)
                    allocate(array(size(tarray,1),size(tarray,2)))
                    array(:,:)=tarray(:,:)
                    deallocate(tarray)			
		endif
	end subroutine reshapeint8_2
end !Module
