! CollectMCsummary - subroutine collects statistic data from all processes with MPI, and the calculates final averaged values.
! PrintMCsummary - subroutine prints final output in Molsim (priorly collected )
#ifdef mpi
Subroutine CollectMCsummary(NAV,global_NAV,NMKSF,global_NMKSF,IUD,global_IUD,EELS,global_EELS,EINTS,global_EINTS,&
            ENERS,global_ENERS,VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE,LRDF,ID,psize,ierror)
    include 'mpif.h'
!Global variables
    integer(4) NAV,global_NAV,&                
                psize,ierror,ID
    integer(8) IUD(:),global_IUD(:),NMKSF,global_NMKSF
    real(8) EELS,global_EELS,&
            EINTS,global_EINTS,&
            ENERS,global_ENERS,&
            VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE
    logical LRDF
! Local variables
    integer(4) i
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Collect NAV
    call MPI_REDUCE(NAV,global_NAV,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect NMKSF
    call MPI_REDUCE(NMKSF,global_NMKSF,1,MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect IUD(:)
    call MPI_REDUCE(IUD(:),global_IUD(:),size(IUD),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)    
! Collect EELS
    call MPI_REDUCE(EELS,global_EELS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect EINTS
    call MPI_REDUCE(EINTS,global_EINTS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect ENERS
    call MPI_REDUCE(ENERS,global_ENERS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect PRESSURE PRES=NSTOT*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*FNR
    call MPI_REDUCE(VIRS,global_VIRS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIIS,global_VIIS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIRE,global_VIRE,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)

    if (ID.eq.1) then ! Calculate mean values and write debug/perform check
        if (global_NAV.ne.(NAV*psize)) write(*,*) 'MPI ERROR in NAV - reduction', global_NAV,'!=',NAV*psize
        if (global_NMKSF.ne.(psize*NMKSF))&
            write(*,*)'MPI ERROR in NMKSF-reduction, global_NMKSF!=NMKSF*psize', global_NMKSF,'!=',NMKSF*psize
    endif
end subroutine
#endif


! This subroutine prints final output in Molsim (priorly collected )
subroutine PrintMCsummary(ID,NAV,NMType,NOP,NMKSF,NMKS0,NSTOT,ITRANS,IROT,NMolMType,IUD,&
        BOXL,BOYL,BOZL,DR,TEMP,EPS,EELS,FKJM,EINTS,&
        ENERS,PRES,VIRS,VIRE,VIIS,FPR,LRDF,NameMType,FILRDF,FILPOT)
!Global variables
    integer(4) ID,NAV,NMType,NOP,NSTOT,ITRANS,IROT
    integer(4) NMolMType(:)
    integer(8) IUD(:),NMKSF,NMKS0
    real(8) BOXL,BOYL,BOZL,DR,TEMP,EPS,EELS,FKJM,EINTS,ENERS,PRES,VIRS,VIRE,VIIS,FPR
            
    logical LRDF    
    character*64 NameMType(:),FILRDF,FILPOT
! Local variables
    real(8) FNR
    integer(4) I 

    write((ID+6),*)'MagiC '
        write((ID+6),*)' Monte Carlo'

    write((ID+6),*)'-----------------------------------'
    if(LRDF)then
        write((ID+6),*)' Inverse simulation'
        write((ID+6),*)' File with original RDF:',FILRDF
    else
        write((ID+6),*)' Direct simulation'
        write((ID+6),*)' Potential is read from ',FILPOT
    end if
    if(NAV.gt.0)then
        FNR=1./dfloat(NAV)
    else
        FNR=0.
    end if
    write((ID+6),*)'Number of molecules:   ',NOP
    write((ID+6),*)'Number of species:     ',NMType
    write((ID+6),'(a,3f12.3)')' Box sizes:            ',BOXL,BOYL,BOZL
    write((ID+6),*)'Sampled configurations:',NAV
    write((ID+6),*)'Number of molecules of each species:'
    do I=1,NMType ! TODO simplify
        write((ID+6),'(i4,i6,3x,a,f6.2)')I,NMolMType(I),NameMType(I)
    end do
    write((ID+6),*)
    write((ID+6),*)'Total MC steps:        ',NMKSF
    write((ID+6),*)'MC steps with averaging',NMKSF-NMKS0
    write((ID+6),*)'Acceptance ratio atoms  ',dfloat(IUD(1))/dfloat(NMKSF)
    write((ID+6),*)'Acceptance ratio trans  ',dfloat(IUD(2))*Itrans/dfloat(NMKSF)
    write((ID+6),*)'Acceptance ratio rot    ',dfloat(IUD(3))*Irot/dfloat(NMKSF)
    write((ID+6),*)'Max. displacemet       ',DR
    write((ID+6),*)'Temperature            ',TEMP
    write((ID+6),*)'Dielectric permitivity ',EPS
    write((ID+6),*)'Electrostatic energy:  ',EELS*FKJM*FNR
    write((ID+6),*)'Bond energy:           ',EINTS*FKJM*FNR
    write((ID+6),*)'------------------------------------'
    write((ID+6),*)'Average energy:        ',ENERS*FKJM*FNR
    PRES=NSTOT*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*FNR
    write((ID+6),*)'Pressure                ',PRES
    write((ID+6),'(a,4f12.2)')'Pressure components    ',&
             -VIRS*FPR*FNR/3.,-VIRE*FPR*FNR/3.,-VIIS*FPR*FNR/3.,NSTOT*FPR
! Here I can add printing of RDF for non inverse run.
end
