module imcmod   
    use precision
    use strings
    use utilsmod
    use dynarray
    implicit none
    
    real(8), allocatable:: RAS(:), SHELV(:)
    integer(8), allocatable:: npairs(:,:)
    real(8), allocatable:: Sref_NB_riSjS(:,:,:), RDFref_NB_riSjS(:,:,:), RDFcalc_NB_riSjS(:,:,:), Scalc_NB_riSjS(:,:,:),&
                           Sref_B_riBond(:,:), RDFref_B_riBond(:,:), Scalc_B_riBond(:,:), RDFcalc_B_riBond(:,:)  
    real(8), allocatable:: COR(:)            ! Before calling potcorr, it keeps difference between <Sref> and <Scalc>, after calling potcorr - it keeps correction to potential.                           
    integer(4) :: NUR
    integer(4), allocatable, private:: IICMP(:),IUCMP(:),IPVT(:)
    real(8),allocatable, private::DIFF(:),CROSS(:,:)
    integer(4), allocatable,private:: STypePairs(:,:)
    character*11 comment
    
    contains
    include "imcmod_calcrdf.f90"
    include "imcmod_buildmatrix.f90"
    include "imcmod_setcorrections.f90"
    include "imcmod_printpotential.f90"
    subroutine PotCorrMain(CORS,REGP,RTM,DPOTM,CORP,RMIN,RMAX,&
            VOL,FRT,PotB_riBond,PotNB_riSjS,RMinPotB,RMaxPotB,IND,&
            NPointsPotB,INDI,NMIN,NSType,NA,&!NI,
            NBond,BTypeBond,MTypeBond,&
            MTypeSType,IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NPTT,&
            IndexGlobB_riBond,ITS,LIMC,LRDF, BaseOutFileName,NameSite,NameSType,LEV,MixEV,npairsrdf,NAV,&
            LPotCorrCheck,NMType,IREP,IterCorr,&
            StateNB_iSjS, StateB_iBond,&
            ID,iPrint, stdout)
    
    implicit none
!-------Global Variables--------------------
    real(8) REGP,RTM,DPOTM,RMIN,RMAX,VOL,FRT,S,MixEV
    real(8),allocatable::PotB_riBond(:,:),PotNB_riSjS(:,:,:),RMinPotB(:),RMaxPotB(:),CORMF(:)
    integer(8), allocatable:: CORS(:),CORP(:,:),npairsrdf(:,:)
    integer(4) NSType,NA,NBond,NMType,NAV,IREP,IterCorr,ID 
    integer(4),allocatable::IND(:,:,:),NPointsPotB(:),INDI(:,:),NMIN(:),MTypeSType(:),&
            IndexGlobNB_riSjS(:,:,:),iSiteinBond(:,:),jSiteinBond(:,:),kSiteinBond(:,:),NPairsTotalBond(:),NPTT(:),&
            IndexGlobB_riBond(:,:),ITS(:), BTypeBond(:),MTypeBond(:)
    logical LIMC,LRDF ,LEV,LPotCorrCheck
    character*64 BaseOutFileName
    character(atom_name_len), allocatable:: NameSite(:),NameSType(:)    
    integer(4) iPrint, stdout
    integer(4), allocatable:: StateNB_iSjS(:,:), StateB_iBond(:)
    
!-------Local Variables--------------------
    integer(4) INFO
    real(8) DevS,DevRDF,FNR 
    real(8) FELC, FELR    
    
! Generating comment-string to mark output lines:
    if (LPotCorrCheck) then
        write(comment,'(a7,i2,i2)') ' Intermediate ',IREP,IterCorr
    else
        write(comment,'(a7,i2)') ' Final ',IREP
    endif
! Calculate RDFs from sampled data:    
    call CalculateRDF(NAV,FNR, stdout, NSType, NA, NBond, vol, RMAX,&!RDFref_NB_riSjS, RDFref_B_riBond,
        CORS,npairsrdf, IndexGlobNB_riSjS, IndexGlobB_riBond, &
        StateNB_iSjS, StateB_iBond, BTypeBond,iSiteinBond, jSiteinBond, kSiteInBond,&
                NpointsPotB,RMaxPotB,RMinPotB, NPairsTotalBond, NameSType, NameSite)
                
! Print RDFs 
    call PrintRDF(LRDF,NAV, iPrint, stdout, NSType, NA, NBond,&! RDFref_NB_riSjS, RDFref_B_riBond,
        CORS, IndexGlobB_riBond, StateNB_iSjS, StateB_iBond, BTypeBond,iSiteinBond, jSiteinBond, kSiteInBond,&
        NpointsPotB,RMaxPotB,RMinPotB, NPairsTotalBond, NameSType, NameSite)
        
! Calculate deviation with reference RDFs        
    call CalcDeviation(DevS, DevRDF, NSType, NBond, NPointsPotB, NA,StateNB_iSjS, StateB_iBond)
    DevS=sqrt(DevS/(NA*(NMType+1)*NMType))
    DevRDF=sqrt(DevRDF/(NA*(NMType+1)*NMType))
    write(stdout,'(a7,a,f9.6,a7,f9.6)') comment, 'Deviation from references: S: ',DevS,' RDF: ',DevRDF
                
! Calculate and regularize differences between reference and sample data    
    call reshape(COR,size(CORS))
    allocate(CORMF(size(CORS)))
    call Regularization(REGP, DPOTM, RTM, CORMF,&
        NSType, NBond, StateNB_iSjS, StateB_iBond, NpointsPotB, IndexGlobNB_riSjS, IndexGlobB_riBond)
        
!   Zeros elemination
    If (LPotCorrCheck) write (stdout,*) "Potential correction convergence check ",IterCorr
    if(LIMC)then
        call IMC_build_matrix(NSType, NA, NBond,IndexGlobNB_riSjS, IndexGlobB_riBond, NPointsPotB, &
             IND, INDI,CORS, CORP, NMIN, FNR, iPrint, stdout)                
        write(stdout,*)' Number of equations ',NUR
        write(stdout,*)' regularization parameter  ',REGP
        write(stdout,*)' max. relative change rdf  ',RTM
        write(stdout,*)' max. change of potential ',DPOTM
        
        call DGESV(NUR,1,CROSS,NUR,IPVT,DIFF,NUR,INFO)

        if(INFO.ne.0)then ! If no solution found
            write(stdout,*)' singular correlation matrix '
            write(stdout,*)' no solution found '
        else ! if solution found
            call IMC_SetCorrections(NSType, NA, NBond, DPOTM, IND, INDI, NMIN,&
            IndexGlobNB_riSjS, IndexGlobB_riBond, NPointsPotB, StateB_iBond, CORS, &
            iPrint,stdout)
        endif
        deallocate(CROSS,IUCMP,IICMP,IPVT)
    else ! NOT LIMC
        COR(1:size(CORS))=CORMF(1:size(CORS)) ! Using correction got within IB (Mean Force approximation)
    end if
    ! Limiting the change of the potential
    where (abs(COR).gt.DPOTM) COR=sign(DPOTM,COR)

        ! Print RDF, RDFref, Pot+Corr, Pot, Corr ! 
    call PrintPotential(NBond,NSType, NA,LIMC, irep, itercorr, FRT, FNR, REGP, &
        LPotCorrCheck,  CORS, IND, INDI,IndexGlobNB_riSjS,IndexGlobB_riBond,&
        NPointsPotB,RMinPotB,RMaxPotB,StateNB_iSjS, StateB_iBond,PotB_riBond,PotNB_riSjS,&
        NameSite,NameSType,&    
        BTypeBond,MTypeBond, NPairsTotalBond,iSiteinBond,jSiteinBond,kSiteinBond,&
        iPrint, stdout)        

    if ((.not. LPotCorrCheck).and.(ID.ne.1)) & ! Do not apply corrections if Check-call or if it is the master-process in MPI run.
!    if (.not. LPotCorrCheck) & 
        call ApplyCorrections(NSType, NBond, NA, StateNB_iSjS, StateB_iBond, NPointsPotB, IndexGlobNB_riSjS, IndexGlobB_riBond,&
            PotNB_riSjS, PotB_riBond, COR)        

    deallocate(CORMF); 
    if (allocated(DIFF)) deallocate(DIFF)
    
    end subroutine
    
subroutine ShiftIntraPotToZero(NBond,NPointsPotB,NMIN,PotB_riBond)
! Subroutine shifts every intramolecular potential in such a way that it has zero value in minimum.    
    use utilsmod
    implicit none
!Global Variables
    integer(4) NBond
    integer(4), allocatable:: NPointsPotB(:),NMIN(:)
    real(8),allocatable:: PotB_riBond(:,:)
!Local variables
    integer(4) NMINP(1),I,iBond, NP
    real(8) POMIN

    do iBond=1,NBond ! Shifting intramolecular potential to touch zero-level
        NP=NPointsPotB(iBond)
        nminp=minloc(PotB_riBond(1:NP,iBond))
        PotB_riBond(1:NP,iBond)=PotB_riBond(1:NP,iBond)-PotB_riBond(nminp(1),iBond)
        NMIN(iBond)=NMINP(1)+5*rand()-2.
        if(NMIN(iBond).lt.1) NMIN(iBond)=1
        if(NMIN(iBond).gt.NP) NMIN(iBond)=NP
    end do
end subroutine


end
