!====================== SETREFS =================================
subroutine SetReferences(NA, NMType, NMolMType, NSiteMType, NBondMType, &
    NMol, MTypeMol, XCOMMol, YCOMMol, ZCOMMol, &
    NSType, NSite, MTypeSite, IndexSTypeSite, ChargeSite, &
    NAtom, MTypeAtom, MolAtom, STypeAtom, SiteAtom, ChargeAtom, &
    NAtomChrgatom, AtomChrgatom,&
    AdrAtomMol, AdrMolMType, AdrSiteMType, AdrAtomMType, AdrBondMType,&
    NBond, StateNB_iSjS, StateB_iBond,  NPairsTotalBond, NMIN, NSiteGroupsInBond, &
    RDFref_NB_riSjS, PotNB_riSjS,& 
    NPointsRDFB, RMaxRDFB, RMinRDFB, &
    NPointsPotB, RMaxPotB, RMinPotB, &
    ! Setreferences-SetAngleBonds                   
    mt_ang_bonds, site_ang_bonds, num_site_ang_bonds, num_mt_ang_bond_types,site_ang_bond_edge,&
    BTypeBond,i_SiteInBond, j_SiteInBond, k_SiteInBond,&
    SX, SY, SZ, WX, WY, WZ,&
    LMoveMType, IROT, ITRANS)
        
! Set up reference arrays, addresses and units
! Global Variables
    integer NA, NMType, NMol, NMolMType(:), NSType, NSite, NAtom, NAtomChrgatom, NPointsNBGpot, NBond, IROT, ITRANS
    integer, allocatable :: AdrAtomMol(:), AdrMolMType(:), AdrSiteMType(:), AdrAtomMType(:),AdrBondMType(:), NBondMType(:), &
    MTypeMol(:), MTypeSite(:), IndexSTypeSite(:), NSiteMType(:),&
    MTypeAtom(:), MolAtom(:), STypeAtom(:), SiteAtom(:), &
    AtomChrgatom(:),  &
    StateNB_iSjS(:,:), StateB_iBond(:),  NPairsTotalBond(:), NSiteGroupsInBond(:),&
    NPointsRDFB(:), NPointsPotB(:), NMIN(:)!,&
    
    real(8), allocatable :: XCOMMol(:), YCOMMol(:), ZCOMMol(:),ChargeSite(:),ChargeAtom(:), SX(:),SY(:),SZ(:),WX(:),WY(:),WZ(:)

    real(8), allocatable:: RDFref_NB_riSjS(:,:,:) ,RMaxRDFB(:), RMinRDFB(:) !, RDFB_iBond(:), RDFA_iBond(:)
    real(8), allocatable:: PotNB_riSjS(:,:,:) ,RMaxPotB(:), RMinPotB(:) !, RDFB_iBond(:), RDFA_iBond(:)
    integer, allocatable:: i_SiteInBond(:,:), j_SiteInBond(:,:), k_SiteInBond(:,:),BTypeBond(:)
    integer(4), allocatable :: mt_ang_bonds(:,:,:,:),& ! I-th triplet of sites involved in covalent-angle bond of type IBB-NII (numbering excluding pair bonds) in molecular type ITYP
        site_ang_bonds(:,:,:),& !I-th dublet of sites involved in same covalent angle bond as site isite. Given site are sides of angle and isite is its edge
        num_site_ang_bonds(:),& !  number of covalent angle bonds which site isite involved in
        num_mt_ang_bond_types(:,:),& !  List of angle bond types in molecular type NMType. Last cell (NII_ang+1,NMType) keeps number of bond types, and first cells in a row keep numbers ov these types.
        site_ang_bond_edge(:,:) !  Site which stands on edge of covalent angle bond between sites isite,jsite
        logical, allocatable:: LMoveMType(:)
    ! Local variables    
    integer I,J,K, gbond, iNMType, IMOL, IAT, IS, IST, IM, JS, iBond

    call reshape(AdrMolMType,NMType+1)
    call reshape(AdrSiteMType,NMType+1)
    call reshape(AdrAtomMType,NMType+1)
    call reshape(AdrBondMType,NMType+1)
! Obtaining total number of  molecules in the system NMol
    NMol = sum(NMolMType(1:NMType))
! Allocate arrays having size of NMol
    call reshape(AdrAtomMol,NMol+1)
    call reshape(MTypeMol,NMol+1)
    call reshape(XCOMMol,NMol)
    call reshape(YCOMMol,NMol)
    call reshape(ZCOMMol,NMol)
    call reshape(MTypeSite,NSite)

!write(stdout,*)' number of atom types (NSType): ',NSType

    !Allocating arrays that depend on NSType
    call reshape2(StateNB_iSjS,NSType,NSType)
    call reshape(StateB_iBond, NBond)
    call reshape(NPairsTotalBond, NBond)    
    call reshape(NMIN, NBond)
!    if (.not. allocated(RDFref_NB_riSjS)) allocate(RDFref_NB_riSjS(NA,NSType,NSType))
!    if (.not. allocated(PotNB_riSjS)) allocate(PotNB_riSjS(NA,NSType,NSType))    
    call reshape(RMaxRDFB,NBond)
    call reshape(RMinRDFB,NBond)
    call reshape(NPointsRDFB,NBond)
    call reshape(RMaxPotB,NBond)
    call reshape(RMinPotB,NBond)
    call reshape(NPointsPotB,NBond)

    do i=1, NMType
        do j=sum(NBondMType(1:i-1))+1, sum(NBondMType(1:i))
            NPairsTotalBond(j)=NSiteGroupsInBond(j)*NMolMType(i)
        enddo
    enddo

    ! Counting total number of atoms NAtom
    NAtom=sum(NMolMType(1:NMType)*NSiteMType(1:NMType))
    ! Allocating arrays depend on NAtom
    call reshape(MTypeAtom,NAtom)
    call reshape(MolAtom,NAtom)
    call reshape(STypeAtom,NAtom)
    call reshape(SiteAtom,NAtom)
    call reshape(AtomChrgatom,NAtom)
    call reshape(ChargeAtom,NAtom)
    call reshape(SX,NAtom)
    call reshape(SY,NAtom)
    call reshape(SZ,NAtom)
    call reshape(WX,NAtom)
    call reshape(WY,NAtom)
    call reshape(WZ,NAtom)

    !  addresses
    AdrMolMType(1)=0
    AdrSiteMType(1)=0
    AdrAtomMType(1)=0
    AdrAtomMol(1)=0
    AdrBondMType(1)=0
    I=0 
    do iNMType=1,NMType	  
        I=I+NMolMType(iNMType)*NSiteMType(iNMType)
        AdrAtomMType(iNMType+1)=I                            !  atom addr
        AdrMolMType(iNMType+1)=AdrMolMType(iNMType)+NMolMType(iNMType)         !  molec adr
        AdrSiteMType(iNMType+1)=AdrSiteMType(iNMType)+NSiteMType(iNMType)       !  site addr
        AdrBondMType(iNMType+1)=AdrBondMType(iNMType)+NBondMType(iNMType)           !  bond addr
    end do
    if(IPRINT.ge.7)then
       write(stdout,*)' Addresses:'
       write(stdout,*)'          Typ     N    AdrMolMType'
       do I=1,NMType
            write(stdout,'(a,3I6)')' mol: ',I,NMolMType(I),AdrMolMType(I)
            write(stdout,'(a,3I6)')'  at: ',I,NMolMType(I)*NSiteMType(I),AdrAtomMType(I)
            write(stdout,'(a,3I6)')' sit: ',I,NSiteMType(I),AdrSiteMType(I)
       end do
    end if
! All references
IMOL=0
IAT=0
IS=0
IST=0
NAtomChrgatom = 0
if(IPRINT.ge.8)write(stdout,*)' References:'
if(IPRINT.ge.8)write(stdout,*) '  at    mol   site   typ  at_typ chrg'
do I=1,NMType
    do IM=1,NMolMType(I)
        IMOL=IMOL+1
        MTypeMol(IMOL)=I                       ! mol -> typ
        AdrAtomMol(IMOL)=IAT                   ! atom addr. in mol
        do IS=AdrSiteMType(I)+1,AdrSiteMType(I+1)
	    IAT=IAT+1
	    MTypeAtom(IAT)=I                   ! atom -> typ
	    MolAtom(IAT)=IMOL                 ! atom -> mol
	    STypeAtom(IAT)=IndexSTypeSite(IS)      ! atom -> site_type
	    SiteAtom(IAT)=IS                  ! atom -> site
            ChargeAtom(IAT)=ChargeSite(IS)
            if(ChargeAtom(IAT).ne.0.d0)then         ! registering charge atoms
                NAtomChrgatom = NAtomChrgatom+1
                AtomChrgatom(NAtomChrgatom)=IAT
            end if
	    if(IPRINT.ge.8)write(stdout,'(5i6,f7.4)') IAT,IMOL,IS,I,STypeAtom(IAT),ChargeAtom(IAT)
	end do
    end do
    do IS=AdrSiteMType(I)+1,AdrSiteMType(I+1)
        MTypeSite(IS)=I                        ! site -> typ
    end do
end do
write(stdout,*) 'Number of  charged atoms present in the system:',NAtomChrgatom

! Add content of setanglebonds_inc
! This section initialize lists of bonded atoms. This should make search of bonded atoms in MCstep and Energy subroutines much faster by extra cost of memory.

!Define maximum number of triplets in one bond type
if(NBOND.lt.1)then
  j=0
else
  j=MAXVAL(NSiteGroupsInBond(1:NBond),mask=BTypeBond(1:NBond).eq.2) ! Maximal number of atom groups
end if   
write(stdout,*) 'Maximum number of triplets in a single bond:',j
!Allocating memory for the arrays
if (allocated(mt_ang_bonds)) deallocate(mt_ang_bonds)
if (allocated(site_ang_bonds)) deallocate(site_ang_bonds)
if (allocated(num_site_ang_bonds)) deallocate(num_site_ang_bonds)
if (allocated(num_mt_ang_bond_types)) deallocate(num_mt_ang_bond_types)
if (allocated(site_ang_bond_edge)) deallocate(site_ang_bond_edge)
allocate(mt_ang_bonds(3,j,NBond,NMType))
allocate(site_ang_bonds(2,j,NSite))
allocate(num_site_ang_bonds(NSite))
allocate(num_mt_ang_bond_types(NBond+1,NMType))
allocate(site_ang_bond_edge(NSite,NSite))
mt_ang_bonds(:,:,:,:)=0
site_ang_bonds(:,:,:)=0
num_site_ang_bonds(:)=0
num_mt_ang_bond_types(:,:)=0
site_ang_bond_edge(:,:)=0
do iNMType=1, NMType ! Over all molecular types of the system
        write(stdout,*) 'Molecular type:',iNMType
        do iBond=1,NBond ! Loop over all bonds in the system            
            if ((MTypeSite(i_SiteInBond(iBond,1)).eq.iNMType).and.(BTypeBond(iBond).eq.2)) then ! atom sites in the bond belongs to the chosen moltype
                write(stdout,*) 'Bond:',iBond
                num_mt_ang_bond_types(NBond+1,iNMType)=num_mt_ang_bond_types(NBond+1,iNMType)+1 !Counter for number of bond types filled
                num_mt_ang_bond_types(num_mt_ang_bond_types(NBond+1,iNMType),iNMType)=iBond !Save bond type to the list
                do i=1,NSiteGroupsInBond(iBond)
                    write(stdout,*) 'Site triplet:',i,' of ',NSiteGroupsInBond(iBond)
                    write(stdout,*) I_SiteInBond(iBond,i),K_SiteInBond(iBond,i),J_SiteInBond(iBond,i)
                    mt_ang_bonds(1,i,iBond,iNMType)=I_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    mt_ang_bonds(2,i,iBond,iNMType)=J_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    mt_ang_bonds(3,i,iBond,iNMType)=K_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    num_site_ang_bonds(K_SiteInBond(iBond,i))=num_site_ang_bonds(K_SiteInBond(iBond,i))+1
                    site_ang_bonds(1,num_site_ang_bonds(K_SiteInBond(iBond,i)),K_SiteInBond(iBond,i))=&
                                I_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    site_ang_bonds(2,num_site_ang_bonds(K_SiteInBond(iBond,i)),K_SiteInBond(iBond,i))=&
                                J_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    if ((site_ang_bond_edge(I_SiteInBond(iBond,i),J_SiteInBond(iBond,i)).eq.0).and.&
                        (site_ang_bond_edge(J_SiteInBond(iBond,i),I_SiteInBond(iBond,i)).eq.0)) then
                        site_ang_bond_edge(I_SiteInBond(iBond,i),J_SiteInBond(iBond,i))=K_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                        site_ang_bond_edge(J_SiteInBond(iBond,i),I_SiteInBond(iBond,i))=K_SiteInBond(iBond,i)-AdrSiteMType(iNMType)
                    else
                        write(stdout,*) 'Pair of sites ',I_SiteInBond(iBond,i),J_SiteInBond(iBond,i),&
                                        ' already bonded with covalent angle bond. iBond=',iBond
                        stop
                    endif
                enddo
            endif
        enddo
enddo
!Writing output for self-check
do i=1,NSite ! Loop over all sites of the system
    write(stdout,*) 'Site:',I,' is involved in ',num_site_ang_bonds(i),' covalent angle bonds ',&
        (site_ang_bonds(1,j,i),site_ang_bonds(2,j,i),':', j=1,num_site_ang_bonds(i))
enddo

do iNMType=1,NMType ! over all molecular types
    write(stdout,*) 'Molecular type ',iNMType,' has ',num_mt_ang_bond_types(NBond+1,iNMType),'covalent angle bonds types'
    do i=1,num_mt_ang_bond_types(NBond+1,iNMType)
        iBond=num_mt_ang_bond_types(i,iNMType)
        write(stdout,*) 'Bond type ',iBond,' has ',NSiteGroupsInBond(iBond),'covalent angle bonds'
        do j=1,NSiteGroupsInBond(iBond)
            write(stdout,*) (mt_ang_bonds(k,j,iBond,iNMType), k=1,3)
        enddo
    enddo
enddo
! ---------------------------------

J=0
do iNMType=1, NMType ! over all molecular types
    if (LMoveMType(iNMType).and.(NSiteMType(iNMType).gt.1)) then 
        j=j+1
    endif
enddo
if (J.eq.0) then
    write(stdout,*) 'WARNING! There are no multiatom molecules allowed to move in the system.',&
' Molecular Rotation/Translation MC steps are disabled'
    IROT=0
    ITRANS=0
endif


end subroutine SetReferences
