subroutine PotReadTableCommon(text, iFilename, bTable, eTable, r, U, PNPoints, PRMAX, PRMIN)    
! Global Variables
    character(pot_text), allocatable :: text(:)
    character(filename_text) iFilename
    integer PNPoints, bTable, eTable
    real(8) PRMax, PRMin
    real(8), allocatable :: r(:), U(:)

! Local Variables
    integer  k,j,ierr
    character(pot_text) str
    real tmp_r, tmp_U
    ! Reading the potential table:
    ! First pass - check for data consistency: Count number of meaningful lines, check read errors 
    k=0
    do j=bTable+1, eTable-1
        call ReadLineFromText(text,j,str,ierr)
        k=k+1
        read(str,*,iostat=ierr) tmp_r, tmp_U
        if (ierr.ne.0) then
            write(stdout,*) 'ERROR: Reading Potential Table: Line ',j,':',str
            stop
        endif
!                r(k)=tmp_r
!                U(k)=tmp_U
    enddo
    if (k.ne.PNPoints) then
        write(stdout,*) 'ERROR: Reading Potential: actual number of lines in the table: ',k, &
        ' is not consistent with defined value of NPoints: ', PNPoints
        stop
    endif
    if (allocated(r)) deallocate(r,U)
    allocate(r(PNPoints))
    allocate(U(PNPoints))
   ! Second pass - reading  the potential-table
    k=0
    do j=bTable+1, eTable-1
        call ReadLineFromText(text,j,str,ierr)
        k=k+1
        read(str,*,iostat=ierr) tmp_r, tmp_U
        r(k)=tmp_r
        U(k)=tmp_U
        if ((r(k).gt.PRMAX).or.(r(k).lt.PRMIN)) then
            write(stdout,*) 'ERROR: Reading Potential Table: Line ',j,':',str
            write(stdout,*) 'The actual point ',r(k),' is beyond the range Min:Max, which is ',PRMIN,':',PRMAX
            stop
        endif
    enddo
    
    ! Check that R-values are within range MIN:MAX
    do j=1, size(r)
        if (r(j).gt.PRMAX) then
            write(stdout,*) 'ERROR: Reading Potential Table: Line ',j,':',str
            stop
        endif               
    enddo
end subroutine

