subroutine ReadGeneralSection(text,iend, N_NB, N_B, N_A, NPoints,RMax, RMin, eGeneral)
! Global variables
!    integer eGeneral
    integer N_NB, N_B, N_A, NPoints, iend
    character(pot_text), allocatable :: text(:)
    real(8)  RMax, RMin
! Local variables        
    integer i,j,k, ierr, iostt, bGeneral, eGeneral, cnt_bGeneral, cnt_eGeneral
    character(pot_text) l,str,field
    
    i=1
    bGeneral=0
    eGeneral=0
    cnt_bGeneral=0
    cnt_eGeneral=0
    do while (i.le.iend)
        call ReadLineFromText(text,i,l,ierr)
        ! Indicate presence of &General 
        if(uppercase(trim(l))=='&GENERAL') then
            bGeneral=i
            cnt_bGeneral=cnt_bGeneral+1
        endif
        if(uppercase(trim(l))=='&ENDGENERAL') then 
            eGeneral=i
            cnt_eGeneral=cnt_eGeneral+1
        endif
        i=i+1
    enddo
    ! Indicate presence of &EndGeneral and check consistency
    if ((cnt_bGeneral.gt.1) .or. (cnt_eGeneral.gt.1)) then
        write(stdout,*) 'ERROR: Reading Potential: more than one line with tag &General or &EndGEneral found '
        stop
    endif
    if (bGeneral.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: Can not find begin of section &General'
        stop
    endif
    if (eGeneral.eq.0) then
        write(stdout,*) 'ERROR: Reading Potential: Can not find end of section &EndGeneral'
        stop 
    endif
    if (bGeneral.gt.eGeneral) then
        write(stdout,*) 'ERROR: Reading Potential: The section &General starts at line ',bGeneral, 'and ends at line ',eGeneral
        stop
    endif
            
    ! Read fields
    ! Default values    
    N_NB=0
    N_B=0
    N_A=0
    RMax=0
    RMin=0
    NPoints=0
    write(stdout,*) 'Reading &General section of the potential file'
    do i=bGeneral,eGeneral
        call ReadLineFromText(text,i,str,ierr)
        l=str
        call split(l,'=',field)
        
        if (uppercase(trim(adjustl(field)))== 'N_NB') then
            call str2value(trim(adjustl(l)),N_NB,ierr)
            if (ierr==0) then
                write(stdout,*) 'Number of Non-bonded potentials to read:', N_NB
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of N_NB:',l
                write(stdout,*) str
                stop
            endif
        endif
        if (uppercase(trim(adjustl(field)))== 'N_B') then
            call str2value(trim(adjustl(l)),N_B,ierr)
            if (ierr==0) then
                write(stdout,*) 'Number of pairwise bonded potentials to read:', N_B
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of N_B:',l
                write(stdout,*) str
                stop
            endif
        endif
        if (uppercase(trim(adjustl(field)))== 'N_A') then
            call str2value(trim(adjustl(l)),N_A,ierr)
            if (ierr==0) then
                write(stdout,*) 'Number of angle-bending bonded potentials to read:', N_A
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of N_A:',l
                write(stdout,*) str
                stop
            endif
        endif
        if (uppercase(trim(adjustl(field)))== 'NPOINTS') then
            call str2value(trim(adjustl(l)),NPoints,ierr)
            if (ierr==0) then
                write(stdout,*) 'Number of points for every non-bonded potential:', NPoints
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of NPoints:',l
                write(stdout,*) str
                stop
            endif
        endif

        if (uppercase(trim(adjustl(field)))== 'MAX') then
            call str2value(trim(adjustl(l)),RMAX,ierr)
            if (ierr==0) then
                write(stdout,*) 'Cut off radius for non-bonded short range interactions:', RMAX
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of MAX:',l
                write(stdout,*) str
                stop
            endif
        endif
        if (uppercase(trim(adjustl(field)))== 'MIN') then
            call str2value(trim(adjustl(l)),RMIN,ierr)
            if (ierr==0) then
                write(stdout,*) 'Minimum distance (hard repulsive wall) for non-bonded short range interactions:', RMIN
            else
                write(stdout,*) 'ERROR: Reading Potential: line:',i,' Can not read the actual value of MIN:',l
                write(stdout,*) str
                stop
            endif
        endif
   enddo
   ! Check if all the necessary values were read
   if (N_NB.eq.0) then
       write(stdout,*) 'ERROR: Reading Potential:'
       write(stdout,*) 'No non-bonded potentials are stated to read'       
       stop
   endif
   if (NPoints.eq.0) then
       write(stdout,*) 'ERROR: Reading Potential:'
       write(stdout,*) 'Number of points for every non-bonded potential (NPOINTS) is not defined.'       
       stop
   endif
   if (RMAX.le.0.0000001) then
       write(stdout,*) 'ERROR: Reading Potential:'
       write(stdout,*) 'Cut off radius for non-bonded short range interactions (RMAX) is not defined'       
       stop
   endif
   if (RMIN.le.0.0000001) then
       write(stdout,*) 'Minimum distance (hard repulsive wall) for non-bonded',&
           'short range interactions (RMIN) is not defined or set to 0'       
       write(stdout,*) 'Default value of 0.0 will be used'
   endif
   if (N_B.eq.0) then
       write(stdout,*) 'WARNING: Reading Potential:'
       write(stdout,*) 'No pairwise bonded potentials are stated to read'
   endif
   if (N_A.eq.0) then
       write(stdout,*) 'WARNING: Reading Potential:'
       write(stdout,*) 'No angle-bending bonded potentials are stated to read'
   endif
end subroutine
