subroutine RDFReadGeneralSection(text,iend,NSType, NameSType, N_NB, N_B, N_A, NPoints,RMax, RMin)
! Read the General section of RDF-file and check that it is in agreement with what we got from mcm-file    
! Global variables
    integer N_NB, N_B, N_A, NPoints, iend, NSType
    character(rdf_text), allocatable :: text(:)
    character(atom_name_len), allocatable:: NameSType(:)
    real(8)  RMax, RMin
! Local variables        
    integer i,j,k, ierr, iostt, bGeneral,eGeneral, cnt_bGeneral, cnt_eGeneral, RDF_LengthNameSType, RDF_NSType
    character(rdf_text) l,str,field
    character(atom_name_len), allocatable:: RDF_NameSType(:)
    logical flag 
    
    i=1
    bGeneral=0
    eGeneral=0
    cnt_bGeneral=0
    cnt_eGeneral=0
    do while (i.le.iend)
        call ReadLineFromText(text,i,l,ierr)
        ! Indicate presence of &General 
        if(uppercase(trim(l))=='&GENERAL') then
            bGeneral=i
            cnt_bGeneral=cnt_bGeneral+1
        endif
        if(uppercase(trim(l))=='&ENDGENERAL') then 
            eGeneral=i
            cnt_eGeneral=cnt_eGeneral+1
        endif
        i=i+1
    enddo
    ! Indicate presence of &EndGeneral and check consistency
    if ((cnt_bGeneral.gt.1) .or. (cnt_eGeneral.gt.1)) then
        write(stdout,*) 'ERROR: Reading RDF: more than one line with tag &General or &EndGeneral found '
        stop
    endif
    if (bGeneral.eq.0) then
        write(stdout,*) 'ERROR: Reading RDF: Can not find begin of section &General'
        stop
    endif
    if (eGeneral.eq.0) then
        write(stdout,*) 'ERROR: Reading RDF: Can not find end of section &EndGeneral'
        stop 
    endif
    if (bGeneral.gt.eGeneral) then
        write(stdout,*) 'ERROR: Reading RDF: The section &General starts at line ',bGeneral, 'and ends at line ',eGeneral
        stop
    endif
            
    ! Read fields
    ! Default values    
    N_NB=0
    N_B=0
    N_A=0
    RMax=0
    RMin=0
    NPoints=0
    RDF_NSType=0
    RDF_LengthNameSType=0
    write(stdout,*) ''
    write(stdout,*) 'Reading &General section of the RDF file'
    do i=bGeneral,eGeneral
        call ReadLineFromText(text,i,str,ierr)
        l=str
        call split(l,'=',field)
        select case (uppercase(trim(adjustl(field))))
        case ('N_NB')
            call str2value(trim(adjustl(l)),N_NB,ierr)
            if (ierr==0) then
                write(stdout,*) '   Number of Non-bonded RDFs to read:', N_NB
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of N_NB:',l
                write(stdout,*) str
                stop
            endif        
        case('N_B')
            call str2value(trim(adjustl(l)),N_B,ierr)
            if (ierr==0) then
                write(stdout,*) '   Number of pairwise bond distributions  to read:', N_B
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of N_B:',l
                write(stdout,*) str
                stop
            endif        
        case('N_A')
            call str2value(trim(adjustl(l)),N_A,ierr)
            if (ierr==0) then
                write(stdout,*) '   Number of angle distributions to read:', N_A
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of N_A:',l
                write(stdout,*) str
                stop
            endif
        case('NPOINTS')
            call str2value(trim(adjustl(l)),NPoints,ierr)
            if (ierr==0) then
                write(stdout,*) '   Number of points for every intermolecular (non-bonded) RDF:', NPoints
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of NPoints:',l
                write(stdout,*) str
                stop
            endif
        case('MAX')
            call str2value(trim(adjustl(l)),RMAX,ierr)
            if (ierr==0) then
                write(stdout,*) '   Cut off radius for non-bonded short range interactions:', RMAX
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of MAX:',l
                write(stdout,*) str
                stop
            endif
        case('MIN')
            call str2value(trim(adjustl(l)),RMIN,ierr)
            if (ierr==0) then
                write(stdout,*) '   Minimum distance (hard repulsive wall) for non-bonded short range interactions:', RMIN
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of MIN:',l
                write(stdout,*) str
                stop
            endif
        case('NTYPES')
            call str2value(trim(adjustl(l)),RDF_NSType,ierr)
            if (ierr==0) then
                write(stdout,*) '   Number of atomic (site) types:', RDF_NSType
            else
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the actual value of NTypes:',l
                write(stdout,*) str
                stop
            endif
        case('TYPES')
            call removesp(l)
            call reshape(RDF_NameSType, NSType)
            call parse(l,',',RDF_NameSType,RDF_LengthNameSType)
            if (RDF_LengthNameSType.eq.0) then
                write(stdout,*) 'ERROR: Reading RDF: line:',i,' Can not read the list of atom/site types'
                write(stdout,*) str
                stop                
            endif
            if (RDF_LengthNameSType.ne.NSType) then
                write(stdout,*) 'ERROR: Reading RDF: line:',i,&
                ' Stated list of atom/site types differs from the declared number of site types',NSType
                write(stdout,*) str
                stop                
            endif
            ! compare types lists from mcmfile and from rdf
            do j=1, NSType
                flag=.false.
                do k=1, NSType
                    if (RDF_NameSType(j).eq.NameSType(k)) then
                        flag=.true.
                        exit
                    endif
                enddo
                if (.not. flag) then
                    write(stdout,*) 'ERROR: Reading RDF: line:',i
                    write(stdout,*) 'Atom/site type ',RDF_NameSType(j), 'which is stated in the RDF-file ',&
                    'does not exist in the list of atom/site types read from mcm-files'
                    stop
                endif
            enddo                                       
            write(stdout,*) '   Atomic (site) types read:', (RDF_NameSType(j), j=1, RDF_LengthNameSType)
        case default
                continue !write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
        end select
   enddo
   ! Check if all the necessary values were read
   if (N_NB.eq.0) then
       write(stdout,*) 'ERROR: Reading RDF:'
       write(stdout,*) 'No non-bonded RDFs are stated to read'       
       stop
   endif
   if (NPoints.eq.0) then
       write(stdout,*) 'ERROR: Reading RDF:'
       write(stdout,*) 'Number of points for every non-bonded RDF (NPOINTS) is not defined.'       
       stop
   endif
   if (RMAX.le.0.0000001) then
       write(stdout,*) 'ERROR: Reading RDF:'
       write(stdout,*) 'Cut off radius for non-bonded short range interactions ',& 
           '(RMAX) is not defined'       
       stop
   endif
   if (RMIN.le.0.0000001) then
       write(stdout,*) '   Minimum distance (hard repulsive wall) for non-bonded',&
           ' short range interactions (RMIN) is not defined or set to 0'       
       write(stdout,*) 'Default value of 0.0 will be used'
   endif
   if (RDF_NSType.eq.0) then
       write(stdout,*) 'ERROR: Reading RDF:'
       write(stdout,*) 'Number of atom/site types (NTypes) is not defined.'       
       stop
   endif
   if (RDF_LengthNameSType.eq.0) then
       write(stdout,*) 'ERROR: Reading RDF:'
       write(stdout,*) 'List of atom/site types (Types) is not defined'       
       stop
   endif

   if (N_B.eq.0) then
       write(stdout,*) 'WARNING: Reading RDF:'
       write(stdout,*) 'No pairwise bonded RDFs are stated to read'
   endif
   if (N_A.eq.0) then
       write(stdout,*) 'WARNING: Reading RDF:'
       write(stdout,*) 'No angle-bending bonded RDFs are stated to read'
   endif
end subroutine
