subroutine RDFReadPropA(text, iFileName, bRDF, eRDF, NameMType, MolType,iBond, NBondMType,AdrSiteMType, PRmin, PRmax, PNpoints,&
    Bond_iSjS, NSiteGroupsInBond, i_SiteInBond, j_SiteInBond, k_SiteInBond, iprint, stdout)
    ! Global Variables
    character(rdf_text), allocatable :: text(:)
    character(filename_text) iFilename
    character(filename_text) NameMType(:), NameMolType    
    integer, allocatable:: NBondMType(:), AdrSiteMType(:)
    integer bRDF, eRDF, PNPoints, bTable, eTable,  MolType, iBond, iprint, stdout
    integer, allocatable:: NSiteGroupsInBond(:), i_SiteInBond(:,:), j_SiteInBond(:,:), k_SiteInBond(:,:),Bond_iSjS(:,:)
    real(8) PRmin, PRmax

! Local Variables
    integer  i,j,ierr, tmpNPairs, tmp, PNPairs, BondNumber
    integer, allocatable:: AtomTriplets(:,:)
    character(rdf_text) PairStr(3),field, l, str
    character(rdf_text) PairsStr(rdf_text)
    logical flag

    BondNumber=0
    NameMolType=''
    do j=bRDF, eRDF ! Read properties which are specific to RDF-type 
        call ReadLineFromText(text,j,str,ierr)
        l=str
        call split(l,'=',field)
        select case (uppercase(trim(adjustl(field))))
            case ('NTRIPLETS')
                call str2value(trim(adjustl(l)),PNPairs,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),&
                    ' line:',j,' Can not read the actual value of NTriplets:',l
                    write(stdout,*) str
                    stop                
                endif
            case ('TRIPLETS')   
                call removesp(l)
                call parse(l,',',PairsStr,tmpNPairs)
                if (tmpNPairs.eq.0) then
                    write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' line:',j,&
                    ' Can not read the list of atom Triplets involved in angle-distribution'
                    write(stdout,*) str
                    stop                
                endif
                if (tmpNPairs.ne.PNpairs) then
                    write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),&
                    ' line:',j,' Declared number of atomic Triplets NTriplets ',PNpairs,&
                    ' differs from actual number of stated triplets ',tmpNPairs
                    write(stdout,*) str
                    stop                
                endif
                if (allocated(AtomTriplets)) deallocate(AtomTriplets)
                allocate(AtomTriplets(tmpNPairs,3))
                do i=1, tmpNPairs
                    l=PairsStr(i)
                    call removesp(l)
                    call parse(l,'-',PairStr,tmp)
                    if (tmp.ne.3) then 
                        write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),&
                        ' line:',j,' Can not read the atom numbers of the triplet ',i,&
                        ' involved in the bond angle distribution'
                        write(stdout,*) l
                        stop
                    endif
                    call str2value(trim(adjustl(PairStr(1))),AtomTriplets(i,1),ierr)
                    call str2value(trim(adjustl(PairStr(2))),AtomTriplets(i,2),ierr)
                    call str2value(trim(adjustl(PairStr(3))),AtomTriplets(i,3),ierr)
                    if (ierr.ne.0) then
                        write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' line:',j,&
                                   ' Can not read the atomic pair included in bond angle distribution.',l
                        write(stdout,*) '"',trim(adjustl(PairStr(1))),'" ',' "',trim(adjustl(PairStr(2))),'"',&           
                                    ' "',trim(adjustl(PairStr(3))),'"'
                        stop
                    endif
                enddo
            case ('MOLTYPE')   
                call removesp(l)
                NameMolType=adjustl(trim(l))
                flag=.false.
                do i=1, size(NameMType)
                    if (adjustl(trim(NameMolType)).eq.adjustl(trim(NameMType(i)))) then 
                        flag=.true.
                        MolType=i
                    endif
                enddo
                if (.not. flag) then
                    write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' line:',j, ' unknown Molecular type ',NameMolType
                    stop
                endif
            case ('BONDNUMBER')
                call str2value(trim(adjustl(l)),BondNumber,ierr)
                if (ierr.ne.0) then
                    write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' line:',j, &
                            ' Can not read the actual number of the bond ',l
                    write(stdout,*) str
                    stop                
                endif
            case default
                continue !write(stdout,*) 'Debug: Unknown line:',trim(adjustl(str))
        end select
    enddo
    if (BondNumber.eq.0) then
        write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' Lines:',bRDF,':', eRDF, ' BondNumber is not stated' 
        stop
    endif
    if (adjustl(trim(NameMolType)).eq.'') then
        write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' Lines:',bRDF,':', eRDF, ' Molecular type name is not stated' 
        stop
    endif
    iBond=sum(NBondMType(1:MolType-1))+BondNumber    
    if (PNPairs.ne.NSiteGroupsInBond(iBond)) then
        write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),&
        ' Bond distribution for bond ', BondNumber, ' of MolType ', trim(NameMType(MolType))
        write(stdout,*) 'Number of site/atom groups defined for the bond: ',PNPairs, ' differs from'
        write(stdout,*) 'number of site/atom groups defined in the mcm file : ', NSiteGroupsInBond(iBond)
        stop
    endif
        
    do i=1, PNPairs ! Pairs stated in the RDF-file        
        flag=.false.
        if (Bond_iSjS(AtomTriplets(i,1)+AdrSiteMType(MolType),AtomTriplets(i,3)+AdrSiteMType(MolType)).ne.iBond) then
            write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),' Bond-angle distribution for bond '&
                    , BondNumber, 'of MolType ', trim(NameMType(MolType))
            write(stdout,*)'According to the molecular description in mcm-file, site triplet (1-2-3):',&
                AtomTriplets(i,1),AtomTriplets(i,2),&
                AtomTriplets(i,3),'does not belong to the bond ',BondNumber
            write(stdout,*) Bond_iSjS(AtomTriplets(i,1)+AdrSiteMType(MolType),AtomTriplets(i,3)+AdrSiteMType(MolType)), iBond
            stop            
        endif
        do j=1, NSiteGroupsInBond(iBond) ! Pairs stated in mcm-file
            if (AtomTriplets(i,1)+AdrSiteMType(MolType).eq.i_SiteInBond(iBond,j).and.&
                AtomTriplets(i,3)+AdrSiteMType(MolType).eq.j_SiteInBond(iBond,j).and.&
                AtomTriplets(i,2)+AdrSiteMType(MolType).eq.k_SiteInBond(iBond,j)) &
                then
                flag=.true.
                exit
            endif
        enddo
        if (.not.flag) then
            write(stdout,*) 'ERROR: Reading RDF: File: ',trim(iFilename),&
            ' Angle distribution for bond ', BondNumber, 'of MolType ', trim(NameMType(MolType))
            write(stdout,*) 'Can not find ',i,' group of sites/atoms defining the bond in the mcm file : ', AtomTriplets(i,1:3)
                    stop
        endif
    enddo
end subroutine
