! CollectMCsummary - subroutine collects statistic data from all processes with MPI, and the calculates final averaged values.
! PrintMCsummary - subroutine prints final output in Molsim (priorly collected )

module magicmod
    implicit none
    contains
    
    subroutine InitMagic(psize, rank, ID)    
        implicit none
        integer(4) psize, rank, ID
#ifdef mpi
    include 'mpif.h'
#endif    
        integer ierror
        character*64 LABEL,DATE*8 ! Labels for program version and date (not used)

    !Init MPI, detecting number of processes and the process ID
#ifdef mpi
        call MPI_INIT(ierror)
        call MPI_COMM_SIZE(MPI_COMM_WORLD,psize,ierror)
        call MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierror)
#endif
        if (psize.ge.1) ID=ID+1+rank ! Set ID if in parallel mode
        if ((psize.ge.1).and.(rank.eq.0)) then
            write((6),*) 'Running in parallel mode on ',psize,' processes'
        elseif ( psize .eq. 0) then
            write((6),*) 'Running in serial mode'
        endif

     ! Version identification
        LABEL='MagiC_v.2.0.2'
        DATE='13.01.16'
        if ((rank.eq.0)) then
            write((6),*)'======================================'
            write((6),*)' Molecular Simulation Program MagiC '
            write((6),*)'  General output '
            write((6),'(4a)')'  Version ',LABEL
            write((6),'(2a)')'  from ',DATE
            write((6),*)'======================================'
            write((6),*) 'Thank',"'",'s for citing the software paper:'
            write((6),*)'A. Mirzoev, A.P.Lyubartsev, MagiC: Software Package for Multiscale Modeling'
            write((6),*)'J. Chem. Theory and Computations, 9(3), 1512-1520 (2013) . DOI: 10.1021/ct301019v'
            write((6),*)'======================================'
#ifdef PBC_MULT
            write((6),*)' PBC_Mult enabled ... '
#endif        
            write((6),*)' starting ... '
        endif    

    end subroutine

    subroutine ResetCounters(stdout, NAV, IUD, ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES, CORS, CORP,npairsrdf)
        implicit none
        integer(4) NAV, stdout
        integer(8) IUD(3)
        integer(8), allocatable:: npairsrdf(:,:), CORS(:), CORP(:,:)
        real(8) ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES
        !  New run, summators=0
            write((stdout),*) 'Set all counters to zero'
            NAV=0
            IUD(:)=0
            ENERS=0.d0
            EELS=0.
            EINTS=0.
            VIRS=0.
            VIIS=0.
            VIRE=0.
            PRES=0.
            CORS(:)=0.d0
            if (allocated(CORP)) CORP(:,:)=0.d0
            npairsrdf(:,:)=0

    end subroutine

    !===================== READCRD ==================================
    ! read initial coordinates and eventually velocities from a "xmol" file s
    subroutine READCRD(SX,SY,SZ,NAtom,SiteAtom,FSTART,ID, stdout)
        implicit none
    ! Global Variables    
        real(8) ::SX(:),SY(:),SZ(:)
        integer(4) NAtom,ID, stdout
        integer(4)::SiteAtom(:)
        character*64 FSTART    
    ! Local Variables
        character*64 LoadFileName ! Local variable to work with, without changing global FSTART
        character*80 CHR
        integer(4) i,iostat,IS,NPP

        LoadFileName=FSTART
        write((6+ID),*) ' Read starting geometry from file ',LoadFileName
        if (ID.ne.0) then
            LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+2)='.p'
            write(LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+3),'(i0.3)') ID
        endif
        LoadFileName(Len_Trim(LoadFileName)+1:Len_Trim(LoadFileName)+11)='.start.xmol'
        open(unit=4,file=LoadFileName,status='old',iostat=iostat)
        if (iostat.ne.0) then
            if (ID.gt.0) write(stdout,*) ' Problems reading file ',trim(LoadFileName),' on process ID:',ID
            write(6,*) ' Problems reading file ',trim(LoadFileName)
            stop
        else
            write((6+ID),*) ' The coordinates were successfully read from file ',LoadFileName
        endif
        read(4,*)NPP
        if(NPP.ne.NAtom)then        
            write(stdout,*) ' wrong N mol. in CRD file ',LoadFileName,' Sholud be:',NAtom,' In file:',NPP,' ID:',ID        
            stop
        end if
        read(4,*)CHR
        do I=1,NAtom
            read(4,*)CHR,SX(I),SY(I),SZ(I)
        end do
        write(stdout,*)' Initial coord.  read from ',LoadFileName
        close(4)
    end subroutine READCRD

    subroutine InitGeometry(LCRD, FSTART, FCRD, LCRDPASS, LMoveMType, ID, stdout, iPrint, iREP, NAtom,&
        NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,NameMType,&
        X, Y, Z, SX,SY,SZ, WX, WY,WZ, RSite,BOXL, BOYL, BOZL)

        use precision
        implicit none
        ! Global
        logical LCRD,  LCRDPASS, LMoveMType(:)
        integer(4)  ID, stdout, iPrint, iREP, NAtom, NSite
        character(filename_text) FSTART, FCRD,NameMType(:)
        integer(4) NMType, NMolMType(:), NSiteMType(:),SiteAtom(:), AdrAtomMType(:), AdrMolMType(:),AdrAtomMol(:)
        real(8) X(:), Y(:), Z(:), SX(:),SY(:),SZ(:), WX(:), WY(:),WZ(:), RSite(:,:),BOXL, BOYL, BOZL
        !Local

        !  Preparing initial state geometry

        write(stdout,*) '' 
        write(stdout,*) 'Prepare starting geometry' 

        if (IREP.eq.1) then  ! The first iteration
            write(stdout,*) 'The first inverse iteration'
            if (LCRD) then
                write(stdout,*) 'Reading initial geometry from file' 
                call READCRD(SX,SY,SZ,NAtom,SiteAtom,FSTART,ID,stdout)
                LCRDPass=.true. !By default, if LCRD=.true., LCRDPAss=.true. as well
            else            
                write(stdout,*) 'Randomly generate initial geometry' 
                call GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,&
                    LMoveMType, NameMType,&
                        X, Y, Z, SX,SY,SZ, WX, WY,WZ, RSite,BOXL, BOYL, BOZL, FCRD, iPrint, stdout )
            endif
        else ! Not the first iteration
            if (.not. LCRDPASS) then
                write(stdout,*) 'Randomly generate initial geometry' 
                call GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,&
                    LMoveMType, NameMType,&
                        X, Y, Z, SX,SY,SZ, WX, WY,WZ, RSite, BOXL, BOYL, BOZL,FCRD, iPrint, stdout )
            else
                write(stdout,*) 'Continue with the existing geometry of the system'             
            endif
        endif        
    end subroutine

    subroutine GenerateGeometry(NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType, AdrAtomMol,LMoveMType,&
        NameMType, X, Y, Z, SX,SY,SZ, WX, WY,WZ, RSite, BOXL, BOYL, BOZL, FCRD, iPrint, stdout)
        use utilsmod
        use precision
        implicit none
        ! Global
        integer(4) NMType, NMolMType(:), NSiteMType(:),SiteAtom(:), AdrAtomMType(:), AdrMolMType(:),AdrAtomMol(:), iPrint, stdout
        logical LMoveMType(:)
        character(filename_text) NameMType(:), FCRD    
        real(8) X(:), Y(:), Z(:), SX(:),SY(:),SZ(:), WX(:), WY(:),WZ(:), RSite(:,:),BOXL, BOYL, BOZL
        !Local
        integer(4) iMType, iostt, IC, iMol, iAtom, im ,is, isp
        real(8) QP(4)        ! quaternion array
        character(atom_name_len) atomname

    ! If there are fixed molecules:
        if (.not. all(LMoveMType)) then
            write(stdout,*) 'There are frozen molecules in the system, their positions will be read from file: ', trim(FCRD)
            open (unit=3,file=FCRD,status='old',iostat=iostt)
            if (iostt.ne.0) stop 'The coordinate file not found'
            read(3,*)
            read(3,*)
        endif

        do iMType=1,NMType
            if(.not.LMoveMType(iMType)) then  !  fixed coordinates (e.g.DNA)        
                IC=AdrAtomMType(iMType)
                do iMol=1,NMolMType(iMType)
                    do iAtom=1,NSiteMType(iMol)
                        IC=IC+1                            
                        read(3,*) atomname,SX(IC),SY(IC),SZ(IC)
                    end do
                end do
            else
                write(stdout,*)' Generate random distribution of molecules ',trim(NameMType(iMType))
    !  random distribution for COM particles
                do iMol=1,NMolMType(iMType)
                    IM=AdrMolMType(iMType)+iMol
                    X(IM)=BOXL*(rand()-0.5d0)
                    Y(IM)=BOYL*(rand()-0.5d0)
                    Z(IM)=BOZL*(rand()-0.5d0)
                    if(IPRINT.ge.7)write(stdout,'(I6,3F12.6)')IM,X(IM),Y(IM),Z(IM)
    ! random rotation - using quaternion
                    call RANQ(QP)
                    if(IPRINT.ge.7)write(stdout,'(4F12.6)') QP(1),QP(2),QP(3),QP(4)                                
                    do iatom=1,NSiteMType(iMType)
                        ISP=AdrAtomMol(IM)+iAtom
                        IS=SiteAtom(ISP)
                        WX(ISP)=RSite(IS,1)
                        WY(ISP)=RSite(IS,2)
                        WZ(ISP)=RSite(IS,3)
                        call ROTATE(QP,WX(ISP),WY(ISP),WZ(ISP)) ! This one rotates molecule on quaternion QP
                        SX(ISP)=X(IM)+WX(ISP)
                        SY(ISP)=Y(IM)+WY(ISP)
                        SZ(ISP)=Z(IM)+WZ(ISP)
                        if(IPRINT.ge.7)write(stdout,'(3I6,3F12.4)') IM,IS,ISP,SX(ISP),SY(ISP),SZ(ISP)
                    end do
                end do
            end if  ! LMoveMType
        end do  ! NMType
        if (.not. all(LMoveMType)) close(3)
    end subroutine
    
    subroutine TerminateMagiC(PotCorrJourn,SumMassMType,PotNB_riSjS,PotB_riBond,&
            X,Y,Z,RSite,SX,SY,SZ, WX,WY,WZ, MassSite, RMN,RMX,&
            NSiteMType,AdrBondMType,AdrSiteMType,AdrMolMType,AdrAtomMType,AdrAtomMol,&
            NBondMType,Bond_iSjS,NPairsTotalBond,NPTT, IndexGlobNB_riSjS,IndexGlobB_riBond,IND,INDI,MTypeSType, MTypeMol, ITS,&
            iSiteinBond,jSiteinBond, kSiteinBond,NAI,NMIN, ITYPE, NNUM,STypeAtom, SiteAtom, ICHRG,&
            CORS,CORP,npairsrdf,global_npairsrdf,global_CORP, global_CORS,& 
            NameSType, NameSite,ID)
        use precision
        implicit none
#ifdef mpi
        include 'mpif.h'
#endif        
        
        real(8), allocatable:: PotCorrJourn(:,:),SumMassMType(:),PotNB_riSjS(:,:,:),PotB_riBond(:,:),&
            X(:),Y(:),Z(:),RSite(:,:),SX(:),SY(:),SZ(:),WX(:),WY(:),WZ(:),&
            MassSite(:), RMN(:),RMX(:)
        integer(4), allocatable :: NSiteMType(:),AdrBondMType(:),AdrSiteMType(:),AdrMolMType(:),AdrAtomMType(:),AdrAtomMol(:),&
            NBondMType(:),Bond_iSjS(:,:),NPairsTotalBond(:),NPTT(:), &
            IndexGlobNB_riSjS(:,:,:),IndexGlobB_riBond(:,:),IND(:,:,:),INDI(:,:),MTypeSType(:), MTypeMol(:), ITS(:),&
            iSiteinBond(:,:),jSiteinBond(:,:), kSiteinBond(:,:),NAI(:),NMIN(:), ITYPE(:), NNUM(:),&
            STypeAtom(:), SiteAtom(:), ICHRG(:)
        integer(8), allocatable:: CORS(:),CORP(:,:),npairsrdf(:,:),global_npairsrdf(:,:),global_CORP(:,:), global_CORS(:) 
        
        character(atom_name_len), allocatable:: NameSType(:), NameSite(:)
        integer(4) ierror, ID
        
        if (allocated(PotCorrJourn)) deallocate(PotCorrJourn)
        deallocate(NSiteMType,AdrSiteMType,AdrMolMType,AdrAtomMType,NBondMType,AdrBondMType,SumMassMType)
        deallocate(PotNB_riSjS,IndexGlobNB_riSjS,IND,NameSType,MTypeSType)
        !deallocate(SHELV,RAS)
        deallocate(AdrAtomMol,MTypeMol,X,Y,Z) ! Depends on total number of molecules (former NPART)
        deallocate(RSite,NameSite,MassSite)
        if(allocated(Bond_iSjS)) deallocate (Bond_iSjS)
        if(allocated(NPairsTotalBond)) deallocate(NPairsTotalBond) 
        deallocate(ITS)
        if(allocated(NPTT))deallocate(NPTT)
        if(allocated(iSiteinBond)) deallocate(iSiteinBond)
        if(allocated(jSiteinBond)) deallocate(jSiteinBond)
        if(allocated(kSiteinBond)) deallocate(kSiteinBond)
        deallocate(RMN,RMX,NAI,NMIN) ! Depends on number of intramolecular potentials (former NSPAR)
        if(allocated(IndexGlobB_riBond))deallocate(IndexGlobB_riBond)
        if(allocated(PotB_riBond)) deallocate(PotB_riBond)
        if(allocated(INDI)) deallocate(INDI)
        deallocate(CORS)
        if (allocated(CORP)) deallocate(CORP)
        if (ID.eq.1) then  ! If in parallel mode on master process
            deallocate(global_CORS)
            deallocate(global_npairsrdf)
            if (allocated(global_corp)) deallocate(global_CORP)
        endif
        deallocate(ITYPE,NNUM,STypeAtom,SiteAtom,ICHRG) ! Depends on total number of atoms (former NTOT)
        deallocate(SX,SY,SZ,WX,WY,WZ)!!Depends on total number of atoms (former NTOT)
        deallocate(npairsrdf)
        if (ID.ne.0) then
            close(6+ID)
        endif
#ifdef mpi
        call MPI_BARRIER(MPI_COMM_WORLD,ierror)
        call MPI_FINALIZE(ierror)
#endif
    end subroutine TerminateMagiC
    
!
!=================== Collect =====================================
!   Subroutine collecting data from different processes using MPI
subroutine Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP,&
	npairsrdf,global_npairsrdf,LIMC)
use dynarray

!-------Global Variables--------------------
integer(4) ID, ierror,rank,psize,NAV,global_NAV
integer(8), allocatable:: CORS(:),CORP(:,:),global_CORS(:),global_CORP(:,:),&
	npairsrdf(:,:),global_npairsrdf(:,:)
logical LIMC
!-------Local Variables--------------------
integer(4) i,j,k
#ifdef mpi
include 'mpif.h'

if (ID.eq.1) then  ! If in parallel mode on master process
        call reshape(global_CORS,size(CORS))
        if (LIMC) call reshape2(global_CORP,size(CORP,1),size(CORP,2))
        call reshape2(global_npairsrdf,size(npairsrdf,1),size(npairsrdf,2))
    endif

#ifdef intel
    if (ID.gt.1) then  ! If in parallel mode, allocate memory on all processes. for intel compilers
        call reshape(global_CORS,size(CORS))
        if (LIMC) call reshape2(global_CORP,size(CORP,1),size(CORP,2))
        call reshape2(global_npairsrdf,size(npairsrdf,1),size(npairsrdf,2))
    endif
#endif        

    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Gathering CORS - self-correlated terms
!   write(ID+6,*) 'AllGather CORS started'
    call MPI_REDUCE(CORS,global_CORS,size(CORS),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
 !   write(ID+6,*) 'AllGather CORS done'
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Gathering CORP - cross-correlated terms
    if (LIMC) then ! If IMC mode, collect cross correlated terms
        call MPI_REDUCE(CORP,global_CORP,size(CORP),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    endif
! Gathering number of counted pairs in RDF calc
    call MPI_REDUCE(npairsrdf,global_npairsrdf,size(npairsrdf),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(NAV,global_NAV,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    if (ID.eq.1) then
        if (global_NAV.ne.(NAV*psize)) write(*,*) 'MPI ERROR in NAV - reduction', global_NAV,'!=',NAV*psize
    endif
#endif
end subroutine

#ifdef mpi
Subroutine CollectMCsummary(NAV,global_NAV,NMKSF,global_NMKSF,IUD,global_IUD,EELS,global_EELS,EINTS,global_EINTS,&
            ENERS,global_ENERS,VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE,LRDF,ID,psize,ierror)
    include 'mpif.h'
!Global variables
    integer(4) NAV,global_NAV,&                
                psize,ierror,ID
    integer(8) IUD(:),global_IUD(:),NMKSF,global_NMKSF
    real(8) EELS,global_EELS,&
            EINTS,global_EINTS,&
            ENERS,global_ENERS,&
            VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE
    logical LRDF
! Local variables
    integer(4) i
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
! Collect NAV
    call MPI_REDUCE(NAV,global_NAV,1,MPI_INTEGER,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect NMKSF
    call MPI_REDUCE(NMKSF,global_NMKSF,1,MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect IUD(:)
    call MPI_REDUCE(IUD(:),global_IUD(:),size(IUD),MPI_INTEGER8,MPI_SUM,0,MPI_COMM_WORLD,ierror)    
! Collect EELS
    call MPI_REDUCE(EELS,global_EELS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect EINTS
    call MPI_REDUCE(EINTS,global_EINTS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect ENERS
    call MPI_REDUCE(ENERS,global_ENERS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
! Collect PRESSURE PRES=NSTOT*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*FNR
    call MPI_REDUCE(VIRS,global_VIRS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIIS,global_VIIS,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)
    call MPI_REDUCE(VIRE,global_VIRE,1,MPI_REAL8,MPI_SUM,0,MPI_COMM_WORLD,ierror)

    if (ID.eq.1) then ! Calculate mean values and write debug/perform check
        if (global_NAV.ne.(NAV*psize)) write(*,*) 'MPI ERROR in NAV - reduction', global_NAV,'!=',NAV*psize
        if (global_NMKSF.ne.(psize*NMKSF))&
            write(*,*)'MPI ERROR in NMKSF-reduction, global_NMKSF!=NMKSF*psize', global_NMKSF,'!=',NMKSF*psize
    endif
end subroutine
#endif


! This subroutine prints final output in Molsim (priorly collected )
subroutine PrintMCsummary(ID,NAV,NMType,NOP,NMKSF,NMKS0,NSTOT,ITRANS,IROT,NMolMType,IUD,&
        BOXL,BOYL,BOZL,DR,TEMP,EPS,EELS,FKJM,EINTS,&
        ENERS,PRES,VIRS,VIRE,VIIS,FPR,LRDF,NameMType,FILRDF,FILPOT)
!Global variables
    integer(4) ID,NAV,NMType,NOP,NSTOT,ITRANS,IROT
    integer(4) NMolMType(:)
    integer(8) IUD(:),NMKSF,NMKS0
    real(8) BOXL,BOYL,BOZL,DR,TEMP,EPS,EELS,FKJM,EINTS,ENERS,PRES,VIRS,VIRE,VIIS,FPR
            
    logical LRDF    
    character*64 NameMType(:),FILRDF,FILPOT
! Local variables
    real(8) FNR
    integer(4) I 

    write((ID+6),*)'MagiC '
        write((ID+6),*)' Monte Carlo'

    write((ID+6),*)'-----------------------------------'
    if(LRDF)then
        write((ID+6),*)' Inverse simulation'
        write((ID+6),*)' File with original RDF:',FILRDF
    else
        write((ID+6),*)' Direct simulation'
        write((ID+6),*)' Potential is read from ',FILPOT
    end if
    if(NAV.gt.0)then
        FNR=1./dfloat(NAV)
    else
        FNR=0.
    end if
    write((ID+6),*)'Number of molecules:   ',NOP
    write((ID+6),*)'Number of species:     ',NMType
    write((ID+6),'(a,3f12.3)')' Box sizes:            ',BOXL,BOYL,BOZL
    write((ID+6),*)'Sampled configurations:',NAV
    write((ID+6),*)'Number of molecules of each species:'
    do I=1,NMType ! TODO simplify
        write((ID+6),'(i4,i6,3x,a,f6.2)')I,NMolMType(I),NameMType(I)
    end do
    write((ID+6),*)
    write((ID+6),*)'Total MC steps:        ',NMKSF
    write((ID+6),*)'MC steps with averaging',NMKSF-NMKS0
    write((ID+6),*)'Acceptance ratio atoms  ',dfloat(IUD(1))/dfloat(NMKSF)
    write((ID+6),*)'Acceptance ratio trans  ',dfloat(IUD(2))*Itrans/dfloat(NMKSF)
    write((ID+6),*)'Acceptance ratio rot    ',dfloat(IUD(3))*Irot/dfloat(NMKSF)
    write((ID+6),*)'Max. displacemet       ',DR
    write((ID+6),*)'Temperature            ',TEMP
    write((ID+6),*)'Dielectric permitivity ',EPS
    write((ID+6),*)'Electrostatic energy:  ',EELS*FKJM*FNR
    write((ID+6),*)'Bond energy:           ',EINTS*FKJM*FNR
    write((ID+6),*)'------------------------------------'
    write((ID+6),*)'Average energy:        ',ENERS*FKJM*FNR
    PRES=NSTOT*FPR-((VIRS+VIIS+VIRE)/3.)*FPR*FNR
    write((ID+6),*)'Pressure                ',PRES
    write((ID+6),'(a,4f12.2)')'Pressure components    ',&
             -VIRS*FPR*FNR/3.,-VIRE*FPR*FNR/3.,-VIIS*FPR*FNR/3.,NSTOT*FPR
! Here I can add printing of RDF for non inverse run.
end subroutine PrintMCsummary
    
end module
