program MAGIC
!*
!*  Monte Carlo - direct and inverse
!*!*
!*  Any pair interaction potentials
!*
!*  Construction of effective potentials for arbitrary molecular
!*  systems from given RDF
!*
!*  Units: distances A
!*         velocity 10^5 m/s = A/fs
!*         time fs        
!*
!*  Alexander Lyubartsev      sasha@physc.su.se
!*  Alexander Mirzoev alexander.mirzoev@mmk.su.se
!*
!*  The main module
!*
!DEC$ FREEFORM
use precision
use modforces
use dynarray
use utilsmod
use iomod 
use imcmod
use magicmod
implicit none

! Subroutine Interfaces
#ifdef mpi
include 'mpif.h'
#endif
include "molsim.h"
include "interface_molsim_inc.f"


! Initialization
call InitMagic(psize, rank, ID)     

! Here all input files are read, and default parameters are set      
call ReadAll(&
! ReadInput 
           NMType, NameMType, NMolMType, NA, LMoveMType, LRDF,LPOT,&            
           NMKS, NMKS0, DR, MCTRANSSTEP, MCROTSTEP, ITRANS, IROT,&
           RECUT, NRS, POTCUT,AF, FQ, IOUT, &
           IAV, REGP, DPOTM, LCRD, LCRDPASS, RTM, IREPT, LIMC, IPOTCORRCHECK, LPotCorrCheck,PotCorrCheckLevel, &
           LEV, BaseOutFileName, FSTART,FCRD, LXMOL, ITR,&                                                                            
           VOL, BOXL, BOYL, BOZL, FRT, TEMP, FKJM, FPR, COULF, EPS, IAV, &
!               
           NSiteMType, NBondMType, SumMassMType, NDAT,NameSite,ITS, RSite, MassSite, CHS, & 
           NSType, MTypeSType, NameSType,&
           IRDF, NII, NII_ANG, Bond_iSjS, iSiteinBond, jSiteinBond, kSiteinBond, NPTT, BTypeBond,MTypeBond, &
! Set references
           NOP,MTypeMol,X,Y,Z, NSTOT, ITYPE, NNUM, STypeAtom, SiteAtom, Q, NCHRG, ICHRG, &
           AdrAtomMol, AdrMolMType, AdrSiteMType, AdrAtomMType, AdrBondMType,&
           NPairsTotalBond, NMIN, SX,SY,SZ,WX,WY,WZ,&
           ! Setreferences-SetAngleBonds                   
           mt_ang_bonds, site_ang_bonds, num_site_ang_bonds, num_mt_ang_bond_types,site_ang_bond_edge,&
! Read RDF               
           FILRDF, StateNB_iSjS, StateB_iBond,&
           IndexGlobNB_riSjS, IndexGlobB_riBond,&
           RMAX,RMIN,RDFref_NB_riSjS,RDFref_B_riBond, &
! Read Pot
           FilPot,PotNB_riSjS, PotB_riBond,NAI, RMX, RMN,&
! Initialize
           NactNB,IND, INDI, CORP,CORS,npairsrdf, RCUT, RDFINC, ID+6,iPrint,rank, ID, psize)


do ! IMC loop -----------------------------------------------------------------------------------------------
    !  Setting zero-th for intramolec. potential
    call ShiftIntraPotToZero(NII+NII_ang,NAI,NMIN,PotB_riBond)
#ifdef mpi
    if (ID.gt.0) call MPI_Bcast(NMIN,size(NMIN),MPI_INTEGER,0,MPI_COMM_WORLD,ierror) ! Sending NMIN from master-process to all other
#endif

    call ResetCounters(ID+6, NAV, IUD, ENERS, EELS, EINTS, VIRS, VIIS, VIRE, PRES, CORS, CORP,npairsrdf)     

!!  Preparing initial state geometry
    call InitGeometry(LCRD, FSTART, FCRD, LCRDPASS, LMoveMType, ID, ID+6, iPrint, iREP, NSTOT,&
    NMType, NMolMType, NSiteMType,SiteAtom, AdrAtomMType, AdrMolMType,AdrAtomMol,NameMType,&
    X, Y, Z, SX,SY,SZ, WX, WY,WZ, RSite,BOXL, BOYL, BOZL)

    call ENERGY(SX,SY,SZ,RCUT,RECUT,Q,COULF,ALPHA,RMN,RMX,EEL,ESF,&
                        EFUR,ENER,DE,EINT,EINT_ang,NSTOT,ITYPE,STypeAtom,NNUM,SiteAtom,Bond_iSjS,LMoveMType,&
                        NKV,NCHRG,ICHRG,RX,RY,RZ,SSIN,SCOS,RKV,&
                        HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,AF,FQ,FKJM,&
                        RMAX,RDFINC,RDINC,PotNB_riSjS,PotB_riBond,&
                        NA,NAI,IPRINT,IND,ID,NII,NII_ang,BTypeBond,&
                        NOP,AdrAtomMol,NSiteMType,MTypeMol,NPTT,num_mt_ang_bond_types,mt_ang_bonds)
    write((ID+6),*)' Init. energy = ',ENER*FKJM,' fourier ',EFUR*FKJM
    
    if(ITR.gt.0) then ! If writing trajectory: Open trajectory output file and write first snapshot
        call OpenTraj(ID,psize,IREP,BaseOutFileName)
        call WriteTraj(psize,0,NSTOT,NameSType,STypeAtom,SX,SY,SZ,BOXL, BOYl, BOZL)
    endif        
!Here we are done with all the initialization  !!!
    
!   Start simulation
    write((ID+6),*) '' 
    write((ID+6),*) '' 
    write((ID+6),*)'==================================================='
    write((ID+6),*)'About to start ',NMKS,' Monte Carlo steps; iteration',IREP
    write((ID+6),*)'==================================================='
    write((ID+6),*)' Total atoms      ',NSTOT
    write((ID+6),*)' Total molecules  ',NOP
    write((ID+6),*)' Interaction sites',NSType
    write((ID+6),*)' Atom sites       ',NDAT
    write((ID+6),*)' Molecule types   ',NMType
    write((ID+6),*)'==================================================='
    if(IPRINT.ge.6)then
        write((ID+6),*)' array Bond_iSjS'
        do I=1,NDAT
            write((ID+6),*)(Bond_iSjS(I,J),J=1,NDAT)
        end do
    end if
    
    do iStep=1, NMKS ! ===================MC simulations Loop=======================================================================       
        
! Monte Carlo              
        
        ! Single atom displacement MC step
        if (IPRINT.gt.7)   write ((ID+6),*) 'IUD:',IUD,'ISTEP:',ISTEP
        call MCStep(Q,SX,SY,SZ,DR,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,ALPHA,RMN,RMX,&
            PotB_riBond,RX,RY,RZ,RKV,SSIN,SCOS,DE,ENER,EEL,EINT,EINT_ang,&
            NSTOT,ITYPE,STypeAtom,NNUM,NA,SiteAtom,Bond_iSjS,NAI,NKV,IUD(1),&
            LMoveMType,ID,NII,NII_ang,BTypeBond, AdrAtomMol,NSiteMType,&
            NOP,MTypeMol,site_ang_bonds,site_ang_bond_edge,num_site_ang_bonds)
        
        ! Molecule translation MC step
        if ((ITRANS.gt.0).and.(mod(ISTEP,ITRANS).eq.0)) then
            call MCTrans(Q,SX,SY,SZ,MCTransStep,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,&
            ALPHA,RX,RY,RZ,RKV,SSIN,SCOS,DE,ENER,EEL,NSTOT,STypeAtom,NNUM,NA,NKV,IUD(2),&
            LMoveMType,AdrAtomMol,MTypeMol,NSiteMType,NOP,IPRINT,ID)
        endif
        
        ! Molecule rotation MC step
        if ((IROT.gt.0).and.mod(ISTEP,IROT).eq.0) then
            call MCROT(MCRotStep,WX,WY,WZ,Q,SX,SY,SZ,X,Y,Z,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,RCUT,&
                    RMAX,RDFINC,PotNB_riSjS,RECUT,COULF,ALPHA,RX,RY,RZ,RKV,SSIN,SCOS,&
                    DE,ENER,EEL,EINT,NSTOT,STypeAtom,NNUM,NA,NKV,IUD(3),LMoveMType,&
                    AdrAtomMol,MTypeMol,NSiteMType,NOP,IPRINT,IND,Bond_iSjS,&
                    SiteAtom,AdrSiteMType,MassSite,SumMassMType,ID)
        endif ! End of rotational MC step       
        
        ! Recalculate COM
        do IMOL=1,NOP
            call GETCOM(IMOL,MTypeMol,SX,SY,SZ,NSiteMType,MassSite,SumMassMType,WX,WY,WZ,HBOXL,HBOYL,HBOZL,&
            BOXL,BOYL,BOZL,AdrAtomMol,AdrSiteMType,X,Y,Z)
        end do
        
    !   Averaging
        LENERGY=.false.
        if((ISTEP.gt.NMKS0).and.(mod(ISTEP,IAV).eq.0)) then
          call Correl(EFUR,RX,RY,RZ,SX,SY,SZ,Q,SSIN,SCOS,RKV,EEL,VIR,VIRI,EINT,EINT_ang,&
                    RMAX,RMIN,RCUT,RDFINC,PotNB_riSjS,RMN,RMX,&
                    PotB_riBond,RDINC,ESF,ENER,CORS,CORP,ENERS,EELS,NKV,NCHRG,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,&
                    RECUT,FQ,COULF,FKJM,ICHRG,NA,STypeAtom,IND,INDI,&
                    IndexGlobNB_riSjS,IndexGlobB_riBond, SiteAtom,Bond_iSjS,NAI,NAV,NSTOT,LMoveMType,LIMC,&
                    npairsrdf,ID,NII,NII_ang,BTypeBond,NOP,AdrAtomMol,NSiteMType,MTypeMol,&
                    num_mt_ang_bond_types,MT_ang_bonds,NPTT,StateNB_iSjS,StateB_iBond)
            LENERGY=.true.
            EINTS=EINTS+EINT
            VIRS=VIRS+VIR; VIIS=VIIS+VIRI; VIRE=VIRE-EEL
            PRES=(NSTOT-(VIR+VIRI-EEL)/3.)*FPR
        end if ! end of Averaging
        
        ! Write coordinates to the trajectory file    
        if(ITR.gt.0.and.mod(ISTEP,ITR).eq.0)  call WriteTraj(psize,ISTEP,NSTOT,NameSType,STypeAtom,SX,SY,SZ,BOXL, BOYl, BOZL)
                
        ! Write MC-simulation status to local output file
        if(mod(ISTEP,IOUT).eq.0)then 
            write((ID+6),'(i9,a4,2f11.3,f10.3,f10.2,a4,3f7.4)') ISTEP,' en:',ENER*FKJM,EEL*FKJM,EINT*FKJM,&
                PRES,' a.r',IUD(1)*1./ISTEP,IUD(2)*1.*Itrans/ISTEP,IUD(3)*1.*Irot/ISTEP
            if(IPRINT.ge.6) write((ID+6),'(4f12.3)')-VIR*FPR/3.,EEL*FPR/3.,-VIRI*FPR/3,NSTOT*FPR               
            if(IPRINT.ge.6) write((ID+6),'(a,4f14.2)')' Pr.comp:',-VIR*FPR,-VIRI*FPR,-EFUR*FPR/3.,NSTOT*FPR                
            ! Recalculate and Check possible deviations in total energy. They can appear from errors in calculation of energy change during MC steps
            if((.not. LENERGY))then
                call ENERGY(SX,SY,SZ,RCUT,RECUT,Q,COULF,ALPHA,RMN,RMX,EEL,ESF,&
                                    EFUR,ENER,DE,EINT,EINT_ang,NSTOT,ITYPE,STypeAtom,NNUM,SiteAtom,Bond_iSjS,LMoveMType,&
                                    NKV,NCHRG,ICHRG,RX,RY,RZ,SSIN,SCOS,RKV,&
                                    HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,AF,FQ,FKJM,&
                                    RMAX,RDFINC,RDINC,PotNB_riSjS,PotB_riBond,&
                                    NA,NAI,IPRINT,IND,ID,NII,NII_ang,BTypeBond,&
                                    NOP,AdrAtomMol,NSiteMType,MTypeMol,NPTT,num_mt_ang_bond_types,mt_ang_bonds)
                LENERGY=.true.                                
            end if
        end if
        
        ! Perform local potential convergence check on every process (master and slaves)
        if (LpotCorrCheck.and.((ISTEP-NMKS0).gt.0).and.(mod(ISTEP-NMKS0,IPotCorrCheck).eq.0)) then
            write((ID+6),*)"Local Potential correction convergence check",(ISTEP+1-NMKS0)/IPotCorrCheck," of ",&
                            (NMKS-NMKS0+1)/IPotCorrCheck
! Local potential correction (and intermediate RDF) on each process                
            call POTCORRMain(CORS,REGP,RTM,DPOTM,CORP,RMIN,RMAX,VOL,FRT,PotB_riBond,PotNB_riSjS,RMN,RMX,&
                   IND,NAI,INDI,NMIN,NSType,NA,NII+NII_ang,BTypeBond,MTypeBond,&
                   MTypeSType,&
                   IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NPTT,IndexGlobB_riBond,ITS,&
                   LIMC,LRDF,BaseOutFileName,NameSite,NameSType,LEV,MixEV,npairsrdf,&
                   NAV,.true.,NMType,IREP,(ISTEP+1-int(NMKS0,4))/IPotCorrCheck,&
                   StateNB_iSjS, StateB_iBond,&
                   ID,iPrint,ID+6)
#ifdef mpi
            if (ID.gt.0) then !If MPI - we gather all the data and perform correction check
                call Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP,&
                npairsrdf,global_npairsrdf,LIMC)
                if (ID.eq.1) then
! General potential correction and RDF
                    write(6,*) "Global Potential correction convergence check",(ISTEP+1-NMKS0)/IPotCorrCheck," of ",&
                                (NMKS-NMKS0+1)/IPotCorrCheck
                    call POTCORRMain(global_CORS,REGP,RTM,DPOTM,global_CORP,RMIN,RMAX,VOL,FRT,PotB_riBond,&
                        PotNB_riSjS,RMN,RMX,&
                        IND,NAI,INDI,NMIN,NSType,NA,NII+NII_ang,BTypeBond,MTypeBond,&
                        MTypeSType,&
                        IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NPTT,&
                        IndexGlobB_riBond,ITS,LIMC,LRDF,BaseOutFileName,NameSite,NameSType,LEV,&
                        MixEV,global_npairsrdf,&
                        global_NAV,.true.,NMType,IREP,(ISTEP+1-int(NMKS0,4))/IPotCorrCheck, &
                        StateNB_iSjS, StateB_iBond,&
                        ID,iPrint,6) !Here I sent ID=0, to write output in general output file
                endif
            endif
#endif
        endif
    enddo !==================== end of MC-simulation loop

!=======================
!   Write Monte Carlo simulation summary
    call PrintMCSummary(ID,NAV,NMType,NOP,int(ISTEP-1,kind=8),&
            NMKS0,NSTOT,ITRANS,IROT,NMolMType,IUD,BOXL,BOYL,BOZL,DR,TEMP,EPS,EELS,FKJM,&
            EINTS,ENERS,PRES,VIRS,VIRE,VIIS,FPR,LRDF,NameMType,FILRDF,FILPOT)       

! Calculating resulting Local Potential Correcion (using data calculated just in this process)
    call POTCORRMain(CORS,REGP,RTM,DPOTM,CORP,RMIN,RMAX,&
            VOL,FRT,PotB_riBond,PotNB_riSjS,RMN,RMX,IND,&
            NAI,INDI,NMIN,NSType,NA,NII+NII_ang,BTypeBond,MTypeBond,&
            MTypeSType,IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NPTT,&
            IndexGlobB_riBond,ITS,LIMC,LRDF, BaseOutFileName,NameSite,NameSType,&
            LEV,MixEV,npairsrdf,NAV,.false.,NMType,IREP,0,&
            StateNB_iSjS, StateB_iBond,&
            ID,iPrint, ID+6)
! If in parallel mode, gather data from all processes
#ifdef mpi
    if (ID.gt.0) then
        call Collect(ID, ierror,rank,psize,NAV,global_NAV,CORS,CORP,global_CORS,global_CORP,&
                    npairsrdf,global_npairsrdf,LIMC)
! Collecting and printing averaged calculation results:
        call CollectMCsummary(NAV,global_NAV,int(iSTEP,kind=8),global_NMKSF,IUD,global_IUD,EELS,&
                    global_EELS,EINTS,global_EINTS,&
                    ENERS,global_ENERS,VIIS,global_VIIS,VIRS,global_VIRS,VIRE,global_VIRE,LRDF,ID,psize,ierror)
        if (ID.eq.1) then
! General resulting potential correction and RDF
            call PrintMCsummary(0,global_NAV,NMType,NOP,global_NMKSF,NMKS0*psize,NSTOT,ITRANS,IROT,NMolMType,global_IUD,&
                        BOXL,BOYL,BOZL,DR,TEMP,EPS,global_EELS,&
                        FKJM,global_EINTS,global_ENERS,global_PRES,global_VIRS,global_VIRE,global_VIIS,FPR,&
                        LRDF,NameMType,FILRDF,FILPOT)
            write(6,*)"Global Final Potential correction"
            call POTCORRMain(global_CORS,REGP,RTM,DPOTM,global_CORP,RMIN,RMAX,VOL,FRT,PotB_riBond,&
                        PotNB_riSjS,RMN,RMX,IND,&
                        NAI,INDI,NMIN,NSType,NA,NII+NII_ang,BTypeBond,MTypeBond,&
                        MTypeSType,&
                        IndexGlobNB_riSjS,iSiteinBond,jSiteinBond,kSiteinBond,NPairsTotalBond,NPTT,&
                        IndexGlobB_riBond,ITS,LIMC,LRDF, BaseOutFileName,NameSite,NameSType,&
                        LEV,MixEV,global_npairsrdf,&
                        global_NAV,.false.,NMType,IREP,0,&
                        StateNB_iSjS, StateB_iBond,&
                        0,iPrint, 6) !Here I sent ID=0, to write output in general output file
        endif
    endif
#endif
    ! Write the potential to a file: 
    call WritePot(BaseOutFileName,NactNB, NII+NII_ANG, NII, NII_ANG, NA, RMax, RMin, NSType, NameSType, NameMType,&
        StateNB_iSjS, PotNB_riSjS, &
        MTypeBond, BTypeBond, NBondMType, AdrSiteMType, &
        NAI, RMX, RMN, StateB_iBond, PotB_riBond,&! rPotB_riBond, &
        NPTT, iSiteinBond, jSiteinBond, kSiteinBond, FRT, irep)
    
    ! Write the final configuration of current IMC iteration to file
    if (LXMOL) call SaveLastConf(ID,NSTOT,IREP,BaseOutFileName,NameSType,STypeAtom,SX,SY,SZ,BOXL, BOYl, BOZL)     
    ! Close the trajectory file for writing
    if (ITR.gt.0) close(19+psize)
    if(IREP.ge.IREPT) exit ! If no more inverse iterations to perform then exit.
    IREP=IREP+1
#ifdef mpi
    ! Passing new potential to all processes
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)        
    call MPI_Bcast(PotNB_riSjS,size(PotNB_riSjS),MPI_REAL8,0,MPI_COMM_WORLD,ierror) !Sending POT        
    call MPI_BARRIER(MPI_COMM_WORLD,ierror)
    call MPI_Bcast(PotB_riBond,size(PotB_riBond),MPI_REAL8,0,MPI_COMM_WORLD,ierror) !Sending PotB_riBond
#endif
enddo  ! end of IMC loop

! Deallocate arrays, close opened files, finalize MPI.
call TerminateMagiC(PotCorrJourn,SumMassMType,PotNB_riSjS,PotB_riBond,&
        X,Y,Z,RSite,SX,SY,SZ, WX,WY,WZ, MassSite, RMN,RMX,&
        NSiteMType,AdrBondMType,AdrSiteMType,AdrMolMType,AdrAtomMType,AdrAtomMol,&
        NBondMType,Bond_iSjS,NPairsTotalBond,NPTT, IndexGlobNB_riSjS,IndexGlobB_riBond,IND,INDI,MTypeSType, MTypeMol, ITS,&
        iSiteinBond,jSiteinBond, kSiteinBond,NAI,NMIN, ITYPE, NNUM,STypeAtom, SiteAtom, ICHRG,&
        CORS,CORP,npairsrdf,global_npairsrdf,global_CORP, global_CORS,& 
        NameSType, NameSite,ID)
end
