!   This file contains subroutines responsible for output of MOLSIM
!       OpenTraj - Open/Create a file to write trajectory and connect to it. The file has name <BaseOutName>.i<iteration>.p<process>.traj.xmol
!       WriteTraj - Writes system's configuration to the trajectory file.
!       SaveLastConf - Writes resulting configuration of the system to a file. The file has name <BaseOutName>.i<iteration>.p<process>.start.xmol
!DEC$ FREEFORM

subroutine OpenTraj(ID,psize,IREP,BaseOutName)
    !Global variables
    integer(4) ID, psize,IREP
    character*64 BaseOutName 
    !Local variables
    character*64 SaveFileName 
    
    SaveFileName=BaseOutName    
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.i'
    write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') IREP
    if (ID.ne.0) then
        SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.p'
        write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') ID
    endif
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+9)='.trj.xmol'
    open(unit=(19+psize),file=SaveFileName)    
end subroutine

subroutine WriteTraj(psize,ISTEP,NSTOT,NameSType,STypeAtom,SX,SY,SZ, BOXL, BOYl, BOZL)
    !Global variables
    integer(4)  ISTEP,NSTOT,psize
    integer(4), allocatable:: STypeAtom(:)
    real(8) BOXL, BOYl, BOZL
    real(8), allocatable:: SX(:),SY(:),SZ(:)
    character*4, allocatable::NameSType(:)
    ! Local variables
    integer(4) I
!    1.3  Dumping a configuration:
!  First Line - number of atoms
    write((19+psize),*) NSTOT
!  Second commentary line of XMOL format contains time in fs or current MC-step
    write((19+psize),*)' after ',ISTEP,' MC steps'        ! for MC
    do I=1,NSTOT
        write((19+psize),'(a,3f14.5)')NameSType(STypeAtom(I)),SX(I),SY(I),SZ(I)
    end do   
end subroutine

subroutine SaveLastConf(ID,NSTOT,IREP,OutFileName,NameSType,STypeAtom,SX,SY,SZ,BOXL, BOYL, BOZL)
! Global Variables
    integer(4) IREP,STypeAtom(:),NSTOT,ID
    real(8) SX(:),SY(:),SZ(:)
    real(8) BOXL, BOYl, BOZL
    character(4) NameSType(:)
    character*64 OutFileName
! Local Variables
    character*64 SaveFileName ! Local variable to work with, without changing global variable OutFileName
    integer(4) i
    SaveFileName=OutFileName
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.i'
    write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') IREP
    if (ID.ne.0) then
        SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+2)='.p'
        write(SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+3),'(i0.3)') ID
    endif
    SaveFileName(Len_Trim(SaveFileName)+1:Len_Trim(SaveFileName)+11)='.start.xmol'
    open(unit=2,file=SaveFileName,status='unknown')
    write(2,*)NSTOT
    write(2,'(a,3f10.4)')' start configuration BOX: ', BOXL, BOYL ,BOZL
    do I=1,NSTOT     ! all molecules
        write(2,'(a,3f14.5)')NameSType(STypeAtom(I)),SX(I),SY(I),SZ(I)
    end do
    close(2)
    write((ID+6),*)'Configuration snapshot file written ',SaveFilename
end subroutine