!DEC$ FREEFORM
!  Set-up subroutines to start molsim program
!
!*     READMOL - read molecular information (.mcm files)
!*     READCRD  - read initial coordinates and eventually velocities from a "xmol" file s
!*     INIVEL  - initialize velocities
!*     SETFORCE
!
!================ READMOL =============================================
!    Read molecular data
!    ----------------------
!    This subroutine read the complete information about the molecule 
!    of type NMType from the .mol file located in molecular database. 
!    Only in a few cases these data may be overwritten by the data
!    from the input file (see subroutine INPUT)
!
subroutine READMOL(ITYP,ISN,IE,IPRINT,NATOM,NATOMS,NSITES,ISITT,IOK,NRB,IRDF,NPSIT,&
                    iSiteinBond,jSiteinBond,kSiteinBond,NPTT,IB,JB,KB,&
                R,MASS,SUMMAS,CH,CHS,PATHDB,NameSite,NAMOL,NX,IBB,ID)
    use dynarray
    use utilsmod
    implicit none
    ! Global variables
    integer(4) ITYP,ISN,IE,IPRINT,IRDF,NX,IBB,ID
    integer(4), allocatable::ISITT(:),NATOMS(:),IOK(:),NPSIT(:,:),NPTT(:),iSiteinBond(:,:),jSiteinBond(:,:),kSiteinBond(:,:),&
    IB(:),JB(:),KB(:),&
                    NATOM(:),NSITES(:),NRB(:)
    real(8), allocatable::r(:,:),CH(:),CHS(:),MASS(:),SUMMAS(:)
    character*64 PATHDB,NAMOL(:)
    character*4, allocatable:: NameSite(:)            
    !Local Variables
    integer(4) :: LD,LN,NAT,NSBEG,NSEND,IX,I,J,K,IS,JS,ISG,II,IAT,JAT,KAT,iostt,flag
    character namn*64,STR*128,FIL*128,NAMH*4,NMF*5!,TAKESTR*128
    integer(4), allocatable:: ISTYPE(:),temparray(:)
    integer(4) NBONDS, NPOTS
    real(8), allocatable:: CHA(:)
    real(8) CM(3),RR2,RR
    
    CM(:)=0.d0
!
!  1 Open the .mcm file
!  ----------------------
    FIL=' '
!  molecule name                    
    NAMN=NAMOL(ITYP)
    IE=0
    FIL=(trim(PATHDB)//'/'//trim(NAMN)//'.mcm')		
    open(unit=4,file=fil,status='old',iostat=iostt)
    if (iostt .ne. 0) then
        write((ID+6),*)' .mol file ',fil,' not found '
	stop
    endif
    if(IPRINT.ge.5) write(ID+6,*) '*** MOLECULE ',NAMN,',  Type  No.',ITYP
!
!  2 Read information about the atoms
!  ------------------------------------
   IE=0
   STR=TAKESTR(4,IE)
!  Number of atoms
   read(STR,*,iostat=iostt) NAT
    if (iostt .ne. 0) then
        write(6+ID,*)' input error in .mol file ',fil
	stop
    endif
    if(IPRINT.ge.6)then
        write(6+ID,*)'*** NR OF ATOMS:                ',NAT
    end if
    call reshape(ISTYPE,NAT)
    call reshape(CHA,NAT)
    call reshape(NameSite,NX+NAT)
    call reshape2(R,NX+NAT,3)
    call reshape(CH,NX+NAT)
    call reshape(CHS,NX+NAT)
    call reshape(MASS,NX+NAT)
    call reshape(IOK,NX+NAT)
    call reshape2(NPSIT,NX+NAT,NX+NAT)      	

    NATOM(ITYP)=NAT     
!   Cycle over atoms
    do I=1,NAT
        STR=TAKESTR(4,IE)
        IX=NX+I
        NameSite(IX)=STR(1:4)
        read(STR(5:80),*,iostat=iostt)(R(IX,J),J=1,3),   &
        MASS(IX),CHA(I),ISTYPE(I)
        if (iostt .ne. 0) then
           write(6+ID,*)' input error in .mol file ',fil
           stop
        endif
        if(IPRINT.ge.6) write(6+ID,'(a5,a4,a4,3f7.3,a3,f8.4,a3,f6.3,a8,i2)') &
            'atom ',NameSite(IX),'  R=',(R(IX,J),J=1,3), ' M=',MASS(IX),' Q=',CHA(I),' Atype=',ISTYPE(I)
        if(ISTYPE(I).le.0.or.ISTYPE(I).gt.(NX+NAT))then
           write(6+ID,*)' Wrong site type ',ISTYPE(I),' file ',fil
           write(6+ID,*)' Atom            ',I
           stop
        end if
    end do
!
!  3 Make local coordinate system COM at zero
!  -------------------------------------------- 
    NSBEG=NX+1
    NSEND=NX+NAT    
!  This is total mass of the molecule in atomic units
    SUMMAS(ITYP) = SUM(MASS(NX+1:NX+NAT))
    write(6+ID,*)' Total mass av type ',ITYP,SUMMAS(ITYP)
    if(IPRINT.ge.6)write(6+ID,*)' Setting local COM to 0.  Type  ',NAMOL(ITYP)
    do K       = 1,3
        do IS      = NSBEG,NSEND
            R(IS,K)    = R(IS,K)-CM(K)
        enddo! OF IS
    enddo! OF K
!  Print extra information
    if(IPRINT.ge.8)then
        write(6+ID,*)'*** Molecular GEOMETRY (C.O.M. IN ORIGIN): '
        do IS      = NSBEG,NSEND
            write(6+ID,'(I4,3x,A4,3X,3(1X,F7.3))') IS,NameSite(IS),R(IS,1),R(IS,2),R(IS,3)
        enddo! OF IS
    end if
    if(IPRINT.ge.10)then
        write(6+ID,*)
        write(6+ID,*)'*** INTER ATOMIC DISTANCES '
        do IS=NSBEG,NSEND
            do JS=IS+1,NSEND
                RR2=0.
	    	do K=1,3
                    RR2=RR2+(R(IS,K)-R(JS,K))**2
	    	end do
 	    	RR=sqrt(RR2)
                write(6+ID,'(3a4,2x,f9.4)')NameSite(IS),' -> ',NameSite(JS),RR
            end do
	end do
    end if
! Checking atomic types present in current molecular type (ISG). They should be enumerated continiously.
    IS=maxval(ISTYPE(1:NAT),1) ! Max number of the atomic type present in the mcm file.
   do i=1+ISN,IS ! Loop over all atom types present in the molecule. Check enumeration.
  	flag=0
 	do j=1,NAT ! Over all sites (atoms) present in the molecule
            if (i.eq.ISTYPE(j)) then ! Atom of type i is present in the molecule
                flag=1
 	  	exit
            endif
 	enddo
 	if (flag.ne.1) then !  No atoms of type i present in the mcm file - stop with error
            write(6+ID,*) "No atoms of atomic type ",i,"present in the mcm file ",fil
            stop
 	endif
    enddo
    ISG=IS
!  Reshaping array NATOMS and ISITT (it has initial size = 1)
    call reshape(NATOMS,ISG)
    call reshape(ISITT,ISG)
	        
    do i=1, NAT
        ISITT(ISTYPE(I))=ITYP
        NATOMS(ISTYPE(I))=NATOMS(ISTYPE(I))+1
        IOK(I+NX)=ISTYPE(I)
        CH(ISTYPE(I))=CHA(I)
        CHS(I+NX)=CHA(I)
    enddo
!  Intramolecular information ("bonds")
    STR=TAKESTR(4,IE)
    NRB(ITYP)=0
!  Number of different "bonds"
    read(STR,*,iostat=iostt)NBONDS
    if (iostt .ne. 0) then
        write(6+ID,*)' input error in .mol file ',fil
	stop
    endif
    call reshape(NPTT,IRDF+NBONDS)
    call reshape2(iSiteinBond,IRDF+NBONDS,NX+NAT)
    call reshape2(jSiteinBond,IRDF+NBONDS,NX+NAT)   	
    
    do II=1,NBONDS ! Over all bonds in mcm file
        STR=TAKESTR(4,IE)
	read(STR,*,iostat=iostt)NPOTS ! Number of pairs in the bond
	if (iostt .ne. 0) then
            write(6+ID,*)' input error in .mol file ',fil
            stop
	endif
	call reshape(IB,IBB+NPOTS)
	call reshape(JB,IBB+NPOTS)
        call reshape(KB,IBB+NPOTS)
	IRDF=IRDF+1
        do I=1,NPOTS ! Over all atom pair in the bond
            STR=TAKESTR(4,IE)
            read(STR,*)IAT,JAT
            IAT=IAT+NX
            JAT=JAT+NX
            iSiteinBond(IRDF,I)=IAT
            jSiteinBond(IRDF,I)=JAT
            NPTT(IRDF)=NPOTS
            NRB(ITYP)=NRB(ITYP)+1
            IBB=IBB+1
            IB(IBB)=IAT
            JB(IBB)=JAT
        end do
    end do

! Reading angle-bonds
!  Intramolecular information ("bending angles")
    STR=TAKESTR(4,IE)
!  Number of different "angles"
    read(STR,*,iostat=iostt)NBONDS  !angle-bonds now
    if (iostt .ne. 0) then
        write(6+ID,*)' Angle-bond section in absent in file ',fil	
    else ! There are angle bonds
        call reshape(NPTT,IRDF+NBONDS)
        call reshape2(iSiteinBond,IRDF+NBONDS,NX+NAT)
        call reshape2(jSiteinBond,IRDF+NBONDS,NX+NAT)
        call reshape2(kSiteinBond,IRDF+NBONDS,NX+NAT)
        do II=1,NBONDS ! Over all angle-bonds in mcm file
            STR=TAKESTR(4,IE)
            read(STR,*,iostat=iostt)NPOTS ! Number of triplets in the anglebond
            if (iostt .ne. 0) then
                write(6+ID,*)' input error in .mol file ',fil
                stop
            endif
            call reshape(IB,IBB+NPOTS)
            call reshape(JB,IBB+NPOTS)
            call reshape(KB,IBB+NPOTS)
            IRDF=IRDF+1
            do I=1,NPOTS ! Over all atom triplets in the bond
                STR=TAKESTR(4,IE)
                read(STR,*) IAT,JAT,KAT !Local site numbers
                IAT=IAT+NX !Global site number
                JAT=JAT+NX
                KAT=KAT+NX
                iSiteinBond(IRDF,I)=IAT
                jSiteinBond(IRDF,I)=JAT
                kSiteinBond(IRDF,I)=KAT
                NPTT(IRDF)=NPOTS
                NRB(ITYP)=NRB(ITYP)+1
                IBB=IBB+1
                IB(IBB)=IAT
                JB(IBB)=JAT
                KB(IBB)=KAT
            end do
        end do
    endif
    NSITES(ITYP)=ISG-ISN                      !  num of atomic types of this moltype!
    ISN=ISG
    NATOM(ITYP)=NAT                      !  num. of sites/atoms in this molecular type
    NX=NX+NAT                            !  shift number of sites for next molecule
    
    deallocate(istype,cha)
    return
end



!
!=============== INIVEL ========================================
!
!  Initialize velocities
SUBROUTINE INIVEL(VX,VY,VZ,MASS,EKIN,TFACT,TEMP,TEMPC,SCT,NSTOT,NSITE,ID)
    use utilsmod
    implicit none
    real(8) EKIN,TFACT,TEMP,TEMPC,SCT
    real(8),allocatable:: VX(:),VY(:),VZ(:),MASS(:)
    integer(4) NSTOT,ID
    integer(4),allocatable::NSITE(:)
    real(8) SUMX,SUMY,SUMZ,TOTMAS,TEMPE,TFK
    integer(4) N,IS

    DO N    = 1,NSTOT
        IS = NSITE(N)
        VX(N)   = (rand()-0.5)/MASS(IS)
        VY(N)   = (rand()-0.5)/MASS(IS)
        VZ(N)   = (rand()-0.5)/MASS(IS)
    END DO! OF N

    SUMX    = 0.D0
    SUMY    = 0.D0
    SUMZ    = 0.D0
    EKIN = 0.d0
    TOTMAS=0.
    DO N    = 1,NSTOT
        IS = NSITE(N)
        SUMX    = SUMX+VX(N)*MASS(IS)
        SUMY    = SUMY+VY(N)*MASS(IS)
        SUMZ    = SUMZ+VZ(N)*MASS(IS)
        TOTMAS=TOTMAS+MASS(IS)
    END DO! OF N
    SUMX    = SUMX/TOTMAS
    SUMY    = SUMY/TOTMAS
    SUMZ    = SUMZ/TOTMAS

    DO N    = 1,NSTOT
        IS = NSITE(N)
        VX(N)   = VX(N)-SUMX
        VY(N)   = VY(N)-SUMY
        VZ(N)   = VZ(N)-SUMZ
        EKIN = EKIN + 0.5*(VX(N)**2+VY(N)**2+VZ(N)**2)*MASS(IS)
    END DO! OF N
    TEMPE = TFACT*EKIN
    TFK = sqrt(TEMP/TEMPE)
    write((ID+6),*)' T=',TEMPE,' scaling by ',TFK
    EKIN = 0.d0
    do N=1,NSTOT
        IS = NSITE(N)
        VX(N) = VX(N)*TFK
        VY(N) = VY(N)*TFK
        VZ(N) = VZ(N)*TFK
        EKIN = EKIN + 0.5*(VX(N)**2+VY(N)**2+VZ(N)**2)*MASS(IS)
    end do
    TEMPC = TFACT*EKIN
    write((ID+6),'(a,f12.5)')' Velocities initialized. Temperature ',TEMPC

    SCT = 0.
    RETURN
END
!================ SETFORCE ===========================================

subroutine SETFORCE(NSType,ISITT,NA,NII,NAI,FORC,POT,RDFINC,FCUT,RMN,RMX,FORI,POTI,LMOVE)
    ! Global Variables
    implicit none
    integer(4) NSType,NA,NII
    integer(4),allocatable::ISITT(:),NAI(:)
    real(8),allocatable:: FORC(:,:,:),POT(:,:,:),RMN(:),RMX(:),FORI(:,:),POTI(:,:)
    real(8) RDFINC,FCUT
    logical LMOVE(:)
    !Local Variables
    integer(4) IS,JS,ITYP,JTYP,I,IPP,NST,NAM
    real(8) R1,R2,RDINC
    do IS=1,NSType
        ITYP=ISITT(IS)
        do JS=1,NSType
            JTYP=ISITT(JS)
            if(LMOVE(ITYP).or.LMOVE(JTYP))then
                do I=1,NA-1
                    FORC(I,IS,JS)=(POT(I+1,IS,JS)-POT(I,IS,JS))/RDFINC
                    if(FORC(I,IS,JS).gt.FCUT)FORC(I,IS,JS)=FCUT
                    if(FORC(I,IS,JS).lt.-FCUT)FORC(I,IS,JS)=-FCUT
                end do
                FORC(NA,IS,JS)=0.
            end if
        end do
    end do
    do IPP=1,NII
        NST=NAI(IPP)
        R1=RMN(IPP)
        R2=RMX(IPP)
        NAM=NAI(IPP)
        RDINC=(R2-R1)/NAM
        do I=1,NST-1
            FORI(I,IPP)=(POTI(I+1,IPP)-POTI(I,IPP))/RDINC
        end do
        FORI(NST,IPP)=FCUT
   end do
end
